/*
  cmd/live/info.c

  General commands for giving and getting game information

*/
#pragma save_binary

inherit "/cmd/std/command_driver";
inherit "/lib/time";

#include <macros.h>
#include <std.h>
#include <log.h>
#include <stdproperties.h>

void 
create()
{
    seteuid(getuid(this_object())); 
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return "info";
}

/* **************************************************************************
 * This is a command soul.
 */
int
query_cmd_soul()
{
    return 1;
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([

	     "bug":"bug",

	     "date":"date",
	     "done":"done",

	     "help":"help",

	     "idea":"idea",

	     "praise":"praise",

	     "sysbug":"sysbug",

	     "typo":"typo",

	     ]);
}

/*
 * Function name: using_soul
 * Description:   Called once by the living object using this soul. Adds
 *		  sublocations responsible for extra descriptions of the
 *		  living object.
 */
public void 
using_soul(object live)
{
}

/* **************************************************************************
 * Here follow some support functions. 
 * **************************************************************************/

/*
 * Function name:   note_edit
 * Description:     Allow players to give multiple lines of comments
 * Arguments:	    The text the player entered.
 * Returns:	    1
 */
int
note_edit(string str)
{
    object tp;
    int quit_note;

    tp = this_player();
    quit_note = 0;

    if (str == "~q")
    {
	write(LOG_ABORT_MSG(tp->query_prop("_player_log_msg")));
	quit_note = 1;
    }

    if (str == "~l")
    {
	write(tp->query_prop("_player_log_text"));
	input_to("note_edit");
	return 1;
    }

    if (str == "**")
    {
	SECURITY->note_something(tp->query_prop("_player_log_text"),
				 tp->query_prop("_player_log_file"),
				 tp->query_prop("_player_log_id"),
				 tp->query_prop("_player_log_env"));
	write(LOG_THANK_MSG(tp->query_prop("_player_log_msg")));
	quit_note = 1;
    }

    if (quit_note)
    {
	tp->remove_prop("_player_log_text");
	tp->remove_prop("_player_log_file");
	tp->remove_prop("_player_log_type");
	tp->remove_prop("_player_log_env");
	tp->remove_prop("_player_log_id");
    }
    else
    {
	if (tp->query_prop("_player_log_text"))
	    tp->add_prop("_player_log_text",
			 tp->query_prop("_player_log_text") + str + "\n");
	else
	    tp->add_prop("_player_log_text", str + "\n");

	write("]");
	input_to("note_edit");
    }	
    return 1;
}

/* **************************************************************************
 * Now the actual commands
 * **************************************************************************/

/*
 * bug - Give a bug report
 */
varargs int
bug(string str)
{
    object tp;

    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_BUG_FILE);
	tp->add_prop("_player_log_msg", LOG_BUG_MSG);
	tp->add_prop("_player_log_id", LOG_BUG_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_BUG_FILE, LOG_BUG_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_BUG_MSG));
    return 1;
}

/* **************************************************************************
 * date - get local time & date + uptime information
 */
int
date()
{
    int mlim, d, cval, h, m, s;
    string mc, bef, aft;

    mlim = SECURITY->query_memory_limit();

    mc = SECURITY->do_debug("malloc");
    sscanf(mc, "%ssbrk requests: %d %d (a) %s", bef, d, cval, aft);

    write("Local time:     " + ctime(time()) + "\n");
    write("Start time:     " + ctime(SECURITY->query_start_time()) + "\n");
    write("Up time:        " + convtime(time() -
					SECURITY->query_start_time()) +	"\n");
    write("Memory usage:   " + ((100 * (cval / 10)) / (mlim / 10)) + "%\n");
#ifdef REGULAR_REBOOT
    write("Regular reboot: " + "Every day after " + REGULAR_REBOOT + ":00\n");
#endif
    return 1;
}

/*
 * done - Report something as done (wizards only)
 */
varargs int
done(string str)
{
    object tp;

    if (WIZ_CHECK < WIZ_NORMAL)
	return 0;
    
    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_DONE_FILE);
	tp->add_prop("_player_log_msg", LOG_DONE_MSG);
	tp->add_prop("_player_log_id", LOG_DONE_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_DONE_FILE, LOG_DONE_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_DONE_MSG));
    return 1;
}
/*
 * help - Get help on a subject
 */
varargs int
help(string what)
{
    int mlen;
    string dir, subject;

    if (what)
    {
	if (sscanf(what, "%s %s", dir, subject) == 2)
	{
	    switch (dir)
	    {
		case "w":
		case "g":
		    what = subject;
		    break;
		default:
		    break;
	    }
	}
	if (dir == "w" || file_size("/doc/help/general/" + what) < 0)
	{
	    if (WIZ_CHECK >= WIZ_APPRENTICE && this_interactive() ==
			this_player())
	    {
		if (file_size("/doc/help/wizard/" + what) < 0)
		{
		    notify_fail("No help on '" + what + "' available.\n");
		    return 0;
		}
	        else
		{
		    this_interactive()->more("/doc/help/wizard/" + what, 1,
			this_player()->query_prop(PLAYER_I_MORE_LEN));
		    return 1;
		}
	    }
	    else
	    {
		notify_fail("No help on " + what + " available.\n");
		return 0;
	    }
	}
	else
	{
	    this_interactive()->more("/doc/help/general/" + what, 1,
		this_player()->query_prop(PLAYER_I_MORE_LEN));
	    return 1;
	}
    }
    else if (file_size("/doc/help/general/help") > 0)
    {
	this_interactive()->more("/doc/help/general/help", 1,
				 this_player()->query_prop(PLAYER_I_MORE_LEN));
	return 1;
    }
    else
    {
	notify_fail("No help available, sorry!\n");
	return 0;
    }
}

/*
 * idea - Give an idea 
 */
varargs int
idea(string str)
{
    object tp;

    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_IDEA_FILE);
	tp->add_prop("_player_log_msg", LOG_IDEA_MSG);
	tp->add_prop("_player_log_id", LOG_IDEA_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_IDEA_FILE, LOG_IDEA_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_IDEA_MSG));
    return 1;
}

/*
 * praise - Give praise for something
 */
varargs int
praise(string str)
{
    object tp;

    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_PRAISE_FILE);
	tp->add_prop("_player_log_msg", LOG_PRAISE_MSG);
	tp->add_prop("_player_log_id", LOG_PRAISE_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_PRAISE_FILE, LOG_PRAISE_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_PRAISE_MSG));
    return 1;
}

/*
 * sysbug - Give a bug report about the system in general
 */
varargs int
sysbug(string str)
{
    object tp;

    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_SYSBUG_FILE);
	tp->add_prop("_player_log_msg", LOG_SYSBUG_MSG);
	tp->add_prop("_player_log_id", LOG_SYSBUG_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_SYSBUG_FILE, LOG_SYSBUG_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_SYSBUG_MSG));
    return 1;
}

/*
 * typo - Give a typo report
 */
varargs int
typo(string str)
{
    object tp;

    if (!str)
    {
	tp = this_player();
	tp->add_prop("_player_log_text", "");
	tp->add_prop("_player_log_file", LOG_TYPO_FILE);
	tp->add_prop("_player_log_msg", LOG_TYPO_MSG);
	tp->add_prop("_player_log_id", LOG_TYPO_ID);
	tp->add_prop("_player_log_env", environment(tp));
	write("Give text (** to end, ~q to abort, ~l to redraw).\n]");
	input_to("note_edit");
	return 1;
    }
    SECURITY->note_something(str + "\n", LOG_TYPO_FILE, LOG_TYPO_ID,
			     environment(this_player()));
    write(LOG_THANK_MSG(LOG_TYPO_MSG));
    return 1;
}
