/*
  cmd/live/social.c

  General commands for 'nonemotive' social behaviour

*/
#pragma save_binary

inherit "/cmd/std/command_driver";

#include <macros.h>
#include <std.h>
#include <stdproperties.h>
#include <filter_funs.h>
#include <composite.h>
#include <cmdparse.h>
#include <language.h>
#include <formulas.h>

#define SHORT (this_player()->short())
#define ENV (environment(this_player()))

create()
{
    seteuid(getuid(this_object())); 
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return "social";
}

/* **************************************************************************
 * This is a command soul.
 */
int
query_cmd_soul()
{
    return 1;
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "aggressive":"aggressive",
	     "ask":"ask",

	     "commune":"commune",

	     "emote":"emote",

	     "forget":"forget_live",

	     "introduce":"intro_live",
	     "introduced":"introduced_list",
	     "invite":"invite",

	     "join":"join",

	     "kill":"kill",

	     "leave":"leave",

	     "present":"intro_live",
	     "players":"who_known",

	     "reply":"reply",
	     "remember":"remember_live",
	     "remembered":"remember_live",

	     "see":"see",
	     "stop":"stop",

	     "team":"team",

	     "who":"who_known"
	     ]);
}

/*
 * Function name: using_soul
 * Description:   Called once by the living object using this soul. Adds
 *		  sublocations responsible for extra descriptions of the
 *		  living object.
 */
public void 
using_soul(object live)
{
    /*
	live->add_subloc(SUBLOC_MISCEXTRADESC, file_name(this_object()));
        live->add_textgiver(file_name(this_object()));
    */
}

/* **************************************************************************
 * Here follows some support functions. 
 * **************************************************************************/

/* Do we need this function?  /Quis */
nomask varargs int
light_fail(string str)
{
    if (!str)
	str = query_verb() + " things";
    notify_fail("It is too dark to " + str + ".\n");
    return 0;
}


int 
kill_access(object ob)
{
    if (!living(ob) || ob->query_ghost() || ob == this_player())
	return 0;
    else
	return 1;
}

int
who_known()
{
    object *list, *known;
    mapping rem;
    string *names;
    int t;

#ifdef MET_ACTIVE
    if (this_player()->query_wiz_level())
	return this_object()->who();
#else
    return this_object()->who();
#endif
    
    list = users();
    known = ({ });
    rem = this_player()->query_remembered();
    if (mappingp(rem)) names = m_indexes(rem);
    rem = this_player()->query_introduced();
    if (mappingp(rem))
    {
	if (!names)
	    names = m_indexes(rem);
	else
	    names += m_indexes(rem);
    }

    known = map(list, "find_known", this_object(), names);
    known -= ({ 0 });

#if 0
    for (t = 0; t < sizeof(list); t++)
    {
	if (member_array(list[t]->query_real_name(), names) >= 0 ||
	    list[t]->query_prop(LIVE_I_ALWAYSKNOWN) ||
	    list[t] == this_player())
	    known += ({ list[t] });
    }
#endif
    
    if (sizeof(users()) > 1) 
	write("There are " + sizeof(users()) +
	      " players in the game, of these you know:\n");
    else
	write("You are alone in the game:\n");
    
    if (query_verb() == "who")
	return this_object()->who(known);
    else
	return this_object()->who_format(known);
}

nomask object
find_known(object ob, string *names)
{
    if (member_array(ob->query_real_name(), names) >= 0 ||
	ob->query_prop(LIVE_I_ALWAYSKNOWN) ||
	ob == this_player())
	return ob;
    return 0;
}

nomask int
intro_one_live(object to_who, string intro_who)
{
    string str, added;
    object ob_intro;
    
    if (!to_who || to_who == this_player())
	return 0;

    if (intro_who == "myself" || intro_who == "me" || 
	capitalize(intro_who) == NAME)
    {
	ob_intro = this_player();
	added = this_player()->query_objective() + "self as:";
    }
    else
    {
	ob_intro = present(intro_who, environment(this_player()));
	if (!ob_intro) 
	    return 0;
	if (!this_player()->query_met(ob_intro->query_real_name()))
	    return 0;
	if (!CAN_SEE(this_player(), ob_intro))
	    return 0;
	if (to_who->query_met(intro_who))
	    if(ob_intro == to_who)
		added = "you to the others as:";
	    else
		added = ob_intro->query_name() + " as:";
	else
	    added = "the " + ob_intro->query_nonmet_name() + " as:";
    }
    if (!living(to_who))
	return 1; /* introduce name to apple :-) */

    ob_intro->reveal_me(1);
    to_who->reveal_me(1);
    this_player()->reveal_me(1);
    to_who->catch_msg(QCTNAME(this_player()) +	" introduces " + added + "\n" +
		      ob_intro->query_presentation() + ".\n");
    
    if ((query_ip_number(to_who)) && (living(ob_intro)))
	to_who->add_introduced(ob_intro->query_real_name());

    return 1;
}
	
nomask string
capn(string n)
{ 
    if (stringp(n)) 
	return capitalize(n);
    else
	return n;
}

/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/

/*
 * Aggressive - Set how aggressive we are
 *
 * This command is used to state when to fight and when to flee
 *
 * ** not ready yet, needs fixes in the combat system **
 */
aggressive(str)
{
    write("Yes you are, very!\n");
    return 1;
}

/*
 * Ask - Ask someone something
 *
 */
ask(str)
{
    object *oblist, *tmplist;
    string msg, players;
    int i;

    if (!CAN_SEE_IN_ROOM(this_player()))
    {
	notify_fail("You can't see in here.\n");
	return 0;
    }

    tmplist = ({});
    notify_fail("Ask what?\n");

    if (!str)
	return 0;

    if (parse_command(str, environment(this_player()), "%l %s", oblist, msg))
	oblist = NORMAL_ACCESS(oblist, 0, 0);

    if (!sizeof(oblist))
    {
	notify_fail("Can't find that person.\n");
	return 0;
    } 
    else if (oblist[0] == this_player())
    {
        notify_fail("Don't ask me, I wouldn't know.\n");
	return 0;
    }
    else
    {
	for (i = 0; i < sizeof(oblist); i++)
	{
	    if (objectp(oblist[i]) && living(oblist[i]) &&
			this_player() != oblist[i])
		tmplist += ({ oblist[i] });
	}

	if (!strlen(msg))
	{
	    write("Ask what to " + COMPOSITE_LIVE(oblist) + "?\n");
	    return 1;
	}

	this_player()->reveal_me(1);

	players = COMPOSITE_LIVE(oblist);
	if (this_player()->query_get_echo())
	    write(break_string("You ask " + players + ": " + msg + "\n", 70));
	else
	    write("Ok.\n");

	say(QCTNAME(this_player()) + " asks " + QCOMPLIVE + " something.\n",
			oblist + ({ this_player() }) );

        for (i = 0; i < sizeof(oblist); i++)
        {
            oblist[i]->catch_msg(QCTNAME(this_player()) + " asks you: " + msg +
                        "\n");
            oblist[i]->catch_question(msg);
	    oblist[i]->reveal_me(1);
        }
    }

    return 1;
}

/*
 * Commune - talk with the wizards
 *
 * This is supposed to be used in extreme emergencies only
 */
commune(str)
{
    object *us, spec;
    int il, flag, busy;
    string *arg, wiz, mess;

    if (!strlen(str))
    {
	write("Please do 'help commune' to see how this rite is performed.\n");
	write("But Beware! Mortals will be stricken by the ultimate wrath of\n");
	write("the deities supreme if communing for insufficient reasons.\n");
	return 1;
    }

    if (!strlen(query_ip_number(this_player())))
    {
	notify_fail("Only true players may commune with the deities.\n");
	return 0;
    }

    if (this_player() != this_interactive())
    {
	tell_object(this_interactive(),
	    "Communing is a decision the player must make alone.\n");
	return 0;
    }

    if (this_player()->query_wiz_level())
    {
	notify_fail("Communing is something relevant only to mortals, " +
	    "seek an audience.\n");
	return 0;
    }

    arg = explode(str, " ");

    if (sizeof(arg) < 2)
    {
	notify_fail("Please do 'help commune' to see how this rite " +
	    "is performed.\n");
	return 0;
    }
    
    wiz = arg[0];
    arg[0] = "";
    mess = implode(arg, " ");

    SECURITY->commune_log(str);

    switch(wiz)
    {
    case "all":
	us = users();
    
	for (flag = 0, il = 0; il < sizeof(us); il++)
	{
	    busy = us[il]->query_prop(WIZARD_I_BUSY_LEVEL);
	    if (us[il]->query_wiz_level() && !(busy & 32) &&
		query_ip_number(us[il]))
	    {
		tell_object(us[il],"COMMUNE anyone from " + 
			    capitalize(this_player()->query_real_name()) + ":" +
			    mess + "\n");
		flag++;
	    }
	}
	break;
	
    default:
	spec = find_player(wiz);

	if (!spec || 
	    !spec->query_wiz_level() ||
	    spec->query_prop(OBJ_I_INVIS) > 0)
	    break;

	busy = spec->query_prop(WIZARD_I_BUSY_LEVEL);
	if ((busy & 32) || !query_ip_number(spec))
	    break;

	if (this_player()->query_mana() >=
	    this_player()->query_max_mana() / 10)
	    this_player()->add_mana(-(this_player()->query_max_mana() / 10));
	else
	{
	    write("You feel far to exhausted to do that.\n");
	    break;
	}

	tell_object(spec, "COMMUNE to you from " + 
		    capitalize(this_player()->query_real_name()) + ":" +
		    mess + "\n");
	flag = 1;
	break;
    }

    if (flag)
	write("You feel you have communed with the deities.\n");
    else
	write("Your prayers remain unheard.\n");
    
    return 1;
}

/*
 * emote - Put here so NPC:s can emote (  No error messages if they do wrong,
 *	   why waste cpu on NPC:s ? ;-)   )
 */
emote(str)
{
    if (!strlen(str) || !this_player()->query_npc())
	return 0;

    say( ({ METNAME + " " + str + "\n", TART_NONMETNAME + " " + str + "\n",
	    UNSEEN_NAME + " " + str + "\n" }) );

    this_player()->reveal_me(1);
    return 1;
}

/*
 * forget - Forget someone we have remembered
 */
forget_live(name)
{
    object ob;

    ob =  find_living(name);
    if (ob && ob->query_prop(LIVE_I_NON_FORGET))
    {
	notify_fail("It seems impossible to forget " +
		    ob->query_objective()+"!\n");
	return 0;
    }
    if (this_player()->remove_remembered(name))
    {
	this_player()->add_introduced(name);
	write("Ok.\n");
	return 1;
    }
    else
    {
	notify_fail("You do not know any " + capitalize(name) + ".\n");
	return 0;
    }
}

/*
 * introduce - Present someone or myself to others
 */
int 
intro_live(string str) /* name */
{
    string *wa;
    object to_who;

    if (!str) return 0;

    wa = explode(str + " ", " ");
    notify_fail("Introduce who?\n");

    if (sizeof(wa) > 3)
	return 0;
    if (sizeof(wa) == 3)
    {
	if (wa[1] != "to")
	    return 0;
	notify_fail("Introduce to whom?\n");
	to_who = present(wa[2], environment(this_player()));
	
	if (!to_who || !CAN_SEE(this_player(), to_who))
	    return 0;
	notify_fail("Introduce who to " + to_who->short() +"?\n");

	if (intro_one_live(to_who, wa[0]))
	{
	    write("Ok.\n");
	    return 1;
	}
    }
    if (sizeof(wa) != 1)
	return 0;
    if (sizeof(filter(all_inventory(environment(this_player())) +
		      all_inventory(this_player()),
		     "intro_one_live", this_object(), wa[0])))
    {
	write("Ok.\n");
	return 1;
    }
    return 0;
}

/*
 * introduced - Give a list of livings we have been introduced to.
 */
introduced_list(str)
{
    object ob;
    mapping tmp;
    
    tmp = this_player()->query_introduced();
    if (mappingp(tmp))
    {
	write("You remember having been introduced to:\n");
	write(break_string(implode(map(m_indexes(tmp),
				       "capn", this_object()),
				   ", "), 70) + "\n");
	return 1;
    }
    else
    {
	write("You don't remember having been introduced to anyone.\n");
	return 1;
    }
}

/*
 * invite - Invite someone to join my team
 */
int
invite(string name)
{
    object *member_list, member;

    if (!name)
    {
	member_list = (object *) this_player()->query_invited();
	if (!member_list || !sizeof(member_list))
	    write("You have not invited anyone to join you.\n");
	else
	{
	    if (sizeof(member_list) == 1)
		write("You have invited " + member_list[0]->short() +
			".\n");
	    else {
		name = (string) COMPOSITE_FILE->desc_live(member_list);
		write("You have invited " +
		      (string) LANG_FILE->word_num(sizeof(member_list)) +
		      " people:\n");
		write(break_string(name, 76, 3));
	    }
	}
	return 1;
    }

    member = find_player(name);

    if (!member || (member && !present(member, environment(this_player()))))
    {
	notify_fail(capitalize(name) +
		    " is not a good potential team member!\n");
	return 0;
    }

    if (this_player()->query_leader())
    {
	notify_fail("You can't be a leader when you have a leader!\n");
	return 0;
    }

    if ((!member->query_met(this_player())) ||
	    (!this_player()->query_met(member)))
	return (notify_fail("You have not been introduced!\n"),0);

    if (member == this_player())
    {
	notify_fail("You do not need an invitation to your own team!\n");
	return 0;
    }

    this_player()->team_invite(member);
    member->catch_msg(QCTNAME(this_player()) + " invites you to join " +
		      this_player()->query_possessive() + " team.\n");
    this_player()->reveal_me(1);
    member->reveal_me(1);
		
    write("Ok.\n");
    return 1;
}

/*
 * join - Join someones team
 */
varargs int 
join(string name)
{
    object          leader;

    notify_fail("You must give the name of the player you want as " +
	"your leader.\n");

    if (!name)
	return 0;

    if (this_player()->query_leader())
    {
	write("You already have a leader!\n");
	return 1;
    }

    if (!this_player()->query_met(name))
    {
	write("You don't know anyone called '" + capitalize(name) + "'.\n");
	return 1;
    }

    leader = present(name, environment(this_player()));

    if (!leader || !leader->check_seen(this_player()))
    {
	write("You don't see " + capitalize(name) + " here.\n");
	return 1;
    }

    /*
     * Can not have a leader with too low DIS
     */
    if (leader->query_stat(SS_DIS) + 10 < this_player()->query_stat(SS_DIS) &&
		!this_player()->query_wiz_level())
    {
	write("You do not have enough faith in " + 
	      LANG_POSS(leader->short()) + " disciplin.\n");
	return 1;
    }

    if (leader->team_join(this_player()))
    {
	write("Your leader is now: " + leader->short() + ".\n");
	say(QCTNAME(this_player()) + " joined the team of " +
		QTNAME(leader) + ".\n", ({ leader, this_player() }));
	leader->catch_msg(QCTNAME(this_player()) + " joined your team.\n");
	return 1;
    }
    else
    {
	write(leader->short() +
	      " has not invited you as a team member.\n");
	return 1;
    }
}

/*
 * kill - Start attacking someone with the purpose to kill
 */
varargs int 
kill(string str)
{
    object      ob, *a;
    string 	str2;
    mixed	tmp;

    if (!CAN_SEE_IN_ROOM(this_player()))
    {
	notify_fail("You can't see anything here.\n");
	return 0;
    }

    if (this_player()->query_ghost())
    {
	notify_fail("Umm yes, killed. That's what you are.\n");
	return 0;
    }
    if (!str)
    {
	notify_fail("Kill what ?\n");
	return 0;
    }

    a = CMDPARSE_ONE_ITEM(str, "kill_access", "kill_access");
    if (sizeof(a) > 1)
    {
	str2 = COMPOSITE_LIVE(a);
	notify_fail(break_string("Be specific, you can't kill " + str2 +
		    " at the same time.\n",76));
	return 0;
    }
    else if (!sizeof(a))
    {
	notify_fail("You find no such living creature.\n");
	return 0;
    }
    ob = a[0];

    if (!ob)
    {
	notify_fail("No " + str + " here !\n");
	return 0;
    }
    if (!living(ob))
    {
	write(str + " is not a living thing !\n");
	say( ({ METNAME + " tries foolishly to kill " + str + ".\n",
		TART_NONMETNAME + " tries foolishly to kill " +
		str + ".\n", "" }) );
	return 1;
    }
    if (ob == this_player())
    {
	write("What ? Attack yourself ?\n");
	return 1;
    }
    if ((object) this_player()->query_enemy() == ob)
    {
	write("Yes, yes.\n");
	return 1;
    }

    if (tmp = environment(this_player())->query_prop(ROOM_M_NO_ATTACK))
    {
	if (stringp(tmp))
	    write(tmp);
	else
	    write("You sense a divine force preventing your attack.\n");
	return 1;
    }

    if (tmp = ob->query_prop(OBJ_M_NO_ATTACK))
    {
	if (stringp(tmp))
	    write(tmp);
	else
	    write("You feel a divine force protecting this being, your " +
		"attack fails.\n");
	return 1;
    }

    if (!this_player()->query_npc() && this_player()->query_met(ob) &&
	this_player()->query_prop(LIVE_O_LAST_KILL) != ob)
    {
	this_player()->catch_msg("Attack " + QTNAME(ob) +
				 "?!? Please confirm by trying again.\n");
	this_player()->add_prop(LIVE_O_LAST_KILL, ob);
	return 1;
    }

    this_player()->reveal_me();
    /*
     * Check if we dare!
     */
    if (!F_DARE_ATTACK(this_player(), ob))
    {
	write("Umm... no, You do not have enough self-discipline to dare!\n");
	return 1;
    }

    say(QCTNAME(this_player()) + " attacks " + QTNAME(ob) + ".\n",
        ({this_player(), ob}));
    ob->catch_msg(QCTNAME(this_player()) + " attacks you!\n");

    this_player()->attack_object(ob);
    this_player()->add_prop(LIVE_O_LAST_KILL, ob);

    write("Ok.\n");
    return 1;
}

/*
 * leave - Leave a team or force someone to leave a team
 */
int 
leave(string name)
{
    object ob;

    if (ob = (object) this_player()->query_leader())
    {
	ob->team_leave(this_player());
	write("You leave your leader.\n");
	ob->catch_msg(QCTNAME(this_player()) + " left the team.\n");
	this_player()->reveal_me(1);
	ob->reveal_me(1);
	return 1;
    }

    notify_fail("You are not in a group, you have no leader.\n");
    if (name)
	ob = find_player(name);
    else
	return 0;
    if (!ob)
    {
	notify_fail("No such player.\n");
	return 0;
    } else {
	if ((object) (ob->query_leader()) == this_player())
 	{
	    this_player()->team_leave(ob);
	    write("You force " + ob->short() + " to leave your team.\n");
	    ob->catch_msg(QCTNAME(this_player()) + 
			  " forces you to leave the team.\n");
	    this_player()->remove_invited(ob);
	    this_player()->reveal_me(1);
	    ob->reveal_me(1);
	    return 1;
	} else {
	    notify_fail("No such member in your team.\n");
	    return 0;
	}
    }
}

/*
 * reply - Allow mortals to reply when something is told to them.
 */
reply(str)
{
    string who;
    object tp, target;

    tp = this_player();

    if (!(who = tp->query_prop("_reply_wizard")))
    {
	notify_fail("You can only reply if someone told you something.\n");
	return 0;
    }

    if (!str)
    {
	notify_fail("Reply what?\n");
	return 0;
    }

    tp->remove_prop("_reply_wizard");

    if (!(target = find_player(who)))
    {
	write("You cannot reply, that person is no longer in the game.\n");
	return 1;
    }

    target->catch_msg(QCTNAME(tp) + " replies: " + str + "\n");
    tp->catch_msg("You replied to " + QTNAME(target) + ".\n");
    return 1;
}

/*
 * remember - Remember one of the livings introduced to us
 */
remember_live(str)
{
    object ob;
    mapping tmp;
    int num;
    
    if (!str || query_verb() == "remembered")
    {
	tmp = this_player()->query_remembered();
	if (mappingp(tmp))
	{
	    if (num = m_sizeof(tmp))
	    {
		num = F_MAX_REMEMBERED(this_player()->query_stat(SS_INT)) - num;
		if (num < 0)
		    num = 0;

		write("These are the people you remember:\n");
		write(break_string(implode(map(m_indexes(tmp),
					       "capn", this_object()),
					   ", "), 70) + "\n");
		write("Your brain can handle " + LANG_WNUM(num) +
			" more name" + (num == 1 ? ".\n" : "s.\n") );
		return 1;
	    }
	    else
	    {
		write("You do not seem to remember anyone.\n");
		return 1;
	    }
	}
	else
	{
	    write("You don't remember knowing anyone at all.\n");
	    return 1;
	}
    }

    str = lower_case(str);    
    if (strlen(str))
	ob =  find_living(str);
    
    if (!ob)
    {
	notify_fail("Remember whom?\n");
	return 0;
    }
    
    if (ob->query_prop(LIVE_I_NON_REMEMBER))
    { 	
	notify_fail("Remember" + ob->query_objective() + "? Never!");
 	return 0;
    }
    switch (this_player()->add_remembered(str))
    {
    case -1:
	write("Your poor brain can't handle any more people.\n");
	return 1;
    case 1:
	write("Ok.\n");
 	return 1;
    case 2:
	write("You refresh your memory of " + capitalize(str) + ".\n");
        return 1;
    default:
	notify_fail("You can't remember having been introduced to " + 
		    capitalize(str) + ".\n");
	return 0;
    }
}

/*
 * see - Do we want to see all other peoples combat messages
 */
varargs int
see(string str)
{
    notify_fail("See what? Fights?\n");
    if (!str)
	return 0;

    if (str == "fights")
    {
    	if (!this_player()->query_see_blood())
	    this_player()->set_see_blood(1);
    	else
    	{
	    notify_fail("You already keep your eyes open for fights.\n");
	    return 0;
    	}
     	write("Ok.\n");
   	return 1;
    } else if (str == "no fights") {
    	if (this_player()->query_see_blood())
	    this_player()->set_see_blood(0);
    	else
    	{
	    notify_fail("You already hide your eyes from combat if possible.\n");
	    return 0;
        }
    	write("Ok.\n");
    	return 1;
    }
    return 0;   	
}   

/*
 * stop - Stop hunting or fighting
 */
int
stop(string str)   /* Rewritten by Left of Hyperborea -- 23.4.92 */
{
    string a,b;
    object *e, *pe;
    
    if (!str)
    {
	notify_fail("Stop what? The world?\n");
	return 0;
    }

    pe = this_player()->query_enemy(-1);

    if (!pe || sizeof(pe) == 0)
    {
	notify_fail("You are not hunting anyone.\n");
	return 0;
    }

    /* use the full power of parse_command on the stop spec.  It will
     * now support "stop hunting all", "stop hunting vogons".. etc
     */
    if (parse_command(str, pe,
		      " 'hunt' / 'hunting' / 'fight' / 'fighting' %i ", e))
    {
	int i;
        for (i = 1; i < sizeof(e) ; i++)
        {
            this_player()->stop_fight(e[i]);
        }
        write("Ok.\n");
        return 1;
    }

    (sscanf(str, "%s %s", a, b) || sscanf(str, "%s", a)); /*gasp* /JnA */

    if (a == "hunt" || a == "hunting" || a == "fight" || a == "fighting")
    {
        /* stop hunting the last one fought */
        if (!b || b == "")
        {
            this_player()->stop_fight(this_player()->query_enemy());
            write("Ok.\n");
            return 1;
        }
        else
        /* give error messages for ones not being fought, and those that
         * do not exist.
         */
        {
            object not_hunted;

            not_hunted = find_player(b);
            if (!not_hunted)
                not_hunted = find_living(b);

            if (not_hunted)
	    {
                if (a == "hunt" || a == "hunting")
                    notify_fail("You are not hunting: " +
				not_hunted->query_the_name(this_player()) +
				"\n");
                else
		    notify_fail("You are not fighting: " +
				not_hunted->query_the_name(this_player()) +
				"\n");
		return 0;
	    }
            else
            {
                notify_fail("No such enemy found: " + b + "\n");
                return 0;
	    }
        }
    }
    else
    {
        notify_fail(capitalize(query_verb()) + " what ?\n");
        return 0;
    }
}

/*
 * team - Tell me what team I am a member 
 */
varargs int
team(string str)
{
    object	leader;
    object	*members;

    if (!str) /* My team */
    {
	if (leader = (object) this_player()->query_leader()) 
	{
	    members = (object *) leader->query_team();
	    write("You are a member of " + leader->short() + 
		  "'s team.");
	    members = filter(members, "filter_other_living", FILTER_FILE);
	    if (!sizeof(members)) 
	    {
		write(" You are the only member.\n");
		return 1;
	    }
	    else
		write(" The other members are:\n");
	}
	else if (sizeof(members = (object *) this_player()->query_team()) > 0)
	    write("You are the leader of your team. The members are:\n");
        if (sizeof(members))
    	{
	    write(break_string(implode(map(members, "map_fun",
						 FILTER_FILE, "short"),
				       " "), 76, 3) + "\n");
	    return 1;
	}
	notify_fail("You are not member of a team.\n");
	return 0;
    }
    notify_fail("You can find out about your team with this command.\n");
    return 0;
}

/*
 * who - Tell what players are logged in and who we know
 */
varargs int
who(object *oblist)
{
    object *list;
    int i;
    
    if (!pointerp(oblist))
	list = users();
    else
	list = oblist;

    i = 0;

    while (i < sizeof(list)) 
    {
	int seen;
	seen = (list[i]->query_prop(OBJ_I_INVIS) <=
	    this_player()->query_prop(LIVE_I_SEE_INVIS));
	if (!seen && (this_player()->query_wiz_level()))
	    write("(" + list[i]->query_real_name() + ")\n");
	else if (seen)
	    write(list[i]->query_presentation() + "\n");
	i++;
    }
    return 1;
}
varargs int
who_format(object *oblist)
{
    object *list;
    int i;
    
    if (!pointerp(oblist))
	list = users();
    else
	list = oblist;

    i = 0;

    while (i < sizeof(list)) 
    {
	int seen;
	seen = (list[i]->query_prop(OBJ_I_INVIS) <=
	    this_player()->query_prop(LIVE_I_SEE_INVIS));
	if (!seen && (this_player()->query_wiz_level()))
	    write("(" + list[i]->query_real_name() + ")\n");
	else if (seen)
	    write(list[i]->query_format_presentation() + "\n");
	i++;
    }
    return 1;
}


