/*
  cmd/live/state.c

  General commands for finding out a livings state.

*/
#pragma save_binary

inherit "/cmd/std/command_driver";

#include <macros.h>
#include <ss_types.h>
#include <std.h>
#include <stdproperties.h>
#include <language.h>
#include <composite.h>
#include <state_desc.h>
#include <cmdparse.h>
#include <filter_funs.h>
#include "/config/login/login.h"

#define SUBLOC_MISCEXTRADESC 	"_subloc_misc_extra"
#define SKILLOB 		"/lib/skill_raise"

/*
 * Global constants
 */
private mixed
        stat_strings, beauty_strings, 
	denom_strings, brute_fact, panic_state, fatigue_state,
	intox_state, head_state, stuff_state, soak_state,
	improve_fact, skillmap, compare_strings;

private string *stat_names, *health, *mana_state, *enc_weight;

private mapping lev_map;

/*
 * Prototypes
 */
public string beauty_text(int num, int sex);

create()
{
    seteuid(getuid(this_object())); 
    
    /* These global arrays are created once for all since they are used
       quite often. They should be considered constant, so do not mess
       with them
    */

    skillmap = 		SS_SKILL_DESC;
    denom_strings = 	SD_STAT_DENOM;

    stat_names = 	SD_STAT_NAMES;

    stat_strings = 	({ SD_STATLEV_STR, SD_STATLEV_DEX, SD_STATLEV_CON,
		      	   SD_STATLEV_INT, SD_STATLEV_WIS, SD_STATLEV_DIS, });

    compare_strings = 	({ SD_COMPARE_STR, SD_COMPARE_DEX, SD_COMPARE_CON,
			   SD_COMPARE_INT, SD_COMPARE_WIS, SD_COMPARE_DIS, });

    beauty_strings = 	({ SD_BEAUTY_FEMALE, SD_BEAUTY_MALE });

    brute_fact = 	SD_BRUTE_FACT;
    health = 		SD_HEALTH;
    mana_state = 	SD_MANA;
    panic_state = 	SD_PANIC;
    fatigue_state = 	SD_FATIGUE;
    soak_state = 	SD_SOAK;
    stuff_state = 	SD_STUFF;
    intox_state = 	SD_INTOX;
    head_state = 	SD_HEADACHE;
    improve_fact = 	SD_IMPROVE;
    enc_weight = 	SD_ENC_WEIGHT;
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return "state";
}

/* **************************************************************************
 * This is a command soul.
 */
int
query_cmd_soul()
{
    return 1;
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "compare":"compare",

	     "echoon":"echoon",
	     "echooff":"echooff",

	     "levels":"levels",

	     "skills":"show_skills",
	     "spells":"show_spells",
	     "stats":"show_stats",

	     "vitals":"show_vitals",
	     "wimpy":"wimpy",
	     "whimpy":"wimpy",
	     ]);
}

/*
 * Function name: using_soul
 * Description:   Called once by the living object using this soul. Adds
 *		  sublocations responsible for extra descriptions of the
 *		  living object.
 */
public void 
using_soul(object live)
{
    live->add_subloc(SUBLOC_MISCEXTRADESC, file_name(this_object()));
    live->add_textgiver(file_name(this_object()));
}

public string
show_subloc_size(object on, object for_obj)
{
    string race, res;
    int val, rval, *proc;

    race = on->query_race();

    if (member_array(race, RACES) >= 0)
    {
        val = on->query_prop(CONT_I_HEIGHT);
        rval = RACEATTR[race][0];
        val = 100 * val / (rval ? rval : val);
        proc = SPREAD_PROC;

        for (rval = 0; rval < sizeof(proc); rval++)
            if (val <= proc[rval])
                break;
        rval = (rval < sizeof(proc) ? rval : sizeof(proc) -1 );

        res = " " + HEIGHTDESC[rval] + " and ";

        val = on->query_prop(CONT_I_WEIGHT);
        rval = 1000 * RACEATTR[race][1];
        val = 100 * val / (rval ? rval : val);
        proc = SPREAD_PROC;

        for (rval = 0; rval < sizeof(proc); rval++)
            if (val <= proc[rval])
                break;
        rval = (rval < sizeof(proc) ? rval : sizeof(proc) -1 );

        res += WIDTHDESC[rval] + " for " + LANG_ADDART(race) + ".\n";
    }
    else
        res = "";

    return res;
}

public string
show_subloc_looks(object on, object for_obj)
{
    if (on->query_prop(NPC_I_NO_LOOKS))
	return "";

    return (for_obj == on ?
		"You look " : capitalize(on->query_pronoun()) + " looks ") +
                beauty_text(for_obj->my_opinion(on),
                            (on->query_gender() == G_FEMALE ? 0 : 1)) + ".\n";
}

public string
show_subloc_fights(object on, object for_obj)
{
    object eob;

    eob = (object)on->query_attack();
    
    return " fighting " + (eob == for_obj ? "you" : 
			   (string)eob->query_the_name(for_obj)) + ".\n";
}

public string
show_subloc_health(object on, object for_obj)
{
    int hp, mhp;

    hp = on->query_hp();
    mhp = on->query_max_hp();
    if (mhp == 0)
	mhp = 1;

    return this_object()->get_proc_text((hp * 100) / mhp, health, 0, ({}) );
}

/*
 * Function name: show_subloc
 * Description:   Shows the specific sublocation description for a living
 */
public string 
show_subloc(string subloc, object on, object for_obj)
{
    string res, cap_pronoun, cap_pronoun_verb, tmp;

    if (on->query_prop(TEMP_SUBLOC_SHOW_ONLY_THINGS))
	return "";

    if (for_obj == on)
    {
	res = "You are";
	cap_pronoun_verb = res;
	cap_pronoun = "You are ";
    }
    else
    {
	res = capitalize(on->query_pronoun()) + " is";
	cap_pronoun_verb = res;
	cap_pronoun = capitalize(on->query_pronoun()) + " seems to be ";
    }
    
    if (strlen(tmp = show_subloc_size(on, for_obj)))
	res += tmp;
    else
	res = "";

    res += show_subloc_looks(on, for_obj);
    
    if (on->query_attack())
        res += cap_pronoun_verb + show_subloc_fights(on, for_obj);

    res += cap_pronoun + show_subloc_health(on, for_obj) + ".\n";

    return res;
}

/****************************************************************
 *
 * A textgiver must supply these functions
 */

/*
 * Returns true if this textgiver describes the given stat
 */
public int 
desc_stat(int stat)
{
    return (stat >= 0 && stat <= SS_NO_EXP_STATS);
}

/*
 * Returns true if this textgiver describes the given skill
 */
public int 
desc_skill(int skill)
{
    if (sizeof(skillmap[skill]))
	return 1;
    else
	return 0;
}

/*
 * Function name: query_stat_string
 * Description:   Gives text information corresponding to a stat
 * Parameters:    stat: index in stat array
 *                value: stat value (range 1..100)
 *                negative value gives the statstring with index corresponding
 *                to -value
 * Returns:       string corresponding to stat/value
 */
public string
query_stat_string(int stat, int value)
{
    if (stat < 0 || stat >= SS_NO_EXP_STATS)
	return "";
    if (value < 0)
    {
	if (value < -sizeof(stat_strings[stat]))
	    value = 0;
	return stat_strings[stat][-value];
    }
    /*
     * 17 levels on each stat, 8 descriptions
     */
    if (value > 135)
	value = 135;
    return stat_strings[stat][sizeof(stat_strings[stat]) * value / 136];
}

public varargs string
get_proc_text(int num, mixed maindesc, int turnpoint, mixed subdesc)
{
    int a, b, c, d,j;
    mixed subs;

    if (!sizeof(maindesc))
	return ">normal<";

    if (num < 0)
	num = 0;
    if (num > 99)
	num = 99;

    j = sizeof(maindesc) * num / 100;    
    
    if (!pointerp(subdesc))
	subs = denom_strings;
    else if (sizeof(subdesc))
	subs = subdesc;
    else
	return maindesc[j];

    a = num - (j * 100 / sizeof(maindesc));

    b = (sizeof(subs) * a * sizeof(maindesc)) / 100;

    if (j < turnpoint)
	b = (sizeof(subs) - 1) - b;

    return subs[b] + maindesc[j];
}

/* **************************************************************************
 * Here follows some support functions. 
 * **************************************************************************/

/*
 * Function name: beauty_text
 * Description:   Return corresponding text to a certain combination of
 *                appearance and opinion
 */
public string
beauty_text(int  num, int sex)
{
    if (sex != 1)
	sex = 0;
    if (num >= 100)
	num = 99;
    if (num < 0)
	num = 0;
    if (!pointerp(beauty_strings) ||
	!pointerp(beauty_strings[sex]))
    {
	log_file("SKUMBUG", ctime(time()) + " " +
		 file_name(this_object()) + "\n");
	create();
	return beauty_text(num, sex);
    }
    return beauty_strings[sex][sizeof(beauty_strings[sex]) * num / 100];
}

/*
 * Function name: compare_living
 * Description:   Support function to compare
 */
mixed
compare_living(int stat, object player1, object player2)
{
    int a, b, c, skill, seed1, seed2, swap;

    a  = player1->query_stat(stat);
    b  = player2->query_stat(stat);
    
    if (player1 != player2 )
    {
	skill = this_player()->query_skill(SS_APPR_MON);
	sscanf(OB_NUM(player1), "%d", seed1);
	sscanf(OB_NUM(player2), "%d", seed2);
	skill = 1000 / (skill + 1);
	a += random(skill, seed1 + seed2 + 27 + stat);
	/* 27 is an arbitrarily selected constant */
	b += random(skill, seed1 + seed2 + stat);
    }

    if (a > b)
	c = 100 - (80 * b) / a;
    else
    {
	c = 100 - (80 * a) / b;
	swap = 1;
    }
    c = (c * sizeof(compare_strings[stat]) / 100);
    if (c > 3)
	c = 3;
    return ({swap, compare_strings[stat][c]});
}

/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/


/*
 * Function name: compare
 * Description:   compare stats of 2 living objects
 */
int
compare(string str)
{
    int i, me;
    string stat, name1, name2;
    mixed p1, p2;
    object pl1, pl2, *ob_list;
    mixed *cstr;

    notify_fail("Correct syntax is: 'compare <stat> with <person>' or\n" +
                "'compare <stat> of <person1> and <person2>'\n");
    if (!str)
        return 0;
    str += " ";
    ob_list = FILTER_LIVE(all_inventory(environment(this_player())));
    if (!parse_command(str, ({}), "%w 'of' %s 'and' %s", stat, name1, name2))
    {
	if (!parse_command(str, ({}), "%w 'with' %s", stat, name2))
	    return 0;
	else
	{
            p2 = FIND_STR_IN_ARR(name2, ob_list);
            if (sizeof(p2) < 1)
            {
            	notify_fail("Compare " + stat + " with who?\n");
            	return 0;
            }
            me = 1;
            pl2 = p2[0];
        }
    }
    else
    {
	p1 = FIND_STR_IN_ARR(name1, ob_list);
	p2 = FIND_STR_IN_ARR(name2, ob_list);
        if (sizeof(p1) < 1 || sizeof(p2) < 1)
	{
            return 0;
	}
        pl1 = p1[0];
        pl2 = p2[0];
    }

    notify_fail("Unknown stat. Possible choices are:\n" +
                implode(stat_names, ", ") + "\n");
    for (i = 0; ; i++)
    {
        if (i >= sizeof(stat_names))
            return 0;
        if (lower_case(stat) == lower_case(stat_names[i]))
            break;
    }

    notify_fail("Unknown player.\n");

    if (me)
    {
        cstr = compare_living(i, this_player(), pl2);
        this_player()->catch_msg((cstr[0] ?
			QCTNAME(pl2) + " is " : "You are ") +
               cstr[1] + " " +
               (cstr[0] ? "you" : (QTNAME(pl2))) + ".\n");
    }
    else
    {
        cstr = compare_living(i, pl1, pl2);
        this_player()->catch_msg ((QCTNAME((cstr[0] ? pl2 : pl1))) +
              " is " + cstr[1] + " " +
              (QTNAME((cstr[0] ? pl1 : pl2))) + ".\n");
    }

    return 1;

}

/*
 * echoon - say/tell/ask/whisper echoing
 */
echoon()
{
    write("Echo-mode is now on.\n");
    this_player()->set_get_echo(1);
    return 1;
}

/*
 * echooff
 */
echooff()
{
    write("Echo-mode is now off.\n");
    this_player()->set_get_echo(0);
    return 1;
}

/*
 * levels - Print a summary of different levels of different things
 */
public int
levels(string str)
{
    string *ix, *levs;

    if (!mappingp(lev_map))
	lev_map = SD_LEVEL_MAP;

    ix = m_indexes(lev_map);

    if (!str)
    {
	notify_fail("Available level descriptions:\n" + 
		    break_string(COMPOSITE_WORDS(ix) + ".", 70, 3) + "\n");
	return 0;
    }
    levs = lev_map[str];
    if (!sizeof(levs))
    {
	notify_fail("No such level descriptions. Available:\n" + 
		    break_string(COMPOSITE_WORDS(ix) + ".", 70, 3) + "\n");
	return 0;
    }	
    write("Level descriptions for: " + capitalize(str) + "\n" +
	  break_string(COMPOSITE_WORDS(levs) + ".", 70, 3) + "\n");
    return 1;
}

/*
 * spells - Show what spells we know
 */
/*
 * Function name: show_spells
 * Description:   List the active spells.
 */
int
show_spells()
{
    int i;
    object *ob_list;

    ob_list = (object *)this_player()->query_spellobjs();

    for (i = 0 ; i < sizeof(ob_list) ; i++)
	ob_list[i]->list_spells();

    return 1;
}

/*
 * vitals - Give vital state information about the living
 */
/*
 * Function name: show_vitals
 * Description:   Gives information on misc items
 */
varargs int
show_vitals(string str)
{
    int a, b, i, j;
    object ob;
    string s, s2, s3, s4, t;

    if (!str)
    {
	ob = this_player();
	s = "You are "; s2 = "you have "; s3 = "You feel "; s4 = "You can ";
    }
    else
    {
	if(!((ob = find_player(str)) && this_player()->query_wiz_level()))
	{
	    notify_fail("Curious aren't we?\n");
	    return 0;
	}
	s = capitalize(str);
	s2 = s + " has ";
	s3 = s + " feels "; 
	s4 = s + " can ";
	s = s + " is ";
    }

    /* height, width, race
     */

    /* Hitpoints and mana
     */
    a = (ob->query_hp() * 100) / ob->query_max_hp();
    t = get_proc_text(a, health, 0, ({}));
    b = (ob->query_mana() * 100) / ob->query_max_mana();

    write(s + "physically " + t + " and mentally " +
	  get_proc_text(b, mana_state, 0, ({})) + ".\n");

    /* panic
     */
    a = (10 + (int)ob->query_stat(SS_DIS) * 3);
    b = ob->query_panic();
    a = 100 * b / (a != 0 ? a : b);
    t = get_proc_text(a, panic_state, 2) + " and ";

    /* fatigue
     */
    a = ob->query_max_fatigue();
    b = a - ob->query_fatigue();
    a = 100 * b / (a != 0 ? a : b);
    t += get_proc_text(a, fatigue_state, 1) + ".";

    write(break_string(s3 + t, 76) + "\n");
    
    /* stuffed
     */
    a = ob->query_prop(LIVE_I_MAX_EAT);
    b = ob->query_stuffed();
    a = 100 * b / (a != 0 ? a : b);
    t = get_proc_text(a, stuff_state, 0, ({})) + " and ";

    /* soaked
     */
    a = ob->query_prop(LIVE_I_MAX_DRINK);
    b = ob->query_soaked();
    a = 100 * b / (a != 0 ? a : b);
    t += get_proc_text(a, soak_state, 0, ({})) + ".";

    write(break_string(s4 + t, 76) + "\n");

    /* intox
     */
    a = ob->query_prop(LIVE_I_MAX_INTOX);
    b = ob->query_intoxicated();
    a = 100 * b / (a != 0 ? a : b);
    if (b)
	write(s + get_proc_text(a, intox_state, 0) + ".\n");
    else
    {
	/* headache
	 */
	a = ob->query_prop(LIVE_I_MAX_INTOX);
	b = ob->query_headache();
	a = 100 * b / (a != 0 ? a : b);
	if (b)
	    write(s + "sober, but " + s2 + 
		  LANG_ADDART(get_proc_text(a, head_state, 1)) +
		  " headache.\n");
	else
	    write(s + "sober.\n");
    }

    /* weight, volume (This tells us what we can carry)
     */

    /*
     * Alignment
     */
    t = ob->query_align_text();

    write(s + t + ".\n");

    /*
     * Carry
     */
    a = ob->query_encumberance_weight();
    if (a >= 20)
    {
	a -= 20;
	write(s + get_proc_text(a * 100 / 75, enc_weight, 0, ({}) ) + ".\n");
    }

    /*
     * Age
     */
    write("Age:" + ob->show_age() + ".\n");

    return 1;

}

/*
 * stats - Give information on the livings stats
 */
/*
 * Function name: show_stats
 * Description:   Gives information on the stats
 */
varargs int
show_stats(string str)
{
    int a, b, i, j;
    float c, d;
    object ob;
    string s, s2, t;
    int *learn;

    if (!str)
    {
	ob = this_player();
	s = "You are ";
	a = ob->query_prop(PLAYER_I_LASTXP);
	b = ob->query_exp();
	j = b - a;
	if (a && j)
	{
	    j = 100 * j / b;
	    write("You have made " + 
		  get_proc_text(j, improve_fact, 0, ({})) +
		  " progress since last you checked.\n");
	}
	else
	    write("You have made no measurable progress since last you checked.\n");
	ob->add_prop(PLAYER_I_LASTXP, b);
    }
    else
    {
	if(!((ob = find_player(str)) && this_player()->query_wiz_level()))
	{
	    notify_fail("Curious aren't we?\n");
	    return 0;
	}
	s = capitalize(str) + " is ";
    }
    
    for (t = s, i = 0; i < SS_NO_EXP_STATS; i++)
    {
	s2 = query_stat_string(i, ob->query_stat(i));

	if (!i)
	    s2 = LANG_ADDART(s2);
	if (i < SS_NO_EXP_STATS - 2) 
	    s2 += ", ";
	else if (i < SS_NO_EXP_STATS - 1)
	    s2 += " and ";
	else
	    s2 += " " + ob->query_nonmet_name() + ".";
	t += s2;
    }
    write(break_string(t,76) + "\n");

    /* brutalfactor  
     */
    c = itof(ob->query_exp());
    d = itof(ob->query_exp_combat());
    j = ftoi(((100.0 * d) / ( c != 0 ? c : d)));
    write(s + get_proc_text(j, brute_fact, 2) + ".\n");

    return 1;

}

/*
 * skills - Give information on a livings skills
 */
/*
 * Function name: show_skills
 * Description:   Gives information on the stats
 */
varargs int
show_skills(string str)
{
    int il, wr, i1, i2;
    object ob;
    int *sk, ski;
    mapping skdesc;
    string s, s1, s2;

    wr = 1;
    skdesc = SS_SKILL_DESC;

    if (!str)
    {
	ob = this_player();
	s = "You have ";
	s2 = "";
    }
    else
    {
	if (sscanf(str, "%s %s", s1, s2) != 2)
	{
	    if (find_player(str))
	    {
		s1 = str;
		s2 = "";
	    } else {
		s1 = this_player()->query_real_name();
		s2 = str;
	    }
	}

	if (!(ob = find_player(s1)))
	{
	    notify_fail("No such player.\n");
	    return 0;
	}

	if (!this_player()->query_wiz_level() && ob != this_player())
	{
	    notify_fail("Curious aren't we?\n");
	    return 0;
	}
	s = capitalize(str) + " has ";
    }
    sk = ob->query_all_skill_types();

    SKILLOB->sk_init();

    i1 = 0;
    i2 = 9999999;
    switch (s2)
    {
    	case "general": i1 = 100; i2 = 200; break;
    	case "fighting": i1 = 0; i2 = 29; break;
    	case "magic": i1 = 30; i2 = 49; break;
    	case "thief": i1 = 50; i2 = 69; break;
	case "all":
	case "": i1 = 0; i2 = 9999999; break;
    	default:
	    notify_fail("Unknown group '" + s2 + "'. Try 'general', " +
		"'fighting', 'magic', 'thief' or 'all'.\n");
	    return 0;
    }

    for (il = 0; il < sizeof(sk); il++)
    {
	ski = sk[il];
	if (ski < i1 || ski > i2)
	    continue;

	if (pointerp(skdesc[sk[il]]))
	    str = skdesc[sk[il]][0];
	else {
	    str = ob->query_skill_name(sk[il]);
	    if (str == "")
		continue;
	    if (!str)
		str = "special";
	}
	if (++wr % 2)
	  {
	    write(sprintf("%-18s %s", str + ":",
			  SKILLOB->sk_rank(ob->query_skill(sk[il]))));
	    write("\n");
	  }
	else
	  {
	    s1 = sprintf("%-18s %s ", str + ":",
			SKILLOB->sk_rank(ob->query_skill(sk[il]))); 
	    write(sprintf("%-40s ", s1));
	  }
    }
    if (wr)
	write("\n");
    else
	write(s + "no skills.\n");

    return 1;
}

/*
 * wimpy - Set the level when we will flee from a fight
 */    
varargs int
wimpy(string str)
{
    string how;
    int wi;

    if (!str)
    {
	wi = this_player()->query_whimpy();
	if (wi == 0)
	{
	    write("You are not wimpy at all.\n");
	    return 1;
	}
	if (wi > 99)
	    wi = 99;
	wi = wi * sizeof(health) / 100;
	write("You are wimpy at: '" + health[wi] + "'.\n");
	return 1;
    }

    if (str == "brave")
    {
	this_player()->set_whimpy(0);
	write("Brave mode.\n");
	return 1;
    }

    wi = member_array(str, health);

    if (wi < 0)
    {
	notify_fail("No such health descriptions (" + str + ") Available:\n" +
		    break_string(COMPOSITE_WORDS(health) + ".", 70, 3) + "\n");
	return 0;
    }

    wi = (100 * (wi + 1)) / sizeof(health);
    if (wi > 99)
	wi = 99;

    how = health[sizeof(health) * wi / 100];

    write("You are now wimpy at: '" + how + "'.\n");
    this_player()->set_whimpy(wi);
    return 1;
}
