/*
 * /cmd/std/command_driver.c
 *
 * These routines handles the actual finding and execution of commands.
 * It is called from cmdhooks.c in the living object that performs the
 * command.
 */

/*
 * If you are going to copy this file, in the purpose of changing
 * it a little to your own need, beware:
 *
 * First try the following:
 *
 *    Create a new empty object, and make an inheritance of this object on the
 *    first line. This will automatically copy all variables and functions from
 *    the original object. Then, add the functions you want to change. The
 *    original function can still be accessed with '::' prepended on the name.
 *
 * The maintainer of this LPmud might become sad with you if you fail
 * to do any of the above. Ask other wizards if you are doubtful.
 *
 * The reason for this, is that the above saves a lot of memory.
 */

#include <macros.h>
#include <cmdparse.h>
#include <stdproperties.h>
#include <filter_funs.h>
#include <std.h>
#include <adverbs.h>

/*
 * Global variables
 */
static mapping cmdlist;    /* The list of verbs and functions */

static int command_loaded; /* Flag to remember of the verbs and functions
                                                        are loaded or not */

/*
 * Prototype
 */
string desc_many(object *oblist);

/* 
 * Function name: do_command
 * Description  : Perform the given command, if present.
 * Returns      : 1/0 depending on success.
 */
nomask int
do_command(string verb, string arg)
{
    string func;

    if (m_sizeof(cmdlist) == 0)
        return 0;

    func = cmdlist[verb];

    if (strlen(func) != 0)
        return (call_other(this_object(), func, arg));
    else
        return 0;
}

/* 
 * Function name: exist_command
 * Description  : Check if a command exists.
 * Returns      : 1/0 depending on success.
 */
nomask int
exist_command(string verb)
{
    if (!command_loaded)
    {
        cmdlist = this_object()->query_cmdlist();
        command_loaded = 1;
    }

    if (strlen(cmdlist[verb]))
        return 1;
    else
        return 0;
}

/*
 * Function name: open_soul
 * Description  : Set the euid of the soul to 0.
 */
nomask void
open_soul(int state)
{
    if (state)
        seteuid(getuid(this_object()));
    else
        SECURITY->remote_setuid();
}

/*
 * Function name: teleledningsanka
 * Description  : This function is used to load a soul and in command
 *                souls also to set the uid, euid to "backbone".
 */
void
teleledningsanka()
{
    SECURITY->remote_setuid();
}


/*
 * Function name: using_soul
 * Description  : this function is called from /std/living when it first
 *                starts to use the soul and can be used for initialization.
 */
void
using_soul(object live)
{
    return;
}

/* **************************************************************************
 * Here follows a couple of functions for easier handling in souls.
 * **************************************************************************
 */

/*
 * actor() the message the performer of an action
 *         gets when acting towards a target.
 * Example: "You hug Fatty."/"You hug the gnome."
 * Format:actor("string", array-of-objects [,"XstringY"]);
 *
 * Where X is usually one of " " / "." / "'"
 * and Y is usually one of "." / "!" / "?"
 *
 */
varargs void
actor(string str, object *oblist, string str1)
{
    this_player()->catch_msg(str + " " + desc_many(oblist) +
        (str1?(str1 + "\n"):".\n"));
}

/*
 * target() the message that the target of an action gets.
 * Example: "Mrpr hugs you."/"The darkly robed human wizard hugs you."
 * Format:target("XstringY", array-of-objects);
 *
 */
void
target(string str, object *oblist)
{
    int i;

    for(i = 0; i < sizeof(oblist); i++)
    {
        if (!CAN_SEE_IN_ROOM(oblist[i]) ||
                !CAN_SEE(oblist[i], this_player()))
            tell_object(oblist[i], UNSEEN_NAME + str + "\n");
        else if (oblist[i]->query_met(this_player()))
            tell_object(oblist[i], METNAME + str + "\n");
        else
            tell_object(oblist[i], TART_NONMETNAME + str + "\n");
    } 
}

/*
 * Call targetbb() if you don't want a person who cannot see the performans
 * to get a message. (target but blind)
 */
void
targetbb(string str, object *oblist)
{
    int i;

    for(i = 0; i < sizeof(oblist); i++)
    {
        if (CAN_SEE_IN_ROOM(oblist[i]) && 
            CAN_SEE(oblist[i], this_player()))
        {
            if (oblist[i]->query_met(this_player()))
                tell_object(oblist[i], METNAME + str + "\n");
            else
                tell_object(oblist[i], TART_NONMETNAME + str + "\n");
        }
    } 
}

/*
 * all() the message that all onlookers get,
 *       when there is only one person acting.
 * Example: "Fatty smiles happily."/"The big fat gnome wizard smiles happily."
 * Format:all("XstringY");
 *
 */
void
all(string str)
{
    say( ({ METNAME + str + "\n",
            TART_NONMETNAME + str + "\n",
            UNSEEN_NAME + str + "\n" }) );
}

/*
 * Call allbb() if you don't want people that cannot see this person to
 * get the message. (all but blind)
 */
void
allbb(string str)
{
    say( ({ METNAME + str + "\n",
            TART_NONMETNAME + str + "\n",
            "" })); 
}

/*
 * all2act() the message that all onlookers get
 *           when two players interact.
 * Example: "Fatty hugs the darkly robed human wizard." /
 *          "The big fat gnome wizard hugs Mrpr."
 * Format:all2act("string", array-of-objects [,"XstringY"]);
 *
 */
varargs void
all2act(string str, object *oblist, string str1)
{
    int i;

    str = " " + str + " " + desc_many(oblist) + (str1 ? (str1 + "\n") : ".\n");

    oblist = oblist + ({ this_player() });
    
    say(QCTNAME(this_player()) + str, oblist);
    for (i = 0 ; i < sizeof(oblist) ; i++)
        oblist[i]->reveal_me(1);
    this_player()->reveal_me(1);
}

/*
 * all2actbb() is to be used when people cannot see who is doing the action
 * i.e. you shouldn't see that someone smiles if you cannot see that person.
 */
varargs void
all2actbb(string str, object *oblist, string str1)
{
    object *ob;
    int i;

    str = " " + str + " " + desc_many(oblist) + (str1 ? (str1 + "\n") : ".\n");

    oblist = oblist + ({ this_player() });
    ob = all_inventory(environment(this_player())) - oblist;
    ob = FILTER_LIVE(ob);
    for(i = 0; i < sizeof(ob); i++)
        if (CAN_SEE_IN_ROOM(ob[i]) && CAN_SEE(ob[i], this_player()))
            ob[i]->catch_msg(QCTNAME(this_player()) + str);
}

/*
 * Function name: parse_this
 * Description  : This is a parser with some extra functions and checks,
 *                specially designed for the soul.           :)
 * Arguments    : str - the string to parse
 *                form - the parse-pattern
 * Returns      : an array of objects
 */
mixed
parse_this(string str, string form)
{
    object *oblist, *tmplist;
    int    i;

    str = lower_case(str);
    tmplist = ({});
    if (!CAN_SEE_IN_ROOM(this_player()))
        return tmplist;

    if (!str || (!parse_command(str, environment(this_player()),
            form, oblist)))
        return ({ });
 
    oblist = NORMAL_ACCESS(oblist, 0, 0);

    if (!sizeof(oblist))
        return ({ });

    for (i = 0; i < sizeof(oblist); i++)
    {
         if (objectp(oblist[i]) && living(oblist[i]) &&
                (this_player() != oblist[i]) &&
                CAN_SEE(this_player(), oblist[i]))
            tmplist = tmplist + ({ oblist[i] });
    }

    return tmplist;
}

/*
 * Function name: parse_adverb
 * Destription  : This function is designed to separate the adverb a player
 *                uses from the rest of the command line.
 * Arguments    : str     - the command line string
 *                def_adv - the default adverb for this emotion
 *                trail   - 1 if the adverb comes behind the target
 * Returns      : array of string: ({ cmd_str, adv_str })
 *                cmd_str - the rest of the line
 *                adv_str - the adverb
 *
 * Examples:
 *     parse_adverb("happ", "sadly", 0/1) returns
 *          ({ "", "happ" })
 *     parse_adverb("happ at the dwarf wizard", "sadly", 0)
 *          ({ "at the wizard", "happily" })
 *     parse_adverb("Mercade", "sadly", 0)
 *          ({ "Mercade", "sadly" })
 *     parse_adverb("Mercade merri", "gracefully", 1)
 *          ({ "Mercade", "merrily" })
 *     parse_adverb("merri Mercade", "gracefully", 1)
 *          ({ "merri Mercade", "gracefully" })
 */
string *
parse_adverb(string str, string def_adv, int trail)
{
    string *words;
    string adverb;

    if (!strlen(str))
        return ({ "", def_adv });

    words = explode(str, " ");

    if (sizeof(words) == 1)
    {
        if (strlen(adverb = FULL_ADVERB(words[0])))
            return ({ "", adverb });
        return ({ str, def_adv });
    }

    if (trail)
    {
        if (strlen(adverb = FULL_ADVERB(words[sizeof(words) - 1])))
            return ({ implode(words[0..(sizeof(words) - 2)], " "), adverb });
        else
            return ({ str, def_adv });
    }
    else
    {
        if (strlen(adverb = FULL_ADVERB(words[0])))
            return ({ implode(words[1..(sizeof(words) - 1)], " "), adverb });
        else
            return ({ str, def_adv });
    }

    /* should never happen */
    return ({ str, def_adv });
}

/*
 * Function name: parse_adverb_with_space
 * Description  : This function returns the adverb from parse_adverb with a
 *                preceding space if it is an adverb and it returns an empty
 *                string if the adverb was the special adverb that means that
 *                the player does not want an adverb.
 * Arguments    : see parse_adverb
 * Returns      : see parse_adverb, the description and /sys/adverbs.h
 */
string *
parse_adverb_with_space(string str, string def_adv, int trail)
{
    string *pa = parse_adverb(str, def_adv, trail);

    return ({ pa[0], ADD_SPACE_TO_ADVERB(pa[1]) });
}

/*
 * Function name: check_adverb
 * Description  : Returns the full adverb the player meant. If he did not
 *                specify an an adverb, return the default adverb.
 * Arguments    : str     - the pattern to match
 *                def_adv - the default adverb
 * Returns      : string  - the full adverb or NO_ADVERB
 */
string
check_adverb(string str, string def_adv)
{
    if (!strlen(str))
        return def_adv;

    return FULL_ADVERB(str);
}

/*
 * Function name: check_adverb_with_space
 * Description  : This function returns the adverb from check_adverb with a
 *                preceding space if it is an adverb and it returns an empty
 *                string if the adverb was the special adverb that means that
 *                the player does not want an adverb.
 * Arguments    : see check_adverb
 * Returns      : see check_adverb, the description and /sys/adverbs.h
 */
string
check_adverb_with_space(string str, string def_adv)
{
    return ADD_SPACE_TO_ADVERB(check_adverb(str, def_adv));
}

/*
 * Gives the player dependant descriptions of all objects
 * adressed by the actor.
 */

string
desc_many(object *oblist)
{
    string *qdesc, qd;

    qdesc = map(oblist, "desc_vbfc", this_object());

    if (sizeof(qdesc) == 1)
        qd = qdesc[0];
    else if (sizeof(qdesc) == 2)
        qd = qdesc[0] + " and " + qdesc[1];
    else
        qd = implode(qdesc[0 .. sizeof(qdesc) - 2], ", ") +
            " and " + qdesc[sizeof(qdesc) - 1];

    return qd;
}

string
desc_vbfc(object ob) { return QTNAME(ob); }
