/*
 * /cmd/std/soul_cmd.c
 *
 * It is the basic soul that is meant to be inherited by the race-specific
 * souls.
 *
 * This soul contains the basic emotions for players. You can use it as an
 * example for your guild-soul if you want to add more emotions to the
 * members as well.
 *
 * This soul has recently been cleaned up and enhanced. If you are going to
 * make any changes or additions to it, please use one of the four basic
 * formats here. Doing so will add to the readability of the soul.
 *
 * Revision history:
 *
 * Dynamic adverb handling and adverb prepositions on emotes that are aimed
 * at targets.
 *         /Plugh and /Mercade, 10 Noverber 1993
 */

/*
 * If you are going to copy this file, in the purpose of changing
 * it a little to your own need, beware:
 *
 * First try the following:
 *
 *    Create a new empty object, and make an inheritance of this object on the
 *    first line. This will automatically copy all variables and functions from
 *    the original object. Then, add the functions you want to change. The
 *    original function can still be accessed with '::' prepended on the name.
 *
 * The maintainer of this LPmud might become sad with you if you fail
 * to do any of the above. Ask other wizards if you are doubtful.
 *
 * The reason for this, is that the above saves a lot of memory.
 */

#pragma save_binary

inherit "/cmd/std/command_driver";

#include <macros.h>
#include <language.h>
#include <cmdparse.h>
#include <stdproperties.h>
#include <filter_funs.h>
#include <living_desc.h>
#include <ss_types.h>
#include <adverbs.h>

#define SUBLOC_SOULEXTRADESC "_std_soul_extra"
#define SOULDESC(x) (this_player()->add_prop(LIVE_S_SOULEXTRA, x))
#define LANGUAGE_FOR_RSAY 45   /* When will you understand an rsay */
#define DEPTH 1  /*  How many rooms away the shout will be heard.  */

/* **************************************************************************
 * Prototype
 */
static object * find_neighbour(object *found, object *search, int depth);

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return "Player emotions";
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alphabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "ack":"ack",
	     "admire":"admire",
	     "agree":"agree",
	     "apologize":"apologize",
	     "applaud":"applaud",

	     "bat":"bat",
	     "beg":"beg",
	     "blush":"blush",
	     "bounce":"bounce",
	     "bow":"bow",
	     "brighten":"brighten",
	     "burp":"burp",

	     "cackle":"cackle",
	     "caress":"caress",
	     "cheer":"cheer",
	     "choke":"choke",
	     "chortle":"chortle",
	     "chuckle":"chuckle",
	     "clap":"clap",
	     "comfort":"comfort",
	     "complain":"complain",
             "compliment":"compliment",
             "conf":"confused",        /* A shortcut for lazy people */
	     "confuse":"confuse",
	     "confused":"confused",
             "congrat":"congratulate", /* A shortcut for lazy people */
	     "congratulate":"congratulate",
	     "converse":"converse",
	     "cough":"cough",
	     "cower":"cower",
	     "cringe":"cringe",
	     "cry":"cry",
	     "cuddle":"cuddle",
	     "curtsey":"curtsey",

	     "dance":"dance",
	     "despair":"despair",
             "disagree":"disagree",
	     "duh":"duh",

	     "eeks":"eeks",
	     "explode":"explodes",
	     "eyebrow":"eyebrow",

	     "fart":"fart",
	     "fawn":"fawn",
	     "finger":"finger",
	     "flip":"flip",
	     "flirt":"flirt",
	     "fondle":"fondle",
             "forgive":"forgive",
	     "french":"french",
	     "frown":"frown",
	     "fume":"fume",

	     "gag":"gag",
	     "gasp":"gasp",
	     "giggle":"giggle",
	     "glare":"glare",
	     "greet":"greet",
	     "grin":"grin",
	     "groan":"groan",
	     "grope":"grope",
	     "grovel":"grovel",
	     "growl":"growl",
	     "grumble":"grumble",

	     "hiccup":"hiccup",
	     "hmm":"hmm",
	     "hold":"hold",
	     "hug":"hug",

	     "ignore":"ignore",

	     "jump":"jump",

	     "kick":"kick",
	     "kiss":"kiss",
	     "knee":"knee",

	     "laugh":"laugh",
	     "lick":"lick",
	     "love":"love",

	     "moan":"moan",

	     "nibble":"nibble",
	     "nod":"nod",
	     "nudge":"nudge",

	     "oops":"oops",

	     "panic":"panic",
	     "pat":"pat",
	     "peer":"peer",
	     "point":"point",
	     "poke":"poke",
	     "ponder":"ponder",
	     "pout":"pout",
	     "puke":"puke",
	     "purr":"purr",

	     "rolleyes":"rolleyes",
	     "rsay":"rsay",
	     "ruffle":"ruffle",

	     "scowl":"scowl",
	     "scratch":"scratch",
	     "scream":"scream",
	     "shake":"shake",
	     "shiver":"shiver",
	     "shout":"shout",
	     "shrug":"shrug",
	     "sigh":"sigh",
	     "sing":"sing",
	     "slap":"slap",
	     "smile":"smile",
	     "smirk":"smirk",
	     "snap":"snap",
	     "snarl":"snarl",
	     "sneeze":"sneeze",
	     "snicker":"snicker",
	     "sniff":"sniff",
	     "snore":"snore",
	     "snuggle":"snuggle",
	     "sob":"sob",
	     "spank":"spank",
	     "spit":"spit",
	     "squeeze":"squeeze",
	     "squirm":"squirm",
	     "stare":"stare",
	     "startle":"startle",
	     "steam":"steam",
	     "stomp":"stomp",
	     "stretch":"stretch",
	     "strut":"strut",
	     "sulk":"sulk",
	     "swear":"swear",

	     "tackle":"tackle",
	     "tap":"tap",
	     "thank":"thank",
	     "think":"think",
	     "tickle":"tickle",
	     "tremble":"tremble",
	     "twiddle":"twiddle",
	     "twinkle":"twinkle",

	     "wave":"wave",
	     "whine":"whine",
	     "whisper":"whisper",
	     "whistle":"whistle",
	     "wiggle":"wiggle",
	     "wink":"wink",
	     "worship":"worship",

	     "yawn":"yawn",
	     "yodel":"yodel"
	 ]);
}

/*
 * Function name: using_soul
 * Description:   Called once by the living object using this soul. Adds
 *                sublocations responsible for extra descriptions of the
 *                living object.
 */
public void
using_soul(object live)
{
    live->add_subloc(SUBLOC_SOULEXTRADESC, file_name(this_object()));
}

/*
 * Function name: show_subloc
 * Description:   Shows the specific sublocation description for a living
 */
public string
show_subloc(string subloc, object on, object for_obj)
{
    string str, cap_pronoun;

    if (on->query_prop(TEMP_SUBLOC_SHOW_ONLY_THINGS))
	return "";

    if (for_obj == on)
	cap_pronoun = "You are ";
    else
	cap_pronoun = capitalize(on->query_pronoun()) + " is ";

    str = on->query_prop(LIVE_S_SOULEXTRA);

    if (stringp(str))
	return cap_pronoun + str + ".\n";
    else
	return "";
}

/*
 * Function name: get_soul_id
 * Description  : Returns a proper name of the soul in order to get a nice
 *                printout. You should replace this function by your own.
 * Returns      : the name
 */
string
get_soul_id()
{
    return "soul";
}

/*
 * Function name: query_cmd_soul
 * Description  : This is a command soul.
 * Returns      : 1
 */
int
query_cmd_soul()
{
    return 1;
}

/* **************************************************************************
 * Here follows the actual functions. Please add new in alphabetical order.
 * **************************************************************************/

int
ack(string str)
{
    if (strlen(str))
    {
	notify_fail("Ack what?\n");
	return 0;
    }

    write("Ack!\n");
    all(" goes ack!");
    return 1;
}

int
admire(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Admire whom?\n");
	return 0;
    }

    target(" shows you " + this_player()->query_possessive() +
	"admiration.", oblist);
    actor("You show", oblist, " your admiration.");
    all2act("shows", oblist, " " + this_player()->query_possessive() +
	" admiration.");
    return 1;
}

int
agree(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "wholeheartedly", 0);

    if (!strlen(how[0]))
    {
	write("You agree" + how[1] + ".\n");
	allbb(" agrees" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[with] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Agree with whom?\n");
	return 0;
    }

    targetbb(" agrees" + how[1] + " with you.", oblist);
    actor("You agree" + how[1] + " with", oblist);
    all2actbb("agrees" + how[1] + " with", oblist);
    return 1;
}

int
apologize(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "remorsefully", 0);

    if (!strlen(how[0]))
    {
	write("You apologize" + how[1] + ".\n");
	all("apologizes" + how[1] + ".\n");
	return 1;
    }

    oblist = parse_this(how[0], "[to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Apologize to whom?\n");
	return 0;
    }

    target(" apologizes to you" + how[1] + ".", oblist);
    actor("You apologize to" , oblist, how[1] + ".");
    all2act("apologizes to", oblist, how[1] + ".");
    return 1;
}

int
applaud(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "wholeheartedly", 0);

    if (!strlen(how[0]))
    {
	if (how[1] == "wholeheartedly")
	    all(" gives a round of applause.");
	else
	    all(" applauds" + how[1] + ".");

	write("You applaud" + how[1] + ".\n");
	return 1;
    }

    oblist = parse_this(how[0], "[for] / [to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Applaud to whom?");
	return 0;
    }

    target(" applauds" + how[1] + " to you.", oblist);
    actor("You applaud" + how[1] + " to", oblist);
    all2act("applauds" + how[1] + " to", oblist);
    return 1;
}

int
bat(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "coquetishly", 1);

    if (!strlen(how[0]))
    {
	write("You bat your eyelashes" + how[1] + ".\n");
	allbb(" bats " + this_player()->query_possessive() +
	    " eyelashes" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[eyelashes] [at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Bat eyelashes at whom?");
	return 0;
    }

    targetbb(" bats " + this_player()->query_possessive() +
	" eyelashes at you" + how[1] + ".", oblist);
    actor("You bat your eyelashes at", oblist, how[1] + ".");
    all2actbb("bats " + this_player()->query_possessive() +
	" eyelashes at", oblist, how[1] + ".");
    return 1;
}

int
beg(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l [for] 'forgiveness'");

    if (sizeof(oblist))
    {
	target(" falls on " + this_player()->query_possessive() +
	    " knees and begs you for forgiveness.", oblist);
	actor("You fall on your knees and beg" +
	    (sizeof(oblist) > 1 ? " each of" : ""), oblist,
	    " for forgiveness.");
	all2act("falls on " + this_player()->query_possessive() +
	    "knees and begs", oblist, " for forgiveness.");
	return 1;
    }

    oblist = parse_this(str, "[the] %l [pardon]");

    if (!sizeof(oblist))
    {
	notify_fail("Beg whom?\n");
	return 0;
    }

    target(" begs your pardon.", oblist);
    actor("You beg", oblist, "'s pardon.");
    all2act("begs", oblist, "'s pardon.");
    return 1;
}

int
blush(string str)
{
    str = check_adverb_with_space(str, "profusely");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Blush how?\n");
	return 0;
    }

    write("Your cheeks are burning.\n");
    SOULDESC("blushing" + str);
    allbb(" blushes" + str + ".");
    return 1;
}

int
bounce(string str)
{
    str = check_adverb_with_space(str, "uncontrollably");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Bounce how?\n");
	return 0;
    }

    write("B O I N G !!\n");
    all(" bounces around" + str + ".");
    return 1;
}

int
bow(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "gracefully", 0);

    if (!strlen(how[0]))
    {
	write("You bow" + how[1] + ".\n");
	allbb(" bows" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[to] / [before] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Bow to whom?\n");
	return 0;
    }

    targetbb(" bows" + how[1] + " before you.", oblist);
    actor("You bow" + how[1] + " to", oblist);
    all2actbb("bows" + how[1] + " to", oblist);
    return 1;
}

int
brighten(string str)
{
    if (strlen(str))
    {
	notify_fail("Brighten what?\n");
	return 0;
    }

    write("You think about it, then it dawns on you! Your face brightens!\n");
    all(" thinks about it, then it dawns on " +
	this_player()->query_objective() + "..." +
	this_player()->query_possessive() + " face brightens.");
    return 1;
}

int
burp(string str)
{
    str = check_adverb_with_space(str, "rudely");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Burp what?\n");
	return 0;
    }

    write("Excuse yourself!\n");
    all(" burps" + str + ".");
    return 1;
}

int
cackle(string str)
{
    str = check_adverb_with_space(str, "with glee");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Cackle how?\n");
	return 0;
    }

    write("You cackle" + str + ".\n");
    all(" throws " + this_player()->query_possessive() +
       " head back and cackles" + str + "!");
    SOULDESC("cackling" + str);
    return 1;
}

int
caress(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "lovingly", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Caress whom?\n");
	return 0;
    }

    target(" caresses you" + how[1] + ".", oblist);
    actor("You caress", oblist, how[1] + ".");
    all2act("caresses", oblist, how[1] + ".");
    return 1;
}

int
cheer(string str)
{
    str = check_adverb_with_space(str, "enthusiastically");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Cheer how?\n");
	return 0;
    }

    write("You cheer" + str + ".\n");
    all(" cheers" + str + ".");
    SOULDESC("cheering" + str);
    return 1;
}

int
choke(string str)
{
    if (strlen(str))
    {
	notify_fail("Choke what?\n");
	return 0;
    }

    write("Cough, cough, cough, hark !!!\n");
    all("'s face colour slowly darkens as " +
	this_player()->query_pronoun() + " chokes.");
    SOULDESC("choking on something");
    return 1;
}

int
chortle(string str)
{
    if (strlen(str))
    {
	notify_fail("Chortle what?\n");
	return 0;
    }

    write("You chortle.\n");
    all(" chortles.");
    SOULDESC("chortling");
    return 1;
}

int
chuckle(string str)
{
    str = check_adverb_with_space(str, "politely");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Chuckle how?\n");
	return 0;
    }

    write("You chuckle" + str + ".\n");
    all(" chuckles" + str + ".");
    SOULDESC("chuckling" + str);
    return 1;
}

int
clap(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "briefly", 0);

    if (!strlen(how[0]))
    {
	all(" claps" + how[1] + ".");
	write("You clap" + how[1] + ".\n");
	return 1;
    }

    oblist = parse_this(how[0], "[for] / [to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Clap for whom?");
	return 0;
    }

    target(" claps" + how[1] + " for you.", oblist);
    actor("You clap" + how[1] + " for", oblist);
    all2act("claps" + how[1] + " for", oblist);
    return 1;
}

int
comfort(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Comfort whom?\n");
	return 0;
    }

    target(" comforts you.", oblist);
    actor("You comfort", oblist);
    all2act("comforts", oblist);
    return 1;
}

int
complain(string str)
{
    object *oblist;

    oblist = parse_this(str, "[to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Complain to whom?\n");
	return 0;
    }

    SOULDESC("complaining miserably");
    target(" complains about the miserable state of things to you.", oblist);
    actor("You complain about the miserable state of things to", oblist);
    all2act("complains to", oblist);
    return 1;
}

int
compliment(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Compliment whom?\n");
	return 0;
    }

    target(" gives " + this_player()->query_possessive() +
        " deepest compliments you.", oblist);
    actor("You give your deepest compliments to ", oblist);
    all2act("gives " + this_player()->query_possessive() +
        " deepest compliments to", oblist);
    return 1;
}

int
confuse(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Confuse whom?\n");
	return 0;
    }

    target(" tries in vain to confuse you.", oblist);
    actor("You try to confuse", oblist, "...\nBut fail...");
    all2act("tries in vain to confuse", oblist);
    return 1;
}

int
confused(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You look very confused.\n");
	all(" looks very confused.");
	SOULDESC("looking very confused");
	return 1;
    }

    oblist = parse_this(str, "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Look confused at whom?\n");
	return 0;
    }

    target(" looks very confused at you.", oblist);
    actor("You look very confused at", oblist);
    all2act("looks very confused at", oblist);
    SOULDESC("looking very confused");
    return 1;
}

int
congratulate(string str)
{
    object *oblist;
    int    i;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Congratulate whom?\n");
	return 0;
    }

    for (i = 0; i < sizeof(oblist); i++)
	if (oblist[i]->query_gender() == 1)
	    actor("You congratulate", ({ oblist[i] }), ". What a gal!");
	else
	    actor("You congratulate", ({ oblist[i] }), ". What a guy!");

    target(" congratulates you.", oblist);
    all2act("congratulates", oblist);
    return 1;
}

int
converse()
{
    write("Entering converse mode.\nGive '**' to stop.\n");
    write("]");
    input_to("converse_more");
    return 1;
}

nomask void
converse_more(string str)
{
    string cmd;

    if (str == "**")
    {
	write("Ok.\nYou just left converse mode.\n");
	return;
    }
    if (str[0] == '!')
    {
	sscanf(str, "!%s", cmd);
	this_player()->command(cmd);
    }
    else
    {
	say( ({ METNAME + LD_SAYS + LD_UNDERSTANDS(str) + "\n",
		TART_NONMETNAME + LD_SAYS + LD_UNDERSTANDS(str) + "\n",
		UNSEEN_NAME + LD_SAYS + LD_UNDERSTANDS(str) + "\n" }) );
    }

    write("]");
    input_to("converse_more");
}

int
cough(string str)
{
    str = check_adverb_with_space(str, "noisily");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Cough how?\n");
	return 0;
    }

    write("Cover your mouth when you do that!\n");
    all(" coughs" + str + ".");
    SOULDESC("coughing" + str);
    return 1;
}

int
cower(string str)
{
    if (strlen(str))
    {
	notify_fail("Cower what?\n");
	return 0;
    }

    write("You cower in the corner.\n");
    allbb(" cowers in the corner.");
    return 1;
}

int
cringe(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You cringe in terror.\n");
	allbb(" cringes in terror.");
	SOULDESC("cringing in terror");
	return 1;
    }

    oblist = parse_this(str, "[to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Cringe to whom?\n");
	return 0;
    }

    target(" cringes in terror at your feet.", oblist);
    actor("You cringe in terror from", oblist);
    all2act("cringes in terror at the feet of", oblist);
    SOULDESC("cringing in terror");
    return 1;
}

int
cry(string str)
{
    if (strlen(str))
    {
	notify_fail("Cry what?\n");
	return 0;
    }

    write("Waaaaah!\n");
    all(" bursts into tears.");
    SOULDESC("crying");
    return 1;
}

int
cuddle(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "lovingly", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Cuddle whom?\n");
	return 0;
    }

    target(" cuddles you" + how[1] + ".", oblist);
    actor("You cuddle", oblist, how[1] + ".");
    all2act("cuddles", oblist, how[1] + ".");
    return 1;
}

int
curtsey(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "gracefully", 0);

    if (!strlen(how[0]))
    {
	write("You curtsey" + how[1] + ".\n");
	all(" curtseys" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Curtsey to whom?\n");
	return 0;
    }

    targetbb(" curtseys" + how[1] + " to you.", oblist);
    actor("You curtsey" + how[1] + " to", oblist);
    all2actbb("curtseys" + how[1] + " to", oblist);
    return 1;
}

int
dance(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("Feels silly, doesn't it?\n");
	all(" does the disco duck.");
	return 1;
    }

    oblist = parse_this(str, "[with] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Dance with whom?\n");
	return 0;
    }

    target(" sweeps you across the dance floor.", oblist);
    actor("You sweep", oblist, " across the dance floor.");
    all2act("sweeps", oblist, " across the dance floor.");
    return 1;
}

int
despair(string str)
{
    if (strlen(str))
    {
	notify_fail("Despair what?\n");
	return 0;
    }

    write("You tear a lump of hair off your scalp.\n");
    all(" tears a lump of hair off " + this_player()->query_possessive() +
       " scalp in despair.");
    SOULDESC("looking desparate");
    return 1;
}

int
disagree(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "totally", 0);

    if (!strlen(how[0]))
    {
	write("You disagree" + how[1] + ".\n");
	allbb(" disagrees" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[with] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Disagree with whom?\n");
	return 0;
    }

    targetbb(" disagrees" + how[1] + " with you.", oblist);
    actor("You disagree" + how[1] + " with", oblist);
    all2actbb("disagrees" + how[1] + " with", oblist);
    return 1;
}

int
duh(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("Uh, duhhh...  Feeling stupid, eh?\n");
	all(" goes duhhh.  Must be feeling dumb.");
	SOULDESC("probably feeling very dumb");
	return 1;
    }

    oblist = parse_this(str, "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Duh at whom?\n");
	return 0;
    }

    if (sizeof(oblist) == 1)
	str = (string)oblist[0]->query_objective();
    else
	str = "them";

    target(" goes duhhh at you! Guess you must have done something dumb!",
	oblist);
    actor("You let", oblist, " know what you think of " + str + "!");
    all2act("goes duhhh at", oblist, ", making " + str + " feel pretty dumb!");
    return 1;
}

int
eeks(string str)
{
    if (strlen(str))
    {
	notify_fail("Eeks what?\n");
	return 0;
    }

    write("Eeks!  Oh my gosh!\n");
    all(" goes eeks!");
    return 1;
}

int
explodes(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You hold your breath and explode in anger!\n");
	all(" explodes with anger.");
	return 1;
    }

    oblist = parse_this(str, "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Explode at whom?\n");
	return 0;
    }

    target(" explodes with anger at you.", oblist);
    actor("You explode in anger at", oblist, "!");
    all2act("explodes with anger at", oblist, "!");
    return 1;
}

int
eyebrow(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "inquisitively", 0);

    if (!strlen(how[0]))
    {
	write("You raise an eyebrow" + how[1] + ".\n");
	allbb(" raises " + this_player()->query_possessive() +
	      " eyebrow" + how[1] + ".");

	return 1;
    }

    oblist = parse_this(how[0], "[eyebrow] [at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Raise your eyebrow at whom?\n");
	return 0;
    }

    targetbb(" raises " + this_player()->query_possessive() + " eyebrow " +
	how[1] + " at you.", oblist);
    actor("You raise your eyebrow" + how[1] + " at", oblist);
    all2actbb("raises " + this_player()->query_possessive() +
	" eyebrow" + how[1] + " at", oblist);
    return 1;
}

int
fart(string str)
{
    if (strlen(str))
    {
	notify_fail("Fart how?\n");
	return 0;
    }

    write("How rude!\n");
    all(" lets off a real rip-roarer.");
    return 1;
}

int
fawn(string str)
{
    object *oblist;

    oblist = parse_this(str, "[at] / [over] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Fawn over whom?\n");
	return 0;
    }

    target (" fawns over you.", oblist);
    actor("You fawn over", oblist);
    all2act("fawns over", oblist);
    return 1;
}

int
finger(string str)
{
    object *oblist;

    oblist = parse_this(str, "[at] / [to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Show finger to whom?\n");
	return 0;
    }

    targetbb(" implies with " + this_player()->query_possessive() +
	" middle finger,\nthat " + this_player()->query_pronoun() +
	" wants you to fuck off.", oblist);
    actor("You give", oblist, " a fuck finger.");
    all2actbb("implies with " + this_player()->query_possessive() +
	" middle finger,\nthat " + this_player()->query_pronoun() +
	" wants", oblist, " to fuck off.");
    return 1;
}

int
flip(string str)
{
    if (strlen(str))
    {
	notify_fail("Flip what?\n");
	return 0;
    }

    write("You flip head over heels.\n");
    all(" flips head over heels.");
    return 1;
}

int
flirt(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "outrageously", 0);

    oblist = parse_this(how[0], "[with] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Flirt with whom?\n");
	return 0;
    }

    targetbb(" flirts" + how[1] + " with you.", oblist);
    actor("You flirt" + how[1] + " with", oblist);
    all2actbb("flirts" + how[1] + " with", oblist);
    return 1;
}

int
fondle(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "absentmindedly", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Fondle whom?\n");
	return 0;
    }

    target(" fondles you" + how[1] + ".", oblist);
    actor("You fondle", oblist, how[1] + ".");
    all2act("fondles", oblist, how[1] + ".");
    return 1;
}

int
forgive(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "generously", 1);

    oblist = parse_this(how[0],
        "[the] %l [his] / [her] / [its] / [their] [sins]");

    if (!sizeof(oblist))
    {
	notify_fail("Forgive whom?\n");
	return 0;
    }

    target(" forgives you your sins" + how[1] + ".", oblist);
    actor("You forgive", oblist, " " +
	((sizeof(oblist) == 1) ? this_player()->query_possessive() : "their") +
	" sins" + how[1] + ".");
    all2act("forgives", oblist, " " +
	((sizeof(oblist) == 1) ? this_player()->query_possessive() : "their") +
        " sins" + how[1] + ".");
    return 1;
}

int
french(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("French whom?\n");
	return 0;
    }

    target(" gives you a deep and passionate kiss...\n" +
	"It seems to take forever...", oblist);
    actor("You give" + (sizeof(oblist) > 1 ? " each of" : ""), oblist,
	" a REAL kiss..it takes a long time..");
    all2act("gives", oblist, " a deep and passionate kiss.\n" +
	"It seems to take forever...Sexy, eh?");
    return 1;
}

int
frown(string str)
{
    if (strlen(str))
    {
	notify_fail("Frown what?\n");
	return 0;
    }

    write("Is something wrong?\n");
    all(" frowns.");
    SOULDESC("frowning");
    return 1;
}

int
fume(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "angrily", 0);

    if (!strlen(how[0]))
    {
	write("You fume" + how[1] + ".\n");
	allbb(" fumes" + how[1] + ".");
	SOULDESC("fuming" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Fume at whom?\n");
	return 0;
    }

    SOULDESC("fuming" + how[1]);
    targetbb(" fumes" + how[1] + " at you.", oblist);
    actor("You fume" + how[1] + " at", oblist,".  Ooh, that feels good!");
    all2actbb("fumes" + how[1] + " at", oblist);
    return 1;
}

int
gag(string str)
{
    if (strlen(str))
    {
	notify_fail("Gag what?\n");
	return 0;
    }

    write("You gag.\n");
    all(" gags.");
    SOULDESC("gagging");
    return 1;
}

int
gasp(string str)
{
    if (strlen(str))
    {
	notify_fail("Gasp what?\n");
	return 0;
    }

    write("You gasp in astonishment.\n");
    all(" gasps in astonishment!");
    SOULDESC("gasping in astonishment");
    return 1;
}

int
giggle(string str)
{
    str = check_adverb_with_space(str, "inanely");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Giggle how?\n");
	return 0;
    }

    write("You giggle" + str + ".\n");
    all(" giggles" + str + ".");
    SOULDESC("giggling" + str);
    return 1;
}

int
glare(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "stonily", 0);

    if (!strlen(how[0]))
    {
	write("You glare" + how[1] + ".\n");
	allbb(" glares" + how[1] + ".");
	SOULDESC("glaring" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Glare at whom?\n");
	return 0;
    }

    SOULDESC("glaring" + how[1]);
    targetbb(" glares" + how[1] + " at you.", oblist);
    actor("You glare" + how[1] + " at", oblist);
    all2actbb("glares" + how[1] + " at", oblist);
    return 1;
}

int
greet(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Greet whom?\n");
	return 0;
    }

    target(" raises " + this_player()->query_possessive() +
	" hand and greets you!", oblist);
    actor("You greet", oblist,".  How friendly!");
    all2act("greets", oblist);
    return 1;
}

int
grin(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "evilly", 0);

    if (!strlen(how[0]))
    {
	write("You grin" + how[1] + ".\n");
	allbb(" grins" + how[1] + ".");
	SOULDESC("grinning" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Grin at whom?\n");
	return 0;
    }

    targetbb(" grins" + how[1] + " at you.", oblist);
    actor("You grin" + how[1] + " at", oblist);
    all2act("grins" + how[1] + " at", oblist);
    SOULDESC("grinning" + how[1]);
    return 1;
}

int
groan(string str)
{
    str = check_adverb_with_space(str, "loudly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Groan how?\n");
	return 0;
    }

    write("You groan" + str + ".\n");
    all(" groans" + str + ".");
    SOULDESC("groaning" + str);
    return 1;
}

int
grope(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Grope whom?\n");
	return 0;
    }

    target(" gropes you.", oblist);
    write("Well what sort of noise do you expect here?\n");
    all2act("gropes", oblist);
    return 1;
}

int
grovel(string str)
{
    object *oblist;

    oblist = parse_this(str, "[to] / [for] / [before] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Grovel in front of whom?\n");
	return 0;
    }

    target(" falls to the ground and grovels in front of you.", oblist);
    actor("You fall to the ground in front of", oblist, ", groveling.");
    all2act("falls to the ground and grovels in front of", oblist);
    return 1;
}

int
growl(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "menacingly", 0);

    if (!strlen(how[0]))
    {
	write("You growl" + how[1] + ".\n");
	all(" growls" + how[1] + ".");
	SOULDESC("growling" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Growl at whom?\n");
	return 0;
    }

    SOULDESC("growling" + how[1]);
    target(" growls" + how[1] + " at you.", oblist);
    actor("You growl" + how[1] + " at", oblist);
    all2act("growls" + how[1] + " at", oblist);
    return 1;
}

int
grumble(string str)
{
    object *oblist;
    string *how;
    string tmp = (this_player()->query_wiz_level() ? "angrily" : "unhappily");
	/* Quod licet Jovi, non licet bovi accourding to Plugh, Mercade ;-) */

    how = parse_adverb_with_space(str, tmp, 0);

    if (!strlen(how[0]))
    {
	write("You grumble" + how[1] + ".\n");
	all(" grumbles" + how[1] + ".");
	SOULDESC("grumbling" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Grumble at whom?\n");
	return 0;
    }

    target(" grumbles" + how[1] + " at you.", oblist);
    actor("You grumble" + how[1] + " at", oblist);
    all2act("grumbles" + how[1] + " at", oblist);
    SOULDESC("grumbling" + how[1]);
    return 1;
}

int
hiccup(string str)
{
    if (strlen(str))
    {
	notify_fail("Hiccup what?\n");
	return 0;
    }

    write("Hic!\n");
    all(" hiccups.");
    SOULDESC("having the hiccups");
    return 1;
}

int
hmm(string str)
{
    if (strlen(str))
    {
	notify_fail("Hmm what?\n");
	return 0;
    }

    write("Hmmmmm.\n");
    all(" goes hmmmm.");
    return 1;
}

int
hold(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l [close]");

    if (!sizeof(oblist))
    {
	notify_fail("Hold whom?\n");
	return 0;
    }

    target(" holds you close.", oblist);
    actor("You hold", oblist, " close.");
    all2act("holds", oblist, " close.");
    return 1;
}

int
hug(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, BLANK_ADVERB, 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Hug whom?\n");
	return 0;
    }

    target(" hugs you" + how[1] + ".", oblist);
    actor("You hug", oblist, how[1] + ".");
    all2act("hugs", oblist, how[1] + ".");
    return 1;
}

int
ignore(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Ignore whom?\n");
	return 0;
    }

    targetbb(" turns " + this_player()->query_possessive() + " back on you " +
	"and starts to ignore you.", oblist);
    actor("You ignore", oblist);
    all2actbb("turns " + this_player()->query_possessive() +
	" back on", oblist);
    return 1;
}

int
jump(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, NO_DEFAULT_ADVERB, 0);

    if (!strlen(how[0]))
    {
	if (how[1] == NO_DEFAULT_ADVERB_WITH_SPACE)
	    how[1] = ADD_SPACE_TO_ADVERB("unexpectedly");

	write("You jump" + how[1] + ".\n");
	all(" jumps" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[on] / [over] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Jump how?\n");
	return 0;
    }

    if (how[1] == NO_DEFAULT_ADVERB_WITH_SPACE)
	how[1] = ADD_SPACE_TO_ADVERB("absentmindedly");

    target(" jumps" + how[1] + " all over you.", oblist);
    actor("You jump" + how[1] + " all over", oblist);
    all2act("jumps" + how[1] + " all over", oblist);
    return 1;
}

int
kick(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "fanatically", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Kick whom?\n");
	return 0;
    }

    target(" kicks you" + how[1] + ".   OUCH!!!", oblist);
    actor("You kick", oblist, how[1] + ".  Yeah, I like that!");
    all2act("kicks", oblist, how[1] + ".");
    return 1;
}

int
kiss(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "gently", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Kiss whom?\n");
	return 0;
    }

    target(" kisses you" + how[1] + ".", oblist);
    actor("You kiss", oblist, how[1] + ".");
    all2act("kisses", oblist, how[1] + ".");
    return 1;
}

int
knee(string str)
{
    object *oblist;
    int    i;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Knee whom?\n");
	return 0;
    }

    for (i = 0; i < sizeof(oblist); i ++)
    {
	if (oblist[i]->query_gender())
	{
	    target(" tries to knee you, without much effect.",
		({ oblist[i] }) );
	    actor("You try to knee", ({ oblist[i] }),
		".\nNot very effective though.");
	    all2act("tries to knee", ({ oblist[i] }),
		".\nNot very effective though.");
	}
	else
	{
	    target(" hits you with " + this_player()->query_possessive() +
		" knee below your belt!\n" +
		"You double over and fall to the ground, writhing in " +
		"excrutiating pain,\nfeeling like you may throw up " +
		"everything you have eaten!", ({ oblist[i] }) );
	    actor("You hit", ({ oblist[i] }), " with your knee!");
	    all2act("suddenly raises " + this_player()->query_possessive() +
		" knee, sending", ({ oblist[i] }),
		" to the floor, writhing in pain!");
	}
    }
    return 1;
}

int
laugh(string str)
{
    object *oblist;
    string *how;

    if (!strlen(str))
    {
	write("You fall down laughing.\n");
	all(" falls down laughing.");
	SOULDESC("laughing");
	return 1;
    }

    how = parse_adverb_with_space(str, "exuberantly", 0);

    if (!strlen(how[0]))
    {
	write("You laugh" + how[1] + ".\n");
	all(" laugh" + how[1] + ".");
	SOULDESC("laughing" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Laugh how?\n");
	return 0;
    }

    target(" laughs" + how[1] + " at you.", oblist);
    actor("You laugh" + how[1] + " at", oblist, ".");
    all2act("laughs" + how[1] + " at", oblist);
    SOULDESC("laughing" + how[1]);
    return 1;
}

int
lick(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Lick whom?\n");
	return 0;
    }

    target(" licks you.", oblist);
    actor("You lick", oblist);
    all2act("licks", oblist);
    return 1;
}

int
love(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Love whom?\n");
	return 0;
    }

    target(" whispers sweet words of love to you.", oblist);
    actor("You tell your true feelings to", oblist);
    all2act("whispers softly to", oblist);
    SOULDESC("hopelessly in love");

    return 1;
}

int
moan(string str)
{
    str = check_adverb_with_space(str, BLANK_ADVERB);

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Moan how?\n");
	return 0;
    }

    write("You start to moan" + str + ".\n");
    all(" starts to moan" + str + ".");
    SOULDESC("moaning" + str);
    return 1;
}

int
nibble(string str)
{
    object *oblist;

    oblist = parse_this(str, "[on] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Nibble on whose ear?\n");
	return 0;
    }

    target(" nibbles on your ear.", oblist);
    actor("You nibble on", oblist, "'s ear.");
    all2act("nibbles on", oblist, "'s ear.");
    return 1;
}

int
nod(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "solemnly", 0);

    if (!strlen(how[0]))
    {
	write("You nod" + how[1] + ".\n");
	allbb(" nods" + how[1] + ".");
	SOULDESC("nodding" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Nod how?\n");
	return 0;
    }

    targetbb(" nods" + how[1] + " at you.", oblist);
    actor("You nod" + how[1] + " at", oblist);
    all2actbb("nods" + how[1] + " at", oblist);
    SOULDESC("nodding" + how[1]);
    return 1;
}

int
nudge(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Nudge whom?\n");
	return 0;
    }

    target(" nudges you.", oblist);
    actor("You nudge", oblist);
    all2act("nudges", oblist);
    return 1;
}

int
oops(string str)
{
    if (strlen(str))
    {
	notify_fail("Oops what?\n");
	return 0;
    }

    write("Oops!\n");
    say("Oops, " + QTNAME(this_player()) +
	" seems to have made a mistake.\n");
    return 1;
}

int
panic(string str)
{
    if (strlen(str))
    {
	notify_fail("Panic what?\n");
	return 0;
    }

    write("PANIC!!!!!\n");
    allbb(" panics and looks for a place to hide.");
    return 1;
}

int
pat(string str)
{
    object *oblist;
    string *zones;
    string one, two;

    zones = ({ "back", "head", "shoulder" });

    notify_fail("Whom are you trying to pat?\n");
    if (!strlen(str))
	return 0;

    if (sscanf(str, "%s %s", one, two) == 2)
    {
	if (member_array(two, zones) == -1)
	    return 0;
	str = one;
    }

    if (!strlen(two))
	two = "head";

    if ((str == (string)this_player()->query_real_name()) ||
	(str == "me") ||
	(str == "myself"))
    {
        if (str != "head")
        {
            notify_fail("Patting yourself on the " + two + ", huh ??");
            return 0;
        }

	write("You pat yourself on your head.\n");
	all(" pats " + this_player()->query_objective() +
	   "self on " + this_player()->query_possessive() + " head.");
	return 1;
    }

    oblist = parse_this(str, "[the] %l [on] [the]");

    if (!sizeof(oblist))
    {
	return 0;
    }

    target(" pats you on your " + two + ".", oblist);
    actor("You pat", oblist, " on " +
        (sizeof(oblist) > 1 ? "their " + two + "s.":
	oblist[0]->query_possessive() + " " + two + "."));
    all2act("pats", oblist, " on " +
        (sizeof(oblist)> 1 ? "their " + two + "s.":
	oblist[0]->query_possessive() + " " + two + "."));
    return 1;
}

int
peer(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "quizzically", 0);

    if (!strlen(how[0]))
    {
	write("You peer" + how[1] + " around.\n");
	allbb(" peers" + how[1] + " around.");
	SOULDESC("peering" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Peer at whom?\n");
	return 0;
    }

    targetbb(" peers" + how[1] + " at you.", oblist);
    actor("You peer" + how[1] + " at", oblist,".  Hmm......");
    all2actbb("peers" + how[1] + " at", oblist);
    SOULDESC("peering" + how[1]);
    return 1;
}

int
point(string str)
{
    object *oblist;
    string *tmp;

    notify_fail("Where do you want to point?\n");

    if (!strlen(str))
    {
	write("And where do you think that is?\n");
	allbb(" points in a general direction.");
	return 1;
    }

    if (str == "up" || str == "down" || str == "north" || str == "south" ||
	str == "west" || str == "east")
    {
	write("You point " + str + ".\n");
	allbb(" points " + str + ".");
	return 1;
    }

    if (str == (string) this_player()->query_real_name() ||
	str == "me" || str == "myself")
    {
	write("You point at yourself.\n");
	allbb(" points at " + this_player()->query_objective() + "self.");
	return 1;
    }

    oblist = parse_this(str, "[at] [the] %l");

    if (!sizeof(oblist))
    {
	tmp = explode(str, " ");
	if (sizeof(tmp) > 1 && tmp[0] == "at")
	    str = implode(tmp[1 .. sizeof(tmp) - 1], " ");
	oblist = FIND_STR_IN_OBJECT(str, environment(this_player()));
	if (!sizeof(oblist))
	{
	    if (environment(this_player())->item_id(str))
	    {
		write("You point at the " + str + ".\n");
		allbb(" points at " + LANG_ADDART(str) + ".");
		return 1;
	    }
	    return 0;
	}
	write("You point at " + LANG_THESHORT(oblist[0]) + ".\n");
	allbb(" points at " + LANG_THESHORT(oblist[0]) + ".");
	return 1;
    }

    targetbb(" points at you.", oblist);
    actor("You point at", oblist);
    all2actbb("points at", oblist);

    return 1;
}

int
poke(string str)
{
    object *oblist;
    string *zones;
    string one, two;

    zones = ({"eye", "ear", "thorax", "abdomen", "shoulder", "ribs"});

    notify_fail("Poke whom?\n");
    if (!strlen(str))
	return 0;

    if (sscanf(str, "%s %s", one, two) == 2)
    {
	if (member_array(two, zones) == -1)
	    return 0;
	str = one;
    }

    if (!strlen(two))
	two = "ribs";

    oblist = parse_this(str, "[the] %l [in] [the]");

    if (!sizeof(oblist))
    {
	return 0;
    }

    target(" pokes you in the " + two + ".", oblist);
    actor("You poke", oblist, " in the " + two + ".");
    all2act("pokes", oblist, " in the " + two + ".");
    return 1;
}

int
ponder(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You ponder the situation.\n");
	all(" ponders the situation.");
	SOULDESC("pondering the situation");
	return 1;
    }

    if (strlen(str) > 50)
    {
	write("You ponder beyond the end of the line and wake up from your reveries.\n");
	all(" ponders the situation.");
	SOULDESC("pondering the situation");
	return 1;
    }

    oblist = parse_this(str, "[about] [the] [proposal] [of] [the] %l");

    if (!sizeof(oblist))
    {
	write("You ponder " + str + ".\n");
	all(" ponders " + str + ".");
	SOULDESC("pondering about something");
	return 1;
    }

    target(" ponders about your proposal.", oblist);
    actor("You ponder about the proposal of", oblist);
    all2act("ponders about the proposal of", oblist);
    SOULDESC("pondering about a proposal");
    return 1;    
}

int
pout(string str)
{
    str = check_adverb_with_space(str, "petulantly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Pout how?\n");
	return 0;
    }

    write("You pout" + str + ".\n");
    all(" pouts" + str + ".");
    SOULDESC("pouting" + str);
    return 1;
}

int
puke(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You puke on your shoes.\n");
	all(" doubles over and pukes.");
	return 1;
    }

    oblist = parse_this(str, "[on] / [over] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Puke on whom?\n");
	return 0;
    }

    target(" pukes all over you!", oblist);
    actor("You puke on", oblist);
    all2act("pukes on", oblist);
    return 1;
}

int
purr(string str)
{
    str = check_adverb_with_space(str, "contentedly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Purr how?\n");
	return 0;
    }

    write("MMMMEEEEEEEEOOOOOOOWWWWWWW!\n");
    all(" purrs" + str + ".");
    SOULDESC("purring" + str);
    return 1;
}

int
rolleyes(string str)
{
    if (strlen(str))
    {
	notify_fail("Roll your eyes what?\n");
	return 0;
    }

    write("You roll your eyes in exasperation.\n");
    all(" rolls " + this_player()->query_possessive() + " eyes in "+
	"exasperation.");
    return 1;
}

int
rsay(string str)
{
    int i;
    object *oblist;

    if (!environment(this_player()))
	return 0;

    if (!strlen(str))
    {
	notify_fail("Say what in your racial tongue?\n");
	return 0;
    }

    oblist = FILTER_OTHER_LIVE(all_inventory(environment(this_player())));

    for (i = 0; i < sizeof(oblist); i++)
    {
	if ((this_player()->query_race_name() ==
		oblist[i]->query_race_name()) ||
	    (oblist[i]->query_wiz_level()) ||
	    (oblist[i]->query_skill(SS_LANGUAGE) >= LANGUAGE_FOR_RSAY))
	{
	    oblist[i]->catch_msg(QCTNAME(this_player()) + " says: " + str +
		"\n");
	}
	else
	{
	    oblist[i]->catch_msg(QCTNAME(this_player()) +
		" says something completely incomprehensible.\n");
	}
    }

    if (this_player()->query_get_echo())
	write("You rsay: " + str + "\n");
    else
	write("Ok.\n");

    return 1;
}

int
ruffle(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "playfully", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Ruffle whom?\n");
	return 0;
    }

    target(" ruffles your hair" + how[1] + ".", oblist);
    actor("You ruffle", oblist, "'s hair" + how[1] + ".");
    all2act("ruffles", oblist, "'s hair" + how[1] + ".");
    return 1;
}

int
scowl(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "menacingly", 0);

    if (!strlen(how[0]))
    {
	write("You scowl" + how[1] + ".\n");
	allbb(" scowls" + how[1] + ".");
	SOULDESC("scowling" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Scowl at whom?\n");
	return 0;
    }

    targetbb(" scowls" + how[1] + " at you.", oblist);
    actor("You scowl" + how[1] + " at", oblist);
    all2actbb("scowls" + how[1] + " at", oblist);
    SOULDESC("scowling" + how[1]);
    return 1;
}

int
scratch(string str)
{
    object *oblist;
    string *zones;
    string one, two;

    zones = ({ "head", "chin", "back", "behind", "nose" });

    if (!strlen(str))
	str = "head";

    if (member_array(str, zones) != -1)
    {
	write("You scratch your " + str + ".\n");
	allbb("scratches " + this_player()->query_possessive() +
	    " " + str + ".");
	return 1;
    }

    notify_fail("Scratch what?\n");
    if (sscanf(str, "%s %s", one, two) == 2)
    {
	if (member_array(two, zones) == -1)
	    return 0;
    }

    if (!strlen(two))
	two = "head";

    oblist = parse_this(one, "[the] %l [at] [the]");

    if (!sizeof(oblist))
    {
	return 0;
    }

    str = ((sizeof(oblist) > 1) ?
	"their " : (oblist[0]->query_possessive() + " "));

    target(" scratches you at your " + two + ".", oblist);
    actor("You scratch", oblist, " at " + str + two + ".");
    all2act("scratches", oblist, " at " + str + two + ".");
    return 1;
}

int
scream(string str)
{
    object *rooms, troom;
    int    i;

    if (strlen(str))
    {
	notify_fail("Scream what?\n");
	return 0;
    }
    if (!(troom = environment(this_player()))) return 0;

    rooms = find_neighbour(({ troom }), ({ troom }), DEPTH);

    for (i = 0; i < sizeof(rooms); i++)
	tell_room(rooms[i], "@@shout_name:" + file_name(this_object()) +
		  "@@ screams loudly!\n", this_player());

    write("ARRGGGGGGHHHHHH!!!!\n");
    return 1;
}

int
shake(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You shake your head in disagreement.\n");
	allbb(" shakes " + this_player()->query_possessive() +
	    " head in disagreement.");
	return 1;
    }

    oblist = parse_this(str, "[with] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Shake hands with whom?\n");
	return 0;
    }

    target(" shakes your hand.", oblist);
    actor("You shake hands with", oblist);
    all2act("shakes", oblist, "'s hand.");
    return 1;
}

int
shiver(string str)
{
    if (strlen(str))
    {
	notify_fail("Shiver what?\n");
	return 0;
    }

    write("Brrrrrr!!!\n");
    all(" shivers from the cold.");
    SOULDESC("shivering from the cold");
    return 1;
}

int
shout(string what)
{
    object *rooms;
    object troom;
    int il;

    if (!this_player())
	return 0;

    if (!what)
	return (notify_fail("Shout what?\n"),0);

    if (!(troom = environment(this_player())))
	return 0;

    rooms = find_neighbour(({ troom }), ({ troom }), DEPTH);

    for (il = 0; il < sizeof(rooms); il++)
	tell_room(rooms[il], "@@shout_name:" + file_name(this_object()) +
		  "@@ shouts: " + what + "\n", this_player());
    if (this_player()->query_get_echo())
	write("You shout: " + what + "\n");
    else
	write("Ok.\n");
    return 1;
}

string
shout_name()
{
    object pobj;
    string str;

    pobj = previous_object(); /* Reciever of message */
    if (file_name(pobj) == "/secure/vbfc_object")
	pobj = previous_object(-1);
    if (pobj->query_met(this_player()))
	return this_player()->query_name();
    else
    {
	if (environment(pobj) == environment(this_player()))
	    str = this_player()->query_nonmet_name();
	else
	    str = this_player()->query_gender_string() + " " +
		this_player()->query_race_name() + " voice";
	return capitalize(LANG_ADDART(str));
    }
}

static object *
find_neighbour(object *found, object *search, int depth)
{
    int il, im;
    mixed *exit_arr;
    object *new_search, *rooms, troom;

    if (!depth)
	return found;

    rooms = found;
    new_search = ({ });

    for (il = 0; il < sizeof(search); il++)
    {
	exit_arr = (mixed *) search[il]->query_exit();

	for (im = 0; im < sizeof(exit_arr); im += 3)
	{
	    if (objectp(exit_arr[im]))
		troom = exit_arr[im];
	    else
		troom = find_object(exit_arr[im]);
	    if ((troom) && (member_array(troom, rooms) < 0))
	    {
		rooms += ({ troom });
		new_search += ({ troom });
	    }
	}
    }

    return find_neighbour(rooms, new_search, depth - 1);
}

int
shrug(string str)
{
    str = check_adverb_with_space(str, "helplessly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Shrug what?\n");
	return 0;
    }

    write("You shrug" + str + ".\n");
    allbb(" shrugs" + str + ".");
    return 1;
}

int
sigh(string str)
{
    str = check_adverb_with_space(str, "deeply");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Sigh what?\n");
	return 0;
    }

    write("You sigh" + str + ".\n");
    all(" sighs" + str + ".");
    SOULDESC("sighing" + str);
    return 1;
}

int
sing(string str)
{
    if (strlen(str))
    {
	notify_fail("Sing what?\n");
	return 0;
    }

    write("Oh sole mio!\n");
    all(" sings in Italian.");
    return 1;
}

int
slap(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Slap whom?\n");
	return 0;
    }

    target(" slaps you!", oblist);
    actor("You slap", oblist);
    all2act("slaps", oblist);
    return 1;
}

int
smile(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "happily", 0);

    if (!strlen(how[0]))
    {
	write("You smile" + how[1] + ".\n");
	allbb(" smiles" + how[1] + ".");
	SOULDESC("smiling" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Smile at whom?\n");
	return 0;
    }

    SOULDESC("smiling" + how[1]);
    targetbb(" smiles" + how[1] + " at you.", oblist);
    actor("You smile" + how[1] + " at", oblist);
    all2actbb("smiles" + how[1] + " at", oblist);
    return 1;
}

int
smirk(string str)
{
    if (strlen(str))
    {
	notify_fail("Smirk what?\n");
	return 0;
    }

    write("You smirk.\n");
    all(" smirks.");
    SOULDESC("smirking");
    return 1;
}

int
snap(string str)
{
    if (strlen(str))
    {
	notify_fail("Snap what?\n");
	return 0;
    }

    write("You snap your fingers.\n");
    all(" snaps " + this_player()->query_possessive() + " fingers.");
    return 1;
}

int
snarl(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "savagely", 0);

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Snarl at whom?\n");
	return 0;
    }

    target(" snarls" + how[1] + " at you. Aren't you scared ?", oblist);
    actor("You snarl" + how[1] + " at", oblist);
    all2act("snarls" + how[1] + " at", oblist);
    SOULDESC("snarling" + how[1]);
    return 1;
}

int
sneeze(string str)
{
    if (strlen(str))
    {
	notify_fail("Sneeze what?\n");
	return 0;
    }

    write("Gezundheit!\n");
    all(" sneezes.");
    return 1;
}

int
snicker(string str)
{
    if (strlen(str))
    {
	notify_fail("Snicker what?\n");
	return 0;
    }

    write("You snicker.\n");
    all(" snickers.");
    SOULDESC("snickering");
    return 1;
}

int
sniff(string str)
{
    str = check_adverb_with_space(str, "pitifully");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Sigh what?\n");
	return 0;
    }

    write("You sniff" + str + ".\n");
    all(" sniffs" + str + ".");
    SOULDESC("sniffing" + str);
    return 1;
}

int
snore(string str)
{
    str = check_adverb_with_space(str, "loudly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Snore what?\n");
	return 0;
    }

    write("Zzzzzzzzzz...\n");
    all(" snores" + str + ".");
    SOULDESC("snoring" + str);
    return 1;
}

int
snuggle(string str)
{
    object *oblist;

    oblist = parse_this(str, "[with] / [up] [to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Snuggle up to whom?\n");
	return 0;
    }

    target(" snuggles up to you.", oblist);
    actor("You snuggle up to", oblist);
    all2act("snuggles up to", oblist);
    return 1;
}

int
sob(string str)
{
    str = check_adverb_with_space(str, "sadly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Sob how?\n");
	return 0;
    }

    write("You sob" + str + ".\n");
    all(" sobs" + str + ".");
    SOULDESC("sobbing" + str);
    return 1;
}

int
spank(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "playfully", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Spank whom?\n");
	return 0;
    }

    target(" spanks you" + how[1] + ".", oblist);
    actor("You spank", oblist, how[1] + ".");
    all2act("spanks", oblist, how[1] + ".");
    return 1;
}

int
spit(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You spit on the ground in disgust.\n");
	all(" spits on the ground in disgust.");
	return 1;
    }

    oblist = parse_this(str, "[on] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Spit at whom?\n");
	return 0;
    }

    target(" spits on you!", oblist);
    actor("You spit on", oblist);
    all2act("spits on", oblist);
    return 1;
}

int
squeeze(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "fondly", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Squeeze whom?\n");
	return 0;
    }

    target(" squeezes you" + how[1] + ".", oblist);
    actor("You squeeze", oblist, how[1] + ".");
    all2act("squeezes", oblist, how[1] + ".");
    return 1;
}

int
squirm(string str)
{
    str = check_adverb_with_space(str, "uncomfortably");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Squirm what?\n");
	return 0;
    }

    write("You squirm" + str + ".\n");
    all(" squirms" + str + ".");
    return 1;
}

int
stare(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("You stare into space.\n");
	allbb(" stares into space.");
	return 1;
    }

    oblist = parse_this(str, "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Stare at whom?\n");
	return 0;
    }

    targetbb(" stares deep into your eyes.", oblist);
    actor("You stare dreamily at", oblist);
    all2actbb("stares dreamily at", oblist);
    return 1;
}

int
startle(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Startle whom?\n");
	return 0;
    }

    target(" startles you.", oblist);
    actor("You startle", oblist);
    all2act("startles", oblist);
    return 1;
}

int
steam(string str)
{
    if (strlen(str))
    {
	notify_fail("Steam what?\n");
	return 0;
    }

    write("Steam comes boiling out of your ears.\n");
    allbb("'s face turns purple and steam starts to boil out of " +
       this_player()->query_possessive() + " ears.");
    return 1;
}

int
stomp(string str)
{
    if (strlen(str))
    {
	notify_fail("Stomp what?\n");
	return 0;
    }

    write("You stomp your feet angrily on the ground.\n");
    all(" stomps " + this_player()->query_possessive() + " feet angrily " +
	"on the ground.");
    return 1;
}

int
stretch(string str)
{
    if (strlen(str))
    {
	notify_fail("Strech what?\n");
	return 0;
    }

    write("You stretch your tired body out.\nAhh, that feels good.\n");
    all(" stretches " + this_player()->query_possessive() + " tired body.");
    return 1;
}

int
strut(string str)
{
    str = check_adverb_with_space(str, "proudly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Strut what?\n");
	return 0;
    }

    write("Strut your stuff!\n");
    all(" struts" + str + ".");
    return 1;
}

int
sulk(string str)
{
    if (strlen(str))
    {
	notify_fail("Sulk what?\n");
	return 0;
    }

    write("You sulk in the corner.\n");
    allbb(" sulks in the corner.");
    return 1;
}

int
swear(string str)
{
    str = check_adverb_with_space(str, "loudly");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Swear what?\n");
	return 0;
    }

    write("Go wash your mouth.\n");
    all(" swears" + str + ".");
    return 1;
}

int
tackle(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Tackle whom?\n");
	return 0;
    }

    if (random(7))
    {
	target(" comes running at you.\n" +
	    capitalize((string) this_player()->query_pronoun()) +
	    " attempts to tackle you and succeds.\n" +
	    "You fall to the ground in a very unflattering way.", oblist);
	actor("You tackle", oblist);
	all2act("tackles", oblist, ".\n" + ((sizeof(oblist) > 1) ? "They fall":
	    capitalize((string) oblist[0]->query_pronoun()) + " falls") +
	    " to the ground in a very unflattering way.");
    }
    else
    {
	target(" comes running at you.\n" +
	    capitalize((string) this_player()->query_pronoun()) +
	    " attempts to tackle you but misses.", oblist);
	actor("You try to tackle", oblist, " but fall flat on your face.");
	all2act("tries to tackle", oblist, " but misses.");
	all(" falls flat on " + this_player()->query_possessive() + " face.");
    }

    return 1;
}

int
tap(string str)
{
    str = check_adverb_with_space(str, "impatiently");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Tap how?\n");
	return 0;
    }

    write("You tap your foot" + str + ".\n");
    all( " taps " + this_player()->query_possessive() + " foot" + str + ".");
    return 1;
}

int
thank(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "gratefully", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Thank whom?\n");
	return 0;
    }

    target(" thanks you" + how[1] + ".", oblist);
    actor("You thank", oblist, how[1] + ".");
    all2act("thanks", oblist, how[1] + ".");
    return 1;
}

int
think(string str)
{
    if (!strlen(str))
    {
	write("You try to look thoughtful but fail.\n");
	allbb(" tries to look thoughtful but fails.");
	return 1;
    }

    write("You think hard about " + str + ".\n");
    allbb(" looks like " + this_player()->query_pronoun() +
	" is thinking hard about " + str + ".\n");
    return 1;
}

int
tickle(string str)
{
    object *oblist;
    string *how;
    int    i;

    how = parse_adverb_with_space(str, "playfully", 1);

    oblist = parse_this(how[0], "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Tickle whom?\n");
	return 0;
    }

    target(" tickles you" + how[1] + ".", oblist);
    actor("You tickle", oblist, how[1] + ".");
    all2act("tickles", oblist, how[1] + ".");

    oblist->command("laugh");

    return 1;
}

int
tremble(string str)
{
    if (strlen(str))
    {
	notify_fail("Tremble what?\n");
	return 0;
    }

    write("You're trembling in your boots.\n");
    allbb(" trembles and quivers like a bowlful of jelly.");
    return 1;
}

int
twiddle(string str)
{
    if (strlen(str))
    {
	notify_fail("Twiddle what?\n");
	return 0;
    }

    write("You twiddle your thumbs.\n");
    allbb(" twiddles " + this_player()->query_possessive() + " thumbs.");
    return 1;
}

int
twinkle(string str)
{
    str = check_adverb_with_space(str, "merrily");

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Twinkle how?\n");
	return 0;
    }

    write("You twinkle your eyes" + str + ".\n");
    all(" twinkles " + this_player()->query_possessive() + " eyes" +
	str + ".");
    return 1;
}

int
wave(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "happily", 0);

    if (!strlen(how[0]))
    {
	write("You wave" + how[1] + ".\n");
	allbb(" waves" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[to] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Wave to whom?\n");
	return 0;
    }

    targetbb(" waves" + how[1] + " in your direction.", oblist);
    actor("You wave" + how[1] + " in", oblist, "'s direction.");
    all2actbb("waves" + how[1] + " in", oblist, "'s direction.");
    return 1;
}

int
whine(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "petulantly", 0);

    if (!strlen(how[0]))
    {
	write("You whine" + how[1] + ".\n");
	allbb(" whines" + how[1] + ".");
	SOULDESC("whining" + how[1]);
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Whine at whom?\n");
	return 0;
    }

    SOULDESC("whining" + how[1]);
    targetbb(" whines" + how[1] + " to you.", oblist);
    actor("You whine" + how[1] + " to", oblist);
    all2actbb("whines" + how[1] + " to", oblist);
    return 1;
}

int
whisper(string str)
{
    object *oblist, *tmplist;
    string msg;
    int    i;

    tmplist = ({});
    notify_fail("Whisper to whom?\n");

    if (!str || (!parse_command(str, environment(this_player()),
	    "[to] [the] %l %s", oblist, msg)))
	return 0;

    oblist = NORMAL_ACCESS(oblist, 0, 0);

    if (!sizeof(oblist))
	return 0;

    for (i = 0; i < sizeof(oblist); i++)
    {
	if (!(!objectp(oblist[i]) || !living(oblist[i]) ||
		this_player() == oblist[i]))
	    tmplist = tmplist + ({ oblist[i] });
    }

    oblist = tmplist;

    if (!sizeof(oblist))
	return 0;

    if (!strlen(msg))
    {
	actor("Whisper what to", oblist);
	return 1;
    }

    target(" whispers to you: " + msg, oblist);
    all2act("whispers something to", oblist);

    if (this_player()->query_get_echo())
 	write("You whisper: " + msg + "\n");
    else
    	write("Ok.\n");

    return 1;
}

int
whistle(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "appreciatively", 0);

    if (!strlen(how[0]))
    {
	write("You whistle" + how[1] + ".\n");
	all(" whistles" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Whistle at whom?\n");
	return 0;
    }

    target(" whistles" + how[1] + " at you.", oblist);
    actor("You whistle" + how[1] + " at", oblist);
    all2act("whistles" + how[1] + " at", oblist);
    return 1;
}

int
wiggle(string str)
{
    str = check_adverb_with_space(str, BLANK_ADVERB);

    if (str == NO_ADVERB_WITH_SPACE)
    {
	notify_fail("Wiggle what?\n");
	return 0;
    }

    write("You wiggle your bottom" + str + ".\n");
    allbb(" wiggles " + this_player()->query_possessive() +
	" bottom" + str + ".");
    return 1;
}

int
wink(string str)
{
    object *oblist;
    string *how;

    how = parse_adverb_with_space(str, "suggestively", 0);

    if (!strlen(how[0]))
    {
	write("You wink" + how[1] + ".\n");
	allbb(" winks" + how[1] + ".");
	return 1;
    }

    oblist = parse_this(how[0], "[at] [the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Wink at whom?\n");
	return 0;
    }

    targetbb(" winks" + how[1] + " at you.", oblist);
    actor("You wink" + how[1] + " at", oblist);
    all2actbb("winks" + how[1] + " at", oblist);
    return 1;
}

int
worship(string str)
{
    object *oblist;

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Worship whom?\n");
	return 0;
    }

    target(" falls on " + this_player()->query_possessive() +
	" knees and shows how much " + this_player()->query_pronoun() +
	" worships you.", oblist);
    all2act("falls on " + this_player()->query_possessive() +
	" knees in front of", oblist,
	((sizeof(oblist) == 1) ? "." : " in order."));
    actor("You worship", oblist, ((sizeof(oblist) == 1) ? "." : " in order."));
    return 1;
}

int
yawn(string str)
{
    object *oblist;

    if (!strlen(str))
    {
	write("My, what big teeth you have!\n");
	all(" yawns.");
	SOULDESC("yawning");
	return 1;
    }

    oblist = parse_this(str, "[the] %l");

    if (!sizeof(oblist))
    {
	notify_fail("Yawn at whom?\n");
	return 0;
    }

    SOULDESC("yawning");
    targetbb(" yawns at you. You must be boring to " +
        this_player()->query_objective() + ".", oblist);
    actor("You yawn at", oblist, " to show " +
        (sizeof(oblist) == 1 ? oblist[0]->query_objective() : "them") +
        " how much " +
        (sizeof(oblist) == 1 ? oblist[0]->query_pronoun() + " bores" :
        "they bore") + " you.");
    all2act("yawns at", oblist, " to show " +
        (sizeof(oblist) == 1 ? oblist[0]->query_objective() : "them") +
        " how boring " +
        (sizeof(oblist) == 1 ? oblist[0]->query_pronoun() + " is" :
        "they are") + ".");
    return 1;
}

int
yodel(string str)
{
    if (strlen(str))
    {
	notify_fail("Yodel what?\n");
	return 0;
    }

    write("Yodelii yodeluu!\n");
    all(" yodels a merry tune.");
    SOULDESC("yodeling");
    return 1;
}
