/*
 *
 * Below are the functions from the object tracer
 *
 * This is a general purpose tool.
 * It can be used to find objects, list info about them,
 * and walk up and down the inventory lists.
 */

#pragma save_binary

inherit "/cmd/std/tracer_tool_base";

#include "/sys/macros.h"
#include "/secure/std.h"

#define CHECK_SO_WIZ 	if (WIZ_CHECK < WIZ_NORMAL) return 0; \
			if (this_interactive() != this_player()) return 0
#define TRACER_STORES	"_tracer_stores"
#define TRACER_VARS	"_tracer_vars"

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return "tracer";
}

/* **************************************************************************
 * This is a tool soul.
 */
int
query_tool_soul()
{
    return 1;
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
		"Call":"Call",
		"Clean":"Clean",

		"Dump":"Dump",
		"Destruct":"Destruct",

		"Goto":"Goto",

		"In":"In",

		"Set":"Set",

		"Trans":"Trans"
	    ]);
}

/*
 * Function: print_value
 * Description: print out a value in a format appropriate for its type.
 */
void
print_value(mixed ret)
{
    if (intp(ret))
        write("(int) " + ret + "\n");
    else if (pointerp(ret))
    {
        write("Array of size " + sizeof(ret) + "\n");
        dump_array(ret);
    } else if (mappingp(ret)) {
        write("Mapping of size " + m_sizeof(ret) + "\n");
        dump_mapping(ret);
    } else if (stringp(ret))
        write("String: '" + ret + "'\n");
    else if (objectp(ret)) {
        write("Object: ");
        write(file_name(ret));
        write("\n");
    }
}
/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/

Dump(str)
{
    int             tmp, i, query_list, ret;
    object          ob;
    string          flag, *props,
                    path;
    mixed *stores, *vars;
    mixed *data;
    object *ob_list;

    CHECK_SO_WIZ;

    vars = this_interactive()->query_prop(TRACER_VARS);
    stores = this_interactive()->query_prop(TRACER_STORES);

    if (str == 0)
    {
	write("All variables:\n");
	while (i < sizeof(vars))
	{
	    if (vars[i])
	    {
		write(vars[i] + ":\t");
		write(stores[i]);
		write("\n");
	    }
	    i += 1;
	}
	return 1;
    }

    if (sscanf(str, "%s %s", path, flag) != 2)
	path = str;
    ob = parse_list(path);
    notify_fail("Dump what object?\n");
    if (ob == 0)
	return 0;

    write(file_name(ob) + ":\n");
    write(ob->short());

    if (living(ob))
	write("(living) ");

    if (tmp = query_ip_name(ob))
	write("(interactive) '" + tmp + "' ");

    write("\n");

    tmp = creator(ob);
    if (tmp)
	write("Creator:\t\t" + tmp + "\n");

    data = SECURITY->do_debug("get_variables", ob);
    vars = m_indexes(data);
    
    if (member_array(flag, vars) > 0)
    {
        write(flag + " : ");
        print_value(data[flag]);
        write("\n");
    }

    if (flag == "list" || flag == "list!")
    {
	ob_list = all_inventory(ob);
	for (i = 0; i < sizeof(ob_list); i++)
	    write((i + 1) + ":\t" + file_name(ob_list[i]) + "\t" +
		(string)ob_list[i]->short() + "\n");

        write("\n");
	return 1;
    }

    if (flag == "shadows" || flag == "shadows!")
    {
        while (ob = shadow(ob,0))
          write(file_name(ob) + "\n");
        write("\n");
 
        return 1;
    }

    if (flag == "vars" || flag == "variables" || 
        flag  == "vars!" || flag == "variables!")
    {
        for (i = 0; i < sizeof(vars); i++)
        {
            write(sprintf(" %-30s : ", vars[i]));
            print_value(data[vars[i]]);
        }
        write("\n");
        return 1;
    }

    if (flag == "inherit" || flag == "inherits" ||
        flag == "inherit!" || flag == "inherits!")
    {
        dump_array(SECURITY->do_debug("inherit_list", ob));

        write("\n");
        return 1;
    }

    if (flag == "props" || flag == "properties" || 
        flag == "props!" || flag == "properties!")
    {
	props = ob->query_props();
	for (i = 0; i < sizeof(props); i++)
	{
	    write(sprintf(" %-30s : ", props[i]));
	    print_value(ob->query_prop(props[i]));
	}
        write("\n");
	return 1;
    }

    if (flag == "flags" || flag == "flags!")
    {
        write(SECURITY->do_debug("object_info", 0, ob));
        write("\n");
        return 1;
    }

    if (flag == "profile" || flag == "profile!")
    {
        write(SECURITY->do_debug("getprofile", ob));
        write("\n");
        return 1;
    }
	
    if (flag == "info" || flag == "info!")
    {
        write(SECURITY->do_debug("object_info", 1, ob));
        write("\n");
        return 1;
    }
	
    if (flag == "cpu" || flag == "cpu!")
    {
        write("Object CPU usage: " + SECURITY->do_debug("object_cpu", ob));
        write("\n");
        return 1;
    }
	
    if (flag == "functions" || flag == "functions!")
    {
        data = SECURITY->do_debug("functionlist", ob);
        for (i = 1; i < sizeof(data); i++)   /* start at 1 to avoid .INIT */
            write("    " + data[i] + "\n");
        write("\n");
        return 1;
    }
    if (flag == "callouts" || flag == "callouts!")
    {
        data = SECURITY->do_debug("call_out_info");
        for (i = 0; i < sizeof(data);  i++)
            if (data[i][0]==ob)
            {
                write("Function  : " + data[i][1] + "\n");
                write("Time Left : " + data[i][2] + "\n");
                write("Arguments :\n");
                dump_array(data[i][3]);
                write("\n\n");
            }
        return 1;
    }
	
    return 1;
}

Destruct(str)
{
    object          ob;
    int dflag;
    string a;

    CHECK_SO_WIZ;

    notify_fail("Destruct what?.\n");
    if (!strlen(str))
	return 0;

    if (sscanf(str, "-D %s", a) == 1)
    {
	dflag = 1;
	str = a;
    }

    ob = parse_list(str);

    if (!ob)
	return 0;

    if (dflag)
	SECURITY->do_debug("destroy", ob);

    if (ob)
	ob->remove_object();

    if (ob)
	SECURITY->do_debug("destroy", ob);

    write("Ok.\n");
    return 1;
}

parse_arg(what)
{
    mixed    apa;

    apa = explode(what + "%%", "%%");
    apa = map(apa, "fix_one_arg", this_object());
    return apa;
}

mixed fix_one_arg(string what)
{
    int narg;
    mixed apa;

    if (sscanf(what,"({%s})",apa) == 1)
    {
	apa = explode(apa + ",", ",");
	apa = map(apa, "fix_one_arg", this_object());
	return apa;
    }
    if (sscanf(what,"%d",narg)==1)
	return narg;
    else
	return what;
}


Call(str)
{
    string          with,
                    what,
		    thing,
                    item;
    int             iwhat;
    object          ob;
    int             ret;

    CHECK_SO_WIZ;

    if (!str)
	return 0;
    if (sscanf(str, "%s %s %s", item, with, what) != 3)
    {
	if (sscanf(str, "%s %s", item, with) == 2)
	    iwhat = 0;
	else
	    return 0;
    }

    ob = parse_list(item);
    notify_fail("Call a function in what object?\n");
    if (!ob)
	return 0;

    if (!function_exists(with, ob))
    {
	write("Function '" + with + "' not defined by " +
	      file_name(ob) + "\n");
	write("It might exist in a shadow though, let's try.\n");
    }

    what = parse_arg(what);

    if (living(ob))
	thing = QTNAME(ob);
    else if (!(thing = ob->short()))
	thing = MASTER_OB(ob);

    say(QCTNAME(this_interactive()) + " @@call_message:" +
	file_name(this_object()) + "|" + with + "@@ " + thing + ".\n");

    if (sizeof(what) == 1)
	ret = call_other(ob, with, what[0]);
    else if (sizeof(what) == 2)
	ret = call_other(ob, with, what[0], what[1]);
    else if (sizeof(what) == 3)
	ret = call_other(ob, with, what[0], what[1], what[2]);
    else if (sizeof(what) == 4)
	ret = call_other(ob, with, what[0], what[1], what[2], what[3]);
    else if (sizeof(what) == 5)
	ret = call_other(ob, with, what[0], what[1], what[2], what[3], 
			 what[4]);
    else if (sizeof(what) == 6)
	ret = call_other(ob, with, what[0], what[1], what[2], what[3],
			 what[4], what[5]);
	
    if (intp(ret))
	write("Got int " + ret + "\n");
    else if (pointerp(ret))
    {
	write("Array of size " + sizeof(ret) + "\n");
	dump_array(ret);
    }
    else if (mappingp(ret))
    {
	write("Mapping of size " + m_sizeof(ret) + "\n");
	dump_mapping(ret);
    }
    else if (stringp(ret))
	write("String: '" + ret + "'\n");
    else if (objectp(ret))
    {
	write("Object: ");
	write(file_name(ret));
	write("\n");
    }
    assign("ret", ret);

    return 1;
}

string
call_message(string command)
{
    object pobj;
    string str;

    pobj = previous_object(-1);
    if (pobj->query_wiz_level())
	return "called '" + command + "' in";
    else if (extract(command, 0, 5) == "query_")
	return "checked the internals of";
    else
	return "patched the internals of";
}

Clean(str)
{
    object ob, o, n, *ob_list;
    int i;

    CHECK_SO_WIZ;

    if (!strlen(str))
	return 0;

    ob = parse_list(str);
    notify_fail("Clean what object?\n");
    if (!ob)
	return 0;
    ob_list = all_inventory(ob);

    for (i = 0; i < sizeof(ob_list); i++)
    {
	if (query_ip_number(ob_list[i]))
	    continue;
	write(ob_list[i]->query_name() + ": Destructed.\n");
	ob_list[i]->remove_object();
	if (ob_list[i])
	    SECURITY->do_debug("destroy", ob_list[i]);
    }
    write("Ok.\n");
    return 1;
}

Trans(str)
{
    object          mark;

    CHECK_SO_WIZ;

    if (!str)
	return 0;
    mark = parse_list(str);
    notify_fail("Trans what object?\n");
    if (!mark)
	return 0;
    if (mark->move(this_interactive()))
    {
	if (mark->move(environment(this_interactive())))
	{
	    write("Failure.\n");
	    return 1;
	}
    }

    write("Ok.\n");
    return 1;
}

Set(str)
{
    object          ob;
    string          item,
                    var;

    CHECK_SO_WIZ;

    if (!str)
	return 0;
    if (sscanf(str, "%s %s", var, item) != 2)
	return 0;
    ob = parse_list(item);
    if (!ob)
	return 0;

    assign(var, ob);
    return 1;
}


Goto(str)
{
    object          mark;

    CHECK_SO_WIZ;

    if (!str)
	return 0;
    mark = parse_list(str);
    notify_fail("Can't find that object.\n");

    if (!mark)
	return 0;

    say(QCTNAME(this_interactive()) + "*" +
	this_interactive()->query_mmsgout() + ".\n");
    this_interactive()->move(mark);
    say(QCNAME(this_interactive()) + "*" + this_interactive()->query_mmsgin() +
	".\n");

    write("Ok.\n");
    return 1;
}

In(str)
{
    string          path,
                    cmd;
    object          ob,
                    old_ob;

    CHECK_SO_WIZ;

    if (!str)
	return 0;
    notify_fail("What do you want to do where?\n");
    if (sscanf(str, "%s %s", path, cmd) != 2)
	return 0;

    ob = parse_list(path);
    if (!ob)
	return 0;

    old_ob = environment(this_interactive());
    this_interactive()->move(ob);
    this_interactive()->command(cmd);
    this_interactive()->move(old_ob);
    return 1;
}
