/*
 * wiz_cmd_apprentice.c
 *
 * This object holds the apprentice wizards commands.
 *
 */

#pragma strict_types
#pragma save_binary

inherit "/cmd/std/command_driver";
inherit "/lib/time";

#include "/cmd/std/tracer_tool_base.c";

#include <stdproperties.h>
#include <filepath.h>
#include <language.h>
#include <std.h>
#include <macros.h>
#include <ss_types.h>
#include <composite.h>
#include <formulas.h>
#include <composite.h>
#include <cmdparse.h>

#define MANHEAD "/doc/man/"
#define SMANHEAD "/doc/sman"
#define MAP_OBJ "/std/map/map"
#define PLAYER_AS_DIRPATH "_player_as_dirpath"
#define CHECK_SO_WIZ 	if (WIZ_CHECK < WIZ_APPRENTICE) return 0; \
			if (this_interactive() != this_player()) return 0

	string	make_list(string *arr, int numbering);
	string	convtime(int time);
	string	busy_string(int busy);
	string	notify_string(int hf);
	int	find_source(string path, string argv);
	int	domaininfo(string domain);
	int	listwiz(string type);
	int	index(string str, string letter);
	string	my_extract(string str, int from, int to);
	int	*get_memberlist(string domain);
        string  get_object_path(object ob);

/* **************************************************************************
 * Return a list of which souls are to be loaded.
 * The souls are listed in order of command search.
 */
string *
get_soul_list()
{
    return ({ WIZ_CMD_APPRENTICE });
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return WIZ_LEV_NAME(WIZLEV_APPRENTICE);
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "adjdesc":"adjdesc",
	     "allcmd":"allcmd",
#ifndef NO_ALIGN_TITLE
	     "altitle":"altitle",
#endif
	     "applications":"applications",
	     "apply":"apply",
	     "audience":"audience",

	     "bit":"bit",
	     "busy":"busy",

	     "calc":"calc",
	     "cat":"cat_file",
	     "cd":"cd",
	     "cbd":"combatdata",
	     "combatdata":"combatdata",
	     "cbs":"combatstat",
	     "combatstat":"combatstat",

	     "dirs":"dirs",

	     "emote":"emote",
	     ":":"emote",

	     "fetch":"fetch",
	     "finger":"finger",

	     "gd":"gd_info",
	     "gfinger":"gfinger",
	     "goto":"goto",
	     "gtell":"gtell",
	     "gwiz":"gwiz",
	     "gwize":"gwiz",

	     "home":"home",

	     "ls":"list_files",
	     "lman":"lman",
	     "localcmd":"localcmd",

	     "man":"man",
	     "map":"map_display",
	     "mapdomains":"mapdomains",
	     "mmap":"mmap",
	     "metflag":"metflag",
	     "more":"more_file",
	     "mudlist":"mudlist",

	     "newhooks":"newhooks",
	     "next":"next",
	     "notify":"notify",

	     "people":"people",
	     "popd":"popd",
	     "pushd":"pushd",
	     "pwd":"pwd",

	     "ranking":"ranking",
	     "regret":"regret",
	     "review":"review",
	     "rsupport":"rsupport",
	     "rwho":"rwho",

             "sanction":"sanction",
	     "second":"second",
	     "setmin":"set_m_in",
	     "setmout":"set_m_out",
	     "setmmin":"set_mm_in",
	     "setmmout":"set_mm_out",
	     "skillstat":"skillstat",
	     "sman": "sman",
	     "start":"start",
	     "stat":"stat",

	     "tail":"tail_file",
	     "tell":"tell",
	     "title":"title",
	     "tradestat":"tradestat",

	     "unmetflag":"unmetflag",

	     "wiz":"wiz",
	     "wize":"wize",
	     ]);
}

/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the
 * same order as in the function name list.
 * **************************************************************************/

/* **************************************************************************
 * adjdesc - display/add descriptional adjectives
 */
int
adjdesc(string str)
{
    string         *parts, *old;
    object          pl;

    CHECK_SO_WIZ;

    if (str)
    {
	parts = explode(str + " ", " ");
	if (pl = find_player(parts[0]))
	{
	    write((string)pl->query_name() + "'s 'nonmet name' is: " +
		  (string)pl->query_nonmet_name() + "\n");
	    return 1;
	}
	old = (string *)this_interactive()->query_adj(1);
	this_interactive()->remove_adj(old);
	if (!((int)this_interactive()->set_adj(parts)))
	{
	    notify_fail("Too many words (max 2) or too long string " +
		"(< 35 chars total).\n");
	    this_interactive()->set_adj(old);
	    return 0;
	}
    }

    write("Your 'nonmet name' is: " +
	(string)this_interactive()->query_nonmet_name() + "\n");

    return 1;
}


/* **************************************************************************
 * allcmd - List all commands available to a player
 */
int
allcmd(string soul)
{
    object ob;
    string *list, *soul_list, soul_id;
    int i, level, specific, other;

    CHECK_SO_WIZ;

    if (soul)
    {
	list = explode(soul + " ", " ");
	if (sizeof(list) > 1)
	{
	    soul = list[1];
	    specific = 1;
	}
	if (list[0] == "me")
	    ob = this_interactive();
	else
	{
	    ob = find_living(list[0]);
	    if (!ob && sizeof(list) > 1)
		return 0;
	    else if (!ob)
		specific = 1;
	}
    }
    if (!ob)
	ob = this_interactive();

    level = (int)SECURITY->query_wiz_level(geteuid(ob));
    if (!specific || soul == "base")
    {
	write("----- Base:\n");
	write(break_string(implode(ob->local_cmd(), ", "), 76) + "\n");
    }

    if (level)
    {
	soul_list = (string *)WIZ_SOUL(level)->get_soul_list();
	for (i = 0; i < sizeof(soul_list); i++)
	{
	    soul_id = (string)soul_list[i]->get_soul_id();
	    if (specific && soul != soul_id)
		continue;
	    write("----- " + capitalize(soul_id) + ":\n");
	    list = m_indexes(soul_list[i]->query_cmdlist());
	    if (sizeof(list) > 0)

		write(break_string(implode(list, ", "), 76) + "\n");
	}
    }

    soul_list = (string *)ob->query_tool_list();
    if (sizeof(soul_list))
    {
	for (i = 0; i < sizeof(soul_list); i++)
	{
	    soul_id = (string)soul_list[i]->get_soul_id();
	    if (specific && soul != soul_id)
		continue;
	    write("----- " + capitalize(soul_id) + ":\n");
	    list = m_indexes(soul_list[i]->query_cmdlist());
	    if (sizeof(list) > 0)
		write(break_string(implode(list, ", "), 76) + "\n");
	}
	}

    soul_list = (string *)ob->query_cmdsoul_list();
    if (sizeof(soul_list))
    {
	for (i = 0; i < sizeof(soul_list); i++)
	{
	    soul_id = (string)soul_list[i]->get_soul_id();
	    if (specific && soul != soul_id)
		continue;
	    write("----- " + capitalize(soul_id) + ":\n");
	    list = m_indexes(soul_list[i]->query_cmdlist());
	    if (sizeof(list) > 0)
		write(break_string(implode(list, ", "), 76) + "\n");
	}
    }

    if (list = (string *)ob->query_spell_verbs())
    {
	write("----- Spells:\n");
	write(break_string(implode(list, ", "), 76) + "\n");
    }
    return 1;
}

/* **************************************************************************
 * altitle - display/change the alignment title
 */
int
altitle(string t)
{
    CHECK_SO_WIZ;

    if (!strlen(t))
    {
	write("Your alignment title is '" +
		(string)this_interactive()->query_al_title() + "'.\n");
	return 1;
    }
    this_interactive()->set_al_title(t);
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * applications - display applications to a domain
 */
int
applications(string domain)
{
    CHECK_SO_WIZ;

    SECURITY->list_applications(domain);

    return 1;
}

/* **************************************************************************
 * apply - apply to a domain
 */
int
apply(string domain)
{
    CHECK_SO_WIZ;

    write(SECURITY->apply_to_domain(domain) + "\n");

    return 1;
}

/* **************************************************************************
 * audience - queue somone for talks.
 */
int
audience(string who)
{
    string *queue_list, name;
    int i;
    object ob, *obs, *objs;

    queue_list = (string *)this_interactive()->query_queue_list();
    if (!strlen(who))
    {
	if (!sizeof(queue_list))
	{
	    write("No one is requesting an audience.\n");
	    return 1;
	}
	for (i = 0; i < sizeof(queue_list); i++)
	    write((i + 1) + ": " + capitalize(queue_list[i]) + "\n");
	return 1;
    }

    if (who == "list")
    {
	obs = users();
	objs = ({ });
	for (i = 0; i < sizeof(obs); i++)
	{
	    if (member_array(this_interactive()->query_real_name(), 
			     obs[i]->query_queue_list()) != -1)
		objs += ({ obs[i] });
	}
	if (!sizeof(objs))
	{
	    write("You are not requesting an audience with anyone.\n");
	    return 1;
	}
	write("You are requesting an audience with :\n");
	for (i = 0; i < sizeof(objs); i++)
	    write(capitalize(objs[i]->query_real_name()) + "\n");
	return 1;
    }

    if (sscanf(who, "cancel %s", name) == 1) 
    {
	name = lower_case(name);
	ob = find_player(name);
	if (ob == this_interactive())
	{
	    write("You are not requesting an audience with yourself.\n");
	    return 1;
	}
	if (!ob)
	{
	    write("Couldn't locate " + name + ".\n");
	    return 1;
	}
	if (ob->pop_queue(this_interactive()->query_real_name()) == "")
	{
	    write("You are not requesting an audience with " +
		  capitalize(name) + ".\n");
	    return 1;
	}

	write("You cancel your request for audience with " + 
	      capitalize(name) + ".\n");
	tell_object(ob, this_interactive()->query_cap_name() +
		    " canceled " + this_interactive()->query_possessive() +
		    " request for audience with you.\n");
	return 1;
    }

    who = lower_case(who);
    ob = find_player(who);
    if (ob == this_interactive())
    {
	write("No, you're not availble for consultations to yourself. Ever.\n");
	return 1;
    }
    if (!ob)
    {
	write("Couldn't locate " + who + ".\n");
	return 1;
    }
    i = (int)ob->add_queue((string)this_interactive()->query_real_name());
    if (!i)
	write("You are already requesting an audience with " +
	    capitalize(who) + ".\n");
    else
    {
	write("You have been queued as number " + i + ".\n");
	tell_object(ob, (string)this_interactive()->query_cap_name() +
		" is requesting an audience with you.\n");
    }
    return 1;
}

/* **************************************************************************
 * bit - affect bits
 */
int
bit(string args)
{
    int argc, group, bit, i, j;
    string *argv, res;
    object ob;

    CHECK_SO_WIZ;

    if (!strlen(args))
    {
	write("Syntax error. Check with the help text.\n");
	return 1;
    }
    argv = explode(args, " ");
    argc = sizeof(argv);

    switch(argv[0])
    {
    case "set":
	if (argc != 4)
	{
	    write("Syntax error. Check with the help text.\n");
	    return 1;
	}
	if (!(ob = find_player(argv[1])))
	{
	    write("Can't locate " + capitalize(argv[1]) + ".\n");
	    return 1;
	}
	if (!sscanf(argv[2], "%d", group) || !(sscanf(argv[3], "%d", bit)))
	{
	    write("Syntax error. Check with the help text.\n");
	    return 1;
	}
	if (!((int)ob->set_bit(group, bit)))
	{
	    write("You were not allowed to set bit " + group + ":" + bit +
		" in " + capitalize(argv[1]) + ".\n");
	    return 1;
	}
	write("Set bit " + bit + " in " + capitalize(argv[1]) + ".\n");
	break;
    case "clear":
	if (argc != 4)
	{
	    write("Syntax error. Check with the help text.\n");
	    return 1;
	}
	if (!(ob = find_player(argv[1])))
	{
	    write("Can't locate " + capitalize(argv[1]) + ".\n");
	    return 1;
	}
	if (!sscanf(argv[2], "%d", group) || !(sscanf(argv[3], "%d", bit)))
	{
	    write("Syntax error. Check with the help text.\n");
	    return 1;
	}
	if (!((int)ob->clear_bit(group, bit)))
	{
	    write("You were not allowed to clear bit " + group + ":" + bit +
		" in " + capitalize(argv[1]) + ".\n");
	    return 1;
	}
	write("Cleared bit " + bit + " in " + capitalize(argv[1]) + ".\n");
	break;
    default:
	if (argc != 2)
	{
	    write("Syntax error. Check with the help text.\n");
	    return 1;
	}
	if (!(ob = find_player(argv[0])))
	{
	    write("Can't locate " + capitalize(argv[0]) + ".\n");
	    return 1;
	}
	res = "";
	for (i = 0; i < 5; i++)
	{
	    for (j = 0; j < 20; j++)
	    {
		if ((int)ob->test_bit(argv[1], i, j))
		    res += "" + i + ":" + j + " ";
	    }
	}
	if (!strlen(res))
	    res = argv[1] + " has no bits set in " +
		capitalize(argv[0]) + ".\n";
	else
	    res = argv[1] + " has set bits: " + res + "\n";
	write(res);
	break;
    }
    return 1;
}

/* **************************************************************************
 * busy - set busy status
 */
int
busy(string what)
{
    int i, busy;

    CHECK_SO_WIZ;

    busy = (int)this_interactive()->query_prop(WIZARD_I_BUSY_LEVEL);

    if (!strlen(what))
    {
	if (!busy)
	{
	    write("You're in an attentive state of mind.\n");
	    return 1;
	}

	write("Busy: " + busy_string(busy) + "\n");
	return 1;
    }

    if (what == "clear")
    {
	if (!busy)
	{
	    write("You are already in an attentive state of mind.\n");
	    return 1;
	}
	busy = 0;
    }
    else
    {
	for (i = 0; i < strlen(what); i++)
	{
	    switch(what[i])
	    {

	    case 'W':
		busy ^= 1;
		break;

	    case 'T':
		busy ^= 2;
		break;

	    case 'M':
		busy ^= 4;
		break;

	    case 'P':
		busy ^= 8;
		break;

	    case 'S':
		busy ^= 16;
		break;

	    case 'C':
		busy ^= 32;
		break;

	    case 'G':
		busy ^= 256;
		break;

	    case 'I':
		busy ^= 512;
		break;

	    case 'F':
		busy = 1023;
		break;

	    default:
		write("Strange busy state: " + extract(what, i, i) + "\n");
		break;
	    }
	}
    }
    this_interactive()->add_prop(WIZARD_I_BUSY_LEVEL, busy);

    write("Ok.\n");
    return 1;
}

string
busy_string(int busy)
{
    string bstring;

    bstring = "";

    if (busy & 1)
	bstring += "W";
    if (busy & 2)
	bstring += "T";
    if (busy & 4)
	bstring += "M";
    if (busy & 8)
	bstring += "P";
    if (busy & 16)
	bstring += "S";
    if (busy & 32)
	bstring += "C";
    if (busy & 128)
	bstring += "F";
    if (busy & 256)
	bstring += "G";
    if (busy & 512)
	bstring += "I";

    return bstring;
}

/* **************************************************************************
 * calc - calculate something (mainly from formulas.h
 */
int
calc(string str)
{
    string *words, str2;
    int a, b;

    if (!str)
    {
	notify_fail("Calculate what?\n");
	return 0;
    }

    words = explode(str, " ");
    if (sizeof(words) > 1)
	str2 = implode(words[1 .. sizeof(words)], " ");
    else
	str2 = "";
	
    switch (words[0])
    {
    case "hp":
	if (sscanf(str2, "%d", a) != 1)
	{
	    notify_fail("calc hp <con>\n");
	    return 0;
	}
	write("Con " + a + " -> " + F_MAX_HP(a) + " hp.\n");
	break;
    case "pen":
	if (sscanf(str2, "%d %d", a, b) != 2)
	{
	    notify_fail("calc pen <wep_pen> <skill>\n");
	    return 0;
	}
	write("Wep pen: " + a + " and skills: " + b + " -> " + F_PENMOD(a, b) + 
		" penetration.\n");
	break;
    case "ac":
	if (sscanf(str2, "%d", a) != 1)
  	{
	    notify_fail("calc ac <ac>\n");
	    return 0;
	}
	write("Armour ac: " + a + " -> " + F_AC_MOD(a) + " modified ac.\n");
	break;
    default:
	notify_fail("No such calculation.\n");
	return 0;
    }

    return 1;
}

/* **************************************************************************
 * cat - display the contents of a file
 */
int
cat_file(string path)
{
    CHECK_SO_WIZ;

    if (!path)
    {
	notify_fail("What file do you want to cat?\n");
	return 0;
    }

    path = FTPATH((string)this_interactive()->query_path(), path);

    if (!strlen(path))
    {
	notify_fail("Bad file name format.\n");
	return 0;
    }

    if (!cat(path))
    {
	notify_fail("No such file: " + path + "\n");
	return 0;
    }
    return 1;
}

/* **************************************************************************
 * cd - change current directory
 */
int
cd(string str)
{
    string          old_path,
                    current_path;
    object ob;

    CHECK_SO_WIZ;

    if (!strlen(str))
	str = (string)SECURITY->query_wiz_path(
	    (string)this_interactive()->query_real_name());

    current_path = (string)this_interactive()->query_path();
    old_path = current_path;

    current_path = FTPATH(old_path, str);

    if (!strlen(current_path))
    {
	if (!(ob = parse_list(str)))
	{
	    notify_fail("Bad pathname format: " + str + "\n");
	    return 0;
	}
	current_path = get_object_path(ob);
    }

    if (file_size(current_path) != -2)
    {
	if (!(ob = parse_list(str)))
	{
	    notify_fail("Not a directory: " + current_path + "\n");
	    return 0;
	}
	current_path = get_object_path(ob);
    }
    write(current_path + "\n");
    this_interactive()->set_path(current_path);

    return 1;
}

string
get_object_path(object ob)
{
    string obpath, *path, newpath;

    if (!objectp(ob)) return "/";

    obpath = file_name(ob);
    path = explode(obpath, "/");
    path = path[0..(sizeof(path)-2)];
    newpath = "/" + implode(path, "/");
    
    return newpath;
}


/* **************************************************************************
 * combatdata - Show combat statistics for a living object
 */
int
combatdata(string str)
{
    object live;

    CHECK_SO_WIZ;

    if (!str)
	live = this_player();
    else if (live = present(str, environment(this_player())));
    else if (live = find_player(str));
    else if (live = find_living(str));

    if (live)
	write(live->combat_data());
    else
    {
	notify_fail("No such living found: " + str + "\n");
	return 0;
    }
    return 1;
}

/* *************************************************************************
 * combatstat - Show some combat statistics for a living object
 */
int
combatstat(string str)
{
    object live;

    CHECK_SO_WIZ;

    if (!str)
        live = this_player();
    else if (live = present(str, environment(this_player())));
    else if (live = find_player(str));
    else if (live = find_living(str));

    if (live)
        write(live->combat_status());
    else
    {
        notify_fail("No such living found: " + str + "\n");
        return 0;
    }
    return 1;
}

/* **************************************************************************
 * dirs - display current pushed dirs
 */
int
dirs()
{
    int i;
    string *paths;

    CHECK_SO_WIZ;

    write(RPATH(this_player()->query_path()));
    paths = this_player()->query_prop(PLAYER_AS_DIRPATH);

    for (i = 0; i < sizeof(paths); i++)
	write(" " + paths[i]);

    write("\n");
    return 1;
}

/* **************************************************************************
 * emote, : - emote something
 */
int
emote(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Emote what?\n");
	return 0;
    }
    say(({ METNAME + " " + str + "\n", TART_NONMETNAME + " " + str + "\n",
	UNSEEN_NAME + " " + str + "\n" }));
    if (this_interactive()->query_get_echo())
	write("You emote: " + METNAME + " " + str + "\n");
    else
    	write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * finger - display information about player/domain/wiztype
 */
int
finger(string str)
{
    object  ob, ob3, *dlist;
    int    li, log_time, mw;
    string str1, str2, str3;
    string pron, level_name;
    mapping gread;
    string *wnames, *snames;

    CHECK_SO_WIZ;

    if (!str)
    {
	notify_fail("Syntax: 'finger name/Domain'\n");
	return 0;
    }

    if (ob = find_player(str))
    {
	write(ob->query_name() + " is currently in the game, Location: " +
	      file_name(environment(ob)) + "\n");
	li = 1;
    }
    else
    {
	ob = (object)SECURITY->finger_player(str);
	li = 0;
    }

    if (!ob)
    {
	if (domaininfo(str))
	    return 1;

	if (listwiz(str))
	    return 1;

	if (str == "domains")
	{
	    write(break_string(implode((string *)SECURITY->query_domain_list(),
		", ") + ".\n", 65, 0));
	    return 1;
	}

	if (str == "madwands" || str == "madwand")
	{
	    dlist = SECURITY->query_domain_list();
	    for (mw = 0, li = 0; li < sizeof(dlist); li++)
	    {
		if (stringp(str1 = SECURITY->query_domain_madwand(dlist[li])))
		{
		    mw = 1;
		    write(dlist[li] + ": " + capitalize(str1) + "\n");
		}
	    }
	    if (!mw)
		write("No Madwand drafted by any domain.\n");
	    return 1;
	}

	if (str == "global" || str == "globals" || str == "global read")
	{
	    gread = SECURITY->query_global_read();
	    wnames = m_indexes(gread);
	    if (!sizeof(wnames))
		write("There are no wizards with global read rights.\n");
	    {
		for (li = 0 ; li < sizeof(wnames) ; li++)
		    write(sprintf("%-10s added by %-10s", capitalize(wnames[li]), capitalize(gread[wnames[li]][0])) + " (" + gread[wnames[li]][1] + ")\n");
	    }
	    return 1;
	}

	notify_fail("No such player, domain or wiztype: " + str + "\n");
	return 0;
    }

    write(ob->long());
    pron = capitalize((string)ob->query_pronoun());
    level_name = WIZ_LEVEL_NAME(ob);

    write(pron + " is level " + SECURITY->query_wiz_level(ob->query_real_name()) + " (" +
	capitalize(level_name) + ").\n");
    str1 = lower_case((string)ob->query_real_name());

    str3 = (string)SECURITY->query_wiz_chl(str1);
    if (strlen(str3))
	write(pron + " was set to this level by " + capitalize(str3) + ".\n");

    str2 = (string)SECURITY->query_wiz_dom(str1);
    if (strlen(str2))
    {
        str3 = (string)SECURITY->query_domain_lord(str2);
	if (WIZ_TYPE(SECURITY->query_wiz_level()) == WIZ_LORD)
	    write(pron + " is " + WIZ_DOMAIN_RULER_NAME(ob) +" of the domain " +
		str2 + ".\n");
	else
	{
            if (strlen(str3))
	    {
		if (!(ob3 = find_player(str3)))
		    ob3 = (object)SECURITY->finger_player(str3);
                write(pron + " is a member of the domain " + str2 +
		      " with the " + WIZ_DOMAIN_RULER_NAME(ob3) +" " + capitalize(str3) +
		      ".\n");
	    }
	    else
		write(pron + " is a member of the domain " + str2 +
		      " with no lord.\n");
	}
	if (SECURITY->query_domain_madwand(str2) == str1)
	    write(pron + " is the Madwand of the " + str2 + " domain.\n");
    }

    str3 = (string)SECURITY->query_wiz_chd(str1);
    if (strlen(str3))
	write(pron + " was added to this domain by " + capitalize(str3) + ".\n");

    snames = ob->query_seconds();

    if (sizeof(snames))
	write(pron + " has the following seconds listed: " + implode(snames, " ") + ".\n");
    else
	write(pron + " has no listed seconds.\n");
    
    if (li)
    {
	write("Currently logged on from: " + ob->query_login_from() + "\n");
	if (query_ip_number(ob))
	{
	    if (query_idle(ob) > 0)
		write("Idle time: " + convtime(query_idle(ob)) + ".\n");
	}
	else
	    write(pron + " is link dead.\n");
    }
    else
    {
        ob->set_adj();   /* Fixa adjektiv */
        log_time = time() - (int)ob->query_login_time();
	str1 = convtime(log_time);

	write("Last login: " + str1 + " ago from " +
	      ob->query_login_from() + "\n");
    }

    write("Age: " + ob->show_age() +".\n");
    write("Email: " + ob->query_mailaddr() + "\n");
    ob->finger_info();
    if (!li)
	ob->remove_object();

    return 1;
}

int
domaininfo(string domain)
{
    string *warr, lord, result, madwand, clord;
    object ob;
    int nmem;

    domain = capitalize(domain);

    warr = SECURITY->query_domain_members(domain);
    if (!sizeof(warr))
	return 0;
    lord = SECURITY->query_domain_lord(domain);

    madwand = SECURITY->query_domain_madwand(domain);

    warr = map(warr, "cap_wiz", this_object());
    nmem = SECURITY->query_domain_max(domain) - sizeof(warr);

    if (strlen(lord))
    {
	if (!(ob = find_player(lord)))
	    ob = (object)SECURITY->finger_player(lord);
        clord = capitalize(lord);
	lord = clord + " is " + capitalize(WIZ_DOMAIN_RULER_NAME(ob));
    }
    else
    {
	lord = "The domain has no Lord";
	clord = "*No lord*";
    }

    if (sizeof(warr - ({ clord })))
    {
	result = "The domain " + domain + " consists of " + sizeof(warr) +
	    " wizards.\n";
	warr -= ({ clord });
	if (sizeof(warr) == 1)
	    result += lord + " and the only member is: " + warr[0] + ".\n";
	else
	    result += lord + " and the members are:\n" +
		implode(slice_array(warr, 0, sizeof(warr) - 2), ", ") +
		    " and " +  warr[sizeof(warr) - 1] + ".\n";
    }
    else
	result =  lord + " with no other members.\n";

    if (nmem == 1)
	write (result + "The domain has one vacancy.\n");
    else
	write (result + "The domain has " + (nmem > 0 ? ("" + nmem) : "no") +
	    " vacancies.\n");

    if (strlen(madwand))
	write(capitalize(madwand) + " is the Madwand of the domain.\n");
    else
	write("The domain has no Madwand.\n");

    result = SECURITY->query_wiz_path(domain) + "/open/finger_info";

    if (file_size(result) > 0)
	write("--------- Special domain supplied info:\n" +
	      read_file(result,1, 10) + "\n");

    return 1;
}

string
cap_wiz(string wiz)
{
    return capitalize(wiz);
}

int
listwiz(string type)
{
    int w_type;
    string *arr;

    type = LANG_SWORD(type);
    w_type = member_array(type, WIZ_T);
    if (w_type < 0)
	return 0;

    arr = (string *)SECURITY->query_wiz_list(WIZ_N[w_type]);
    if (!sizeof(arr))
    {
	notify_fail("No wizards of that type registered.\n");
	return 0;
    }
    arr = map(arr, "cap_wiz", this_object());
    write(break_string(implode(arr, ", ") + ".\n", 65, 0));
    return 1;
}

/* **************************************************************************
 * gd_info - Get some relevant Game driver information
 */
int
gd_info(string icmd)
{
    string inf, *p;
    object ob;
    int f;

    if (icmd)
    {
	inf = SECURITY->do_debug(icmd);
	if (!inf)
	{
	   if (sizeof(p = explode(icmd, " ")) > 1)
	   {
	       if (sizeof(p) > 2)
	       {
		   f = 0;
		   if (sscanf(p[1],"%d", f) != 1)
		       f = 0;
		   ob = parse_list(p[2]);
		   inf = SECURITY->do_debug(p[0], f, ob);
	       }
	       else
	       {
		   ob = parse_list(p[1]);
		   inf = SECURITY->do_debug(p[0], ob);
	       }
	   }
       }
    }
    else
    {
	write("The gd command must have an argument.\n");
	return 1;
    }

    if (stringp(inf))
	write(icmd + ":\n" + inf + "\n");
    else
	dump_array(inf);
    write("\n");
    return 1;
}

/* **************************************************************************
 * gfinger - finger showone at another mud
 */
int
gfinger(string str)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_gfinger(str);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * goto - teleport somewhere
 */
int
goto(string dest)
{
    object          ob,
                    wlx,
                    wly,
                    slx,
                    sly;
    string name;

    CHECK_SO_WIZ;

    if (!dest)
    {
	notify_fail("Goto where ?\n");
	return 0;
    }

    if (!(ob = find_player(dest))) ob = find_living(dest);
    if ((ob) && (environment(ob)))
    {
	ob = environment(ob);

	this_interactive()->move_living("X", ob);
	return 1;
    }

    if (sscanf(dest, "x%d.%dy%d.%d", wlx, slx, wly, sly) == 4)
    {
	dest = (string)"/std/room"->map_file(dest);
    }
    name = dest;
    dest = FTPATH((string)this_interactive()->query_path(), dest);

    if (!strlen(dest) || catch(this_interactive()->move_living("X", dest)))
    {
	notify_fail("Invalid monster name or file name: " + dest + "\n");
	return 0;
    }
    return 1;
}

/* **************************************************************************
 * gtell - tell showone at another mud
 */
int
gtell(string str)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_gtell(str);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * gwiz - intermud wizline
 */
int
gwiz(string str)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_gwiz(str, query_verb() == "gwize");
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * home - go home, to a domain workroom, or to an other wizards home
 */
int
home(string str)
{
    string n_home, w_name;

    CHECK_SO_WIZ;

    if (str == "admin")
	n_home = ADMIN_HOME;
    else
    {
	if (!strlen(str))
	    str = (string)this_interactive()->query_real_name();

	n_home = (string)SECURITY->wiz_home(str);
    }

    if(this_interactive()->move_living("X", n_home))
	write("There is no such workroom.\n");
    return 1;
}

/* **************************************************************************
 * ls - list the files in a directory
 */
int
list_files(string path)
{
    string *ppart, mode;
    int i, pindex;

    CHECK_SO_WIZ;

    mode = "";
    if (!strlen(path))
	path = ".";
    else
    {
	ppart = explode(path, " ");
	path = "";
	for (i = 0 ; i < sizeof(ppart) ; i++)
	{
	    if (ppart[i][0] == '-')
		mode += ppart[i][1..strlen(ppart[i])];
	    else
		path = ppart[i];
	}
	if (!strlen(path))
	    path = ".";
    }

    path = FTPATH((string)this_interactive()->query_path() + "/", path);

    ls(path, mode);

    return 1;
}

/* **************************************************************************
 * localcmd - list my available commands
 */
int
localcmd(string arg)
{
    return allcmd((arg ? arg : "me") + " base");
}

/* Support functions for man, sman, lman
*/
int g_mannum;

string man_num() { return sprintf("\n#%d:", g_mannum++); }

/* **************************************************************************
 * lman - Give information on 'local' source code
 */
varargs int
lman(string entry, string docdir)
{
    mixed *argv, *man_arr;
    int argc, i, num, flag;
    string *sdirarr, path, *p_parts, man_chapt, str;

    CHECK_SO_WIZ;

    if (!entry)
    {
        notify_fail("Syntax error, for instructions on usage, " +
	    "do 'help lman'.\n");
	return 0;
    }

    if (!strlen(docdir))
	docdir = this_player()->query_path();

    if (!SRCMAN->valid_docdir(docdir))
    {
        notify_fail("The directory: " + docdir +
		    " is not a valid documentation directory\n");
	return 0;
    }

    argv = explode(entry, " ");
    argc = sizeof(argv);

    switch(argv[0])
    {

        case "-k":

            if (argc == 2)
	    {
	        sdirarr = (string *)SRCMAN->get_subdirs(docdir);
		if (member_array(argv[1], sdirarr) < 0)
		{
		    for (i = 0 ; i < sizeof(sdirarr) ; i++)
		    {
			man_arr = SRCMAN->get_keywords(docdir, sdirarr[i],
							 argv[1])[1];
			if (sizeof(man_arr))
			{
			    write("--- " + sdirarr[i] + ":\n" +
				  sprintf("%-*#s\n", 76,
					  implode(man_arr, "\n")) + "\n");
			    flag = 1;
			}
		    }
		    if (!flag)
			write("No match.\n");
		}
		else
		{
		    man_chapt = argv[1];
		    man_arr = SRCMAN->get_index(docdir, man_chapt)[1];
		    this_interactive()->add_prop(WIZARD_S_SMAN_SEL_DIR,
						 man_chapt);
		    this_interactive()->add_prop(WIZARD_AM_SMAN_SEL_ARR,
						 man_arr);
		    g_mannum = 2;
		    str = process_string("#1:" +
					 implode(man_arr, "@@man_num@@"));
		    write("Available functions:\n" +
			  sprintf("%-*#s\n", 76, str));
		}
	    }

	    if (argc == 3)
	    {
	        if (member_array(argv[1], SRCMAN->get_subdirs(docdir)) < 0)
		{
		    write("No such subdir '" + argv[1] + "' available.\n");
		    break;
		}
		man_chapt = argv[1];
		man_arr = SRCMAN->get_keywords(docdir, man_chapt, argv[2])[1];
		this_interactive()->add_prop(WIZARD_S_SMAN_SEL_DIR,
					     man_chapt);
		this_interactive()->add_prop(WIZARD_AM_SMAN_SEL_ARR,
					     man_arr);
		g_mannum = 2;
		str = process_string("#1:" +
				     implode(man_arr, "@@man_num@@"));
		write("Available subjects:\n" +
		      sprintf("%-*#s\n", 76, str));
	    }
	    break;

	case "-?":
	    man_chapt = this_interactive()->
		query_prop(WIZARD_S_SMAN_SEL_DIR);
	    if (!man_chapt)
	    {
	        write("You haven't made any selection yet.\n" +
		    "Do 'lman -k <dir>' to select a directory.\n");
		break;
	    }
	    man_arr = this_interactive()->query_prop(WIZARD_AM_SMAN_SEL_ARR);

	    g_mannum = 2;
	    str = process_string("#1:" +
				 implode(man_arr, "@@man_num@@"));
	    write("Selected functions (" + man_chapt + "):\n" +
		  sprintf("%-*#s\n", 76, str));
	    break;

	case "-c":
	    write("Available subdirs:\n" +
		  sprintf("%-*#s\n", 76, implode(SRCMAN->get_subdirs(docdir),
						 "\n")));
	    break;

	case "-u":
	    SRCMAN->update_index(docdir);
	    write("Ok.\n");
	    break;

	case "-s":
	case "-e":
	    if (argc > 1)
	    {
		sscanf(argv[1], "#%d", num);
		if (num)
		{
		    man_chapt = this_interactive()->
			query_prop(WIZARD_S_SMAN_SEL_DIR);
		    man_arr = this_interactive()->
			query_prop(WIZARD_AM_SMAN_SEL_ARR);

		    if (!man_chapt)
		    {
			write("You haven't made any selection yet.\n");
			break;
		    }
		    if (num < 1 || num > sizeof(man_arr))
		    {
			write("The possible interval is 1-" +
			      sizeof(man_arr) + ".\n");
			break;
		    }
		    path = docdir + man_chapt + "/" + man_arr[num - 1];

		    if (!find_source(path, argv[0]))
		    {
			write("No such source code found.\n");
			break;
		    }
		    break;
		} else if (file_size(docdir +  argv[1]) < 0) {
		    sdirarr = SRCMAN->get_subdirs(docdir);
		    if (argc == 3)
		    {
			if (member_array(argv[1], sdirarr) < 0)
			{
			    write("No such subdir '" + argv[1] + 
				"' available.\n");
			    break;
			}
			man_chapt = argv[1];
			man_arr = SRCMAN->get_keywords(docdir, man_chapt,
							argv[2])[1];
		    } else {
			for (i = 0; i < sizeof(sdirarr); i++)
			{
			    man_chapt = sdirarr[i];
			    man_arr = SRCMAN->get_keywords(docdir, man_chapt,
							argv[1])[1];
			    if (sizeof(man_arr) > 0)
				break;
			}
		    }

		    if (sizeof(man_arr) == 0)
			write("No command: " + argv[1] + "\n");
		    else
		    {
			path = docdir + man_chapt + "/" + man_arr[0];

			if (file_size(path) < 0)
			    write("Function not found.\n");
			else
			    find_source(path, argv[0]);
		    }
		    break;
		} else {
		    find_source(docdir + argv[1], argv[0]);
		}
		break;
	    }

	default:
	    sscanf(argv[0], "#%d", num);
	    if (num)
	    {
	        man_chapt = this_interactive()->
		    query_prop(WIZARD_S_SMAN_SEL_DIR);
	        man_arr = this_interactive()->
		    query_prop(WIZARD_AM_SMAN_SEL_ARR);

	        if (!man_chapt)
		{
		    write("You haven't made any selection yet.\n");
		    break;
		}
	        if (num < 1 || num > sizeof(man_arr))
		{
		    write("The possible interval is 1-" +
			  sizeof(man_arr) + ".\n");
		    break;
		}
		path = docdir + man_chapt + "/" + man_arr[num - 1];

		write("File: " + path + "\n");
		this_interactive()->more(path, 1,
					 this_interactive()->
					 query_prop(PLAYER_I_MORE_LEN));
		break;
	    }

	    if (file_size(docdir + argv[0]) < 0)
	    {
	        sdirarr = (string *)SRCMAN->get_subdirs(docdir);
		if (argc == 2)
		{
		    if (member_array(argv[0], sdirarr) < 0)
		    {
			write("No such subdir '" + argv[0] +
			      "' available.\n");
			break;
		    }
		    man_chapt = argv[0];
		    man_arr = (mixed *)SRCMAN->get_keywords(docdir, man_chapt,
							     argv[1])[1];
		    this_interactive()->add_prop(WIZARD_S_SMAN_SEL_DIR,
						 man_chapt);
		    this_interactive()->add_prop(WIZARD_AM_SMAN_SEL_ARR,
						 man_arr);
		}
		else
		{
		    for (i = 0 ; i < sizeof(sdirarr) ; i++)
		    {
			man_chapt = sdirarr[i];
			man_arr = (mixed *)SRCMAN->get_keywords(docdir,
								man_chapt,
								argv[0])[1];
			if (sizeof(man_arr) > 0)
			    break;
		    }
		}
		if (sizeof(man_arr) == 0)
		    write("No command: " + argv[0] + "\n");
		else
		{
		    path = docdir + man_chapt + "/" + man_arr[0];

		    if (file_size(path) < 0)
			write("No command: " + argv[0] + "\n");
		    else
		    {
			write("File: " + path + "\n");
			this_interactive()->
			    more(path, 1, this_interactive()->
				 query_prop(PLAYER_I_MORE_LEN));
		    }
		}
		break;
	    }
	    else
	    {
		write("File: " + docdir + argv[0] + "\n");
		this_interactive()->
		    more(docdir + argv[0], 1,
			 this_interactive()->query_prop(PLAYER_I_MORE_LEN));
	    }
	    break;
    }

    return 1;
}

int
find_source(string docfile, string argv)
{
    string          docs,
                    str,
                    src;
    int             line;

    if (file_size(docfile) < 0)
	return 0;

    docs = read_file(docfile);
    if (sscanf(docs, "%d:%s %s", line, src, str) == 3)
    {
	write("File: " + src + " Line: " + line + "\n\n");
	if (argv == "-s")
	    this_interactive()->more(src, line,
		this_interactive()->query_prop(PLAYER_I_MORE_LEN));
	else if (WIZ_TYPE(this_interactive()->query_wiz_level()) < WIZ_NORMAL)
	    write("Only true wizards can use the -e option.\n");
	else
	    ed(src);

	return 1;
    }
    else
	return 0;
}

/* **************************************************************************
 * man - display a manual page
 */
int
man(string entry)
{
    mixed *argv, *man_arr;
    int argc, i, num, wr_flag;
    string *chaparr, path, *p_parts, man_chapt, str;

    CHECK_SO_WIZ;

    if (!entry)
    {
        notify_fail("Syntax error, for instructions on usage, do 'help man'.\n")
;
	return 0;
    }

    argv = explode(entry, " ");
    argc = sizeof(argv);

    switch(argv[0])
    {

        case "-k":

            if (argc == 2)
	    {
	        chaparr = (string *)MANCTRL->get_chapters();
		if (member_array(argv[1], chaparr) < 0)
		{
		    for (i = 0 ; i < sizeof(chaparr) ; i++)
		    {
			man_arr = MANCTRL->get_keywords(chaparr[i],
							 argv[1])[1];
			if (sizeof(man_arr))
			{
			    write("--- " + chaparr[i] + ":\n" +
				  sprintf("%-*#s\n", 76,
					  implode(man_arr, "\n")) + "\n");
			    wr_flag = 1;
			}
		    }
		    if (!wr_flag)
			write("No such chapter.\n");
		}
		else
		{
		    man_chapt = argv[1];
		    man_arr = MANCTRL->get_index(man_chapt)[1];
		    this_interactive()->add_prop(WIZARD_S_MAN_SEL_CHAPT,
						 man_chapt);
		    this_interactive()->add_prop(WIZARD_AM_MAN_SEL_ARR,
						 man_arr);
		    g_mannum = 2;
		    str = process_string("#1:" +
					 implode(man_arr, "@@man_num@@"));
		    write("Available subjects:\n" +
			  sprintf("%-*#s\n", 76, str));
		}
	    }
	    else if (argc == 3)
	    {
	        if (member_array(argv[1], MANCTRL->get_chapters()) < 0)
		{
		    write("No such chapter '" + argv[1] + "' available.\n");
		    break;
		}
		man_chapt = argv[1];
		man_arr = MANCTRL->get_keywords(man_chapt, argv[2])[1];
		this_interactive()->add_prop(WIZARD_S_MAN_SEL_CHAPT,
					     man_chapt);
		this_interactive()->add_prop(WIZARD_AM_MAN_SEL_ARR,
					     man_arr);
		g_mannum = 2;
		str = process_string("#1:" +
				     implode(man_arr, "@@man_num@@"));
		write("Available subjects:\n" +
		      sprintf("%-*#s\n", 76, str));

	    } else {
		write("Try 'man -c' to see possible chapters.\n");
	    }
	    break;

	case "-?":
	    man_chapt = this_interactive()->query_prop(WIZARD_S_MAN_SEL_CHAPT);
	    if (!man_chapt)
	    {
	        write("You haven't made any selection yet.\n");
		break;
	    }
	    man_arr = this_interactive()->query_prop(WIZARD_AM_MAN_SEL_ARR);

	    g_mannum = 1;
	    str = process_string("#1:" +
				 implode(man_arr, "@@man_num@@"));
	    write("Selected subjects (" + man_chapt + "):\n" +
		  sprintf("%-*#s\n", 76, str));
	    break;

	case "-c":
	    write("Available chapters:\n" +
		  sprintf("%-*#s\n", 76,
			  implode(MANCTRL->get_chapters(), "\n")));
	    break;

	case "-u":
	    if (find_object(MANCTRL))
		find_object(MANCTRL)->remove_object();
	    write("Ok.\n");
	    break;

	default:
	    sscanf(argv[0], "#%d", num);
	    if (num)
	    {
	        man_chapt = this_interactive()->
		    query_prop(WIZARD_S_MAN_SEL_CHAPT);
	        man_arr = this_interactive()->
		    query_prop(WIZARD_AM_MAN_SEL_ARR);

	        if (!man_chapt)
		{
		    write("You haven't made any selection yet.\n");
		    break;
		}
	        if (num < 1 || num > sizeof(man_arr))
		{
		    write("The possible interval is 1-" +
			  sizeof(man_arr) + ".\n");
		    break;
		}
		path = MANHEAD + man_chapt + "/" + man_arr[num - 1];

		write("File: " + path + "\n");
		this_interactive()->more(path, 1,
					 this_interactive()->
					 query_prop(PLAYER_I_MORE_LEN));
		break;
	    }

	    if (file_size(MANHEAD + argv[0]) < 0)
	    {
	        chaparr = (string *)MANCTRL->get_chapters();
		if (argc == 2)
		{
		    if (member_array(argv[0], chaparr) < 0)
		    {
			write("No such chapter '" + argv[0] +
			      "' available.\n");
			break;
		    }
		    man_chapt = argv[0];
		    man_arr = (mixed *)MANCTRL->get_keywords(man_chapt,
							     argv[1])[1];
		    this_interactive()->add_prop(WIZARD_S_MAN_SEL_CHAPT,
						 man_chapt);
		    this_interactive()->add_prop(WIZARD_AM_MAN_SEL_ARR,
						 man_arr);
		}
		else
		{
		    for (i = 0 ; i < sizeof(chaparr) ; i++)
		    {
			man_chapt = chaparr[i];
			man_arr = (mixed *)MANCTRL->get_keywords(man_chapt,
								 argv[0])[1];
			if (sizeof(man_arr) > 0)
			    break;
		    }
		    if (!sizeof(man_arr) && member_array(argv[0], chaparr) >= 0)
		    {
			man_arr = MANCTRL->get_index(argv[0])[1];
			g_mannum = 2;
			str = process_string("#1:" +
				implode(man_arr, "@@man_num@@"));
			write("Available subjects:\n" +
				sprintf("%-*#s\n", 76, str));
			return 1;
		    }
		}
		if (sizeof(man_arr) == 0)
		    write("No command: " + argv[0] + "\n");
		else
		{
		    path = MANHEAD + man_chapt + "/" + man_arr[0];

		    if (file_size(path) < 0)
			write("No command: " + argv[0] + "\n");
		    else
		    {
			write("File: " + path + "\n");
			this_interactive()->
			    more(path, 1, this_interactive()->
				 query_prop(PLAYER_I_MORE_LEN));
		    }
		}
		break;
	    }
	    else
	    {
		write("File: " + MANHEAD + argv[0] + "\n");
		this_interactive()->
		    more(MANHEAD + argv[0], 1,
			 this_interactive()->query_prop(PLAYER_I_MORE_LEN));
	    }
	    break;
    }

    return 1;
}


/* **************************************************************************
 * metflag - Set the metflag, a wizard can decide if met or unmet with everyone
 */
int metflag(string str)
{
    object pl;

    CHECK_SO_WIZ;

    if (!str)
	pl = this_player();
    else
	pl = find_player(str);

    if (pl)
    {
	pl->set_wiz_unmet(0);
	write("Ok.\n");
	return 1;
    }
    else
    {
	notify_fail("Set met-flag for who?\n");
	return 0;
    }
}

/* **************************************************************************
 * more - display the contents of a file
 */
int
more_file(string path)
{
    string *argv;
    int len;

    CHECK_SO_WIZ;

    if (!strlen(path))
    {
	notify_fail("More what file?\n");
	return 0;
    }

    argv = explode(path, " ");

    if (sizeof(argv) == 2)
    {
	if (argv[0] == "-l")
	{
	    sscanf(argv[1], "%d", len);
	    if (!len)
		len = 20;
	    this_interactive()->add_prop(PLAYER_I_MORE_LEN, len);
	    write("Ok.\n");
	    return 1;
	}
    }

    path = FTPATH((string)this_interactive()->query_path(), path);
    if (!path)
    {
	notify_fail("Bad file name format.\n");
	return 0;
    }
    if (file_size(path) < 0)
    {
	notify_fail("No such file: " + path + "\n");
	return 0;
    }
    this_interactive()->
	more(path, 0, (int)this_interactive()->query_prop(PLAYER_I_MORE_LEN));
    return 1;
}

/* **************************************************************************
 * mudlist - List the muds known to us
 */
int
mudlist(string arg)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_mudlist(arg);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * newhooks - get new command hooks
 */
int
newhooks(string str)
{
    object pl;

    CHECK_SO_WIZ;

    if (WIZ_CHECK < WIZ_ARCH || (!strlen(str)) ||
	(find_player(str) == this_interactive()))
    {
	write("Will update your command hooks.\n");
	this_interactive()->update_hooks();
    }
    else
    {
	pl = find_player(str);
	if (!pl)
	    pl = present(str, environment(this_interactive()));
	if (!pl || !living(pl))
	    pl = find_living(str);
	if (!pl)
	{
	    notify_fail("No such living object.\n");
	    return 0;
	}
	pl->update_hooks();
	write("Updated command hooks for " + str +
	      "(" + file_name(pl) + ").\n");
    }
    return 1;
}

/* **************************************************************************
 * next - auidence next person in the list
 */
int
next(string cmd)
{
    string name;
    object ob;

    name = (string)this_interactive()->pop_queue();
    if (!strlen(name))
    {
	write("Your audience list is empty.\n");
	return 1;
    }
    ob = find_player(name);
    if (!ob)
    {
	write("Couldn't locate " + capitalize(name) + ".\n");
	return 1;
    }

    if (cmd == "refuse")
    {
	tell_object(ob, (string)this_interactive()->query_cap_name() +
		    " has refused your request for an audience.\n");
	write("You have refused " + capitalize(name) + "'s request for an audience.\n");
    }
    else
    {
	tell_object(ob, (string)this_interactive()->query_cap_name() +
		    " has granted you an audience.\n");
	write("You have granted " + capitalize(name) + " an audience.\n");
    }
    return 1;
}

/* **************************************************************************
 * notify - Set whether wizard is to be notified of logins or not
 */
int
notify(string what)
{
    int i, nf;

    CHECK_SO_WIZ;

    nf = this_player()->query_notify();

    if (!strlen(what))
    {
        if (!nf)
        {
            write("You have the notifier turned off.\n");
	    return 1;
	}
	write("Your notification level is: " + notify_string(nf) + ".\n");
	return 1;
    }

    if (what == "clear" || what == "O" || what == "off")
    {
	if (!nf)
	{
	    write("The notifier was already shut off.\n");
	    return 1;
	}
	nf = 0;
    }
    else
    {
	for (i = 0; i < strlen(what); i++)
	{
	    switch (what[i])
	    {
	    case 'A':
		nf ^= 1;
		break;
	    case 'W':
		nf ^= 2;
		break;
	    case 'L':
		nf ^= 4;
		break;
	    case 'D':
		nf ^= 8;
		break;
	    case 'I':
		nf ^= 16;
		break;
	    case 'X':
		nf ^= 32;
		break;
	    case 'B':
		nf ^= 64;
		break;
	    default:
		write("Strange notify state: " + extract(what, i, i) + ".\n");
		break;
	    }
	}
    }

    this_interactive()->set_notify(nf);
    write("Ok.\n");
    return 1;
}

string
notify_string(int nf)
{
    string nstring;

    nstring = "";

    if (nf & 1)
	nstring += "A";
    if (nf & 2)
	nstring += "W";
    if (nf & 4)
	nstring += "L";
    if (nf & 8)
	nstring += "D";
    if (nf & 16)
	nstring += "I";
    if (nf & 32)
	nstring += "X";
    if (nf & 64)
	nstring += "B";

    return nstring;
}

/* **************************************************************************
 * people - list the current players
 */
#define P_NAME		0
#define P_AGE		1
#define P_BUSY		2
#define P_DOMAIN	3
#define P_IDLE		4
#define P_LOCATION	5
#define P_IPNR		6
#define P_IPNAME	7
#define P_RTITLE	8
#define P_RLEVEL	9
#define P_SNOOP_STR	10
#define P_SPECIES	11
#define P_MET		12
#define P_AVERAGE	13
#define P_DESC		14
#define P_NUM		15
int
people(string opts)
{
    object *list, tmpob;
    int i, j, a, *spcs;
    string tmp_name, tmp_str, item, *tmp_arr;
    mixed *contents;

    CHECK_SO_WIZ;

    /* Fix to take care of lousy initialization */
    contents = allocate(P_NUM);
    for (i = 0 ; i < P_NUM ; i++)
	contents[i] = ({});
    spcs = ({});

    if (!strlen(opts))
	opts = "rdail";

    list = users();
    sizeof(list);

    write("--- " + SECURITY->get_mud_name() + ": " + sizeof(list) +
	  (sizeof(list) == 1 ? " player" : " players"));

    for (i = 0, a = 0; i < sizeof(list) ; i++)
	if (query_idle(list[i]) >= 5 * 60)
	    a++;

    if (a)
	write(" (" + (sizeof(list) - a) + " active)");



    write(". " + SECURITY->do_debug("load_average") + "\n");

    if (index(opts, "L") >= 0)
	list = sort_array(list, "sort_level", this_object());

    if (index(opts, "A") >= 0)
	list = sort_array(list, "sort_name", this_object());

    spcs = allocate(strlen(opts) + 1);

    for (i = 0 ; i < sizeof(list) ; i++)
    {
	tmp_name = (string)list[i]->query_possessed();
	if (!strlen(tmp_name))
	    tmp_name = (string)list[i]->query_real_name();
	if (!strlen(tmp_name))
	    tmp_name = (string)list[i]->query_name();
	if (!strlen(tmp_name))
	    tmp_name = "unknown";
	contents[P_NAME] += ({ capitalize(tmp_name) });
	if (!strlen(list[i]->short()))
	    contents[P_NAME][i] = "(" + capitalize(tmp_name) + ")";
	spcs[0] = spcs[0] >
	    strlen(contents[P_NAME][i]) ? spcs[0] : strlen(contents[P_NAME][i]);

	for (j = 0 ; j < strlen(opts) ; j++)
	{
	    switch(opts[j])
	    {
	    case '-':
	    case 'L':
	    case 'A':
	    case ' ':
		break;

	    case 'a': /* age */
		a = (int)list[i]->query_age();

		if (a / 43200 > 0)
		{
		    item = "" + (a / 43200);
		    contents[P_AGE] += ({ item + " d" });
		}
		else if (a / 1800 > 0)
		{
		    item = "" + (a / 1800);
		    contents[P_AGE] += ({ item + " h" });
		}
		else
		{
		    item = "" + (a / 30);
		    contents[P_AGE] += ({ item + " m" });
		}

		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_AGE][i]) ?
		    spcs[j + 1] : strlen(contents[P_AGE][i]);
		break;

	    case 'b': /* busy */
		contents[P_BUSY] += ({ busy_string((int)list[i]->
		    query_prop(WIZARD_I_BUSY_LEVEL)) });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_BUSY][i]) ?
		    spcs[j + 1] : strlen(contents[P_BUSY][i]);
		break;

	    case 'd': /*domain */
		contents[P_DOMAIN] += ({ (string)SECURITY->
		    query_wiz_dom(tmp_name) });
		if (!strlen(contents[P_DOMAIN][i]))
		    contents[P_DOMAIN][i] = "---";
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_DOMAIN][i]) ?
		    spcs[j + 1] : strlen(contents[P_DOMAIN][i]);
		break;

	    case 'i': /* idle */
		if (query_idle(list[i]) >= 5 * 60)
		    contents[P_IDLE] += ({ "I" });
		else
		    contents[P_IDLE] += ({ "" });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_IDLE][i]) ?
		    spcs[j + 1] : strlen(contents[P_IDLE][i]);
		break;

	    case 'l': /* location */
		if (strlen((string)list[i]->query_possessed()))
		{
		    if (function_exists("possess", list[i]) == "/std/possob")
			contents[P_LOCATION] += ({ "Possessed by " +
		            capitalize((string)list[i]->query_demon()) + "." });
		    else
			contents[P_LOCATION] += ({ capitalize((string)list[i]->
			    query_real_name()) + "@" + (environment(list[i]) ?
			    RPATH(file_name(environment(list[i]))) :
			    "In the big black void.") });
		}
		else
		    contents[P_LOCATION] += ({ environment(list[i]) ?
		        RPATH(file_name(environment(list[i]))) :
		        "In the big black void." });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_LOCATION][i]) ?
		    spcs[j + 1] : strlen(contents[P_LOCATION][i]);
		break;

	    case 'm': /* met */
		contents[P_MET] += ({ ((list[i]->
		    query_met(this_interactive())) ? "M" : "") });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_MET][i]) ?
		    spcs[j + 1] : strlen(contents[P_MET][i]);
		break;

	    case 'n': /* ip-number */
		contents[P_IPNR] += ({ query_ip_number(list[i]) });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_IPNR][i]) ?
		    spcs[j + 1] : strlen(contents[P_IPNR][i]);
		break;

	    case 'N': /* ip-name */
		contents[P_IPNAME] += ({ query_ip_name(list[i]) });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_IPNAME][i]) ?
		    spcs[j + 1] : strlen(contents[P_IPNAME][i]);
		break;

	    case 'r': /* title rank */
		tmp_str = capitalize(WIZ_LEVEL_NAME(list[i]));
		contents[P_RTITLE] += ({ tmp_str });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_RTITLE][i]) ?
		    spcs[j + 1] : strlen(contents[P_RTITLE][i]);
		break;

	    case 'R': /* level rank */
		contents[P_RLEVEL] += ({"" + (int)list[i]->query_wiz_level()});
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_RLEVEL][i]) ?
		    spcs[j + 1] : strlen(contents[P_RLEVEL][i]);
		break;

	    case 's': /* snoop */
		if ((tmpob = SECURITY->query_snoop(list[i])))
		{
		    if (objectp(tmpob) && strlen(tmpob->query_real_name()))
			contents[P_SNOOP_STR] +=
			    ({ capitalize(tmpob->query_real_name()) });
		    else
			contents[P_SNOOP_STR] += ({ "S" });
		}
		else
		    contents[P_SNOOP_STR] += ({ "---" });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_SNOOP_STR][i]) ?
		    spcs[j + 1] : strlen(contents[P_SNOOP_STR][i]);
		break;

	    case 'S': /* Species */
		if (tmp_str = capitalize((string)list[i]->query_race()))
		    contents[P_SPECIES] += ({ tmp_str });
		else
		    contents[P_SPECIES] += ({ "---" });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_SPECIES][i]) ?
		    spcs[j + 1] : strlen(contents[P_SPECIES][i]);
		break;

	    case 'V': /* aVerage stat */
		contents[P_AVERAGE] += ({ list[i]->query_average_stat() + "" });
		spcs[j + 1] = spcs[j + 1] > strlen(contents[P_AVERAGE][i]) ?
		    spcs[j + 1] : strlen(contents[P_AVERAGE][i]);
		break;

	    case 'D': /* nonmet description */
		if (sizeof(tmp_arr = list[i]->query_adj(1)) > 0)
		    contents[P_DESC] += ({ implode(tmp_arr, " ") });
		else
		    contents[P_DESC] += ({ "---" });
                spcs[j + 1] = spcs[j + 1] > strlen(contents[P_DESC][i]) ?
                    spcs[j + 1] : strlen(contents[P_DESC][i]);
		break;

	    default: /* Strange... */
		break;
	    }
	}
    }

    for (i = 0 ; i < sizeof(list) ; i++)
    {
	item = sprintf("%-*s ", spcs[0], contents[P_NAME][i]);
	for (j = 0 ; j < strlen(opts) ; j++)
	{
	    switch(opts[j])
	    {
	    case '-':
	    case 'L':
	    case 'A':
	    case ' ':
		break;

	    case 'a': /* age */
		item += sprintf("%*s ", spcs[j + 1], contents[P_AGE][i]);
		break;

	    case 'b': /* busy */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_BUSY][i]);
		break;

	    case 'd': /*domain */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_DOMAIN][i]);
		break;

	    case 'i': /* idle */
		item += sprintf("%*s ", spcs[j + 1], contents[P_IDLE][i]);
		break;

	    case 'l': /* location */
		/* I remove this since it makes the normal people command have
		 * a lot of extra spacing when there are long path names for
		 * rooms. people -lr won't look as nice though, but I hope we
		 * can live with it.
		 *
		 * Nick
		 *
		item += sprintf("%-*s ", spcs[j + 1], contents[P_LOCATION][i]);
		 */
		item += sprintf("%-*s ", 1, contents[P_LOCATION][i]);
		break;

	    case 'm': /* met */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_MET][i]);
		break;

	    case 'n': /* ip-number */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_IPNR][i]);
		break;

	    case 'N': /* ip-name */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_IPNAME][i]);
		break;

	    case 'r': /* title rank */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_RTITLE][i]);
		break;

	    case 'R': /* level rank */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_RLEVEL][i]);
		break;

	    case 's': /* snoop */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_SNOOP_STR][i]);
		break;

	    case 'S': /* Species */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_SPECIES][i]);
		break;

	    case 'V': /* aVerage stat */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_AVERAGE][i]);
		break;

	    case 'D': /* description */
		item += sprintf("%-*s ", spcs[j + 1], contents[P_DESC][i]);
		break;

	    default: /* Strange... */
		item += "'" + extract(opts, j, j) + "' ";
		break;
	    }
	}
	write (item + "\n");
    }
    return 1;
}

int
sort_level(object a, object b)
{
    if (a == b)
	return 0;
    if (!a)
	return -1;
    if (!b)
	return 1;
    if (a->query_wiz_level() < b->query_wiz_level()) return -1;
    if (a->query_wiz_level() == b->query_wiz_level()) return 0;
    return 1;
}

int
sort_name(object a, object b)
{
    string sa, sb;

    if (a == b)
	return 0;
    if (!a)
	return -1;
    if (!b)
	return 1;
    sa = a->query_real_name();
    sb = b->query_real_name();
    if (sa == sb) return 0;
    if (!sa)
	return -1;
    if (!sb)
	return 1;
    if (sa < sb) return -1;
    return 1;
}

int
index(string str, string letter)
{
    return member_array(letter, explode(str, ""));
}

/* **************************************************************************
 * popd - pop a directory from the stack
 */
int
popd(string arg)
{
    string *paths;
    int which;

    if (strlen(arg))
	sscanf(arg, "+%d", which);

    paths = this_player()->query_prop(PLAYER_AS_DIRPATH);

    if (sizeof(paths) == 0)
    {
	notify_fail("Directory stack empty.\n");
	return 0;
    }
    which -= 1;

    if (which >= 0)
    {
	if (which > sizeof(paths))
	{
	    notify_fail("Directory stack not that deep.\n");
	    return 0;
	}

	this_player()->set_path(FTPATH(this_player()->query_path(),
	    paths[which]));
	paths -= ({ paths[which] });
	this_player()->add_prop(PLAYER_AS_DIRPATH, paths);
    }
    else
    {
	this_player()->set_path(FTPATH(this_player()->query_path(), paths[0]));
	paths -= ({ paths[0] });
	this_player()->add_prop(PLAYER_AS_DIRPATH, paths);
    }

    dirs();
    return 1;
}

/* **************************************************************************
 * pushd - push a directory to the stack
 */
int
pushd(string arg)
{
    string *paths, the_dir;
    int which;


    if (strlen(arg))
	sscanf(arg, "+%d", which);

    paths = this_player()->query_prop(PLAYER_AS_DIRPATH);
    if (!pointerp(paths))
	paths = ({ });

    which -= 1;

    if (which >= 0)
    {
	if (which > sizeof(paths))
	{
	    notify_fail("Directory stack not that deep.\n");
	    return 0;
	}

	the_dir = paths[which];
	paths -= ({ paths[which] });
	paths = ({ RPATH(this_player()->query_path()) }) + paths;
	this_player()->set_path(FTPATH(this_player()->query_path(), the_dir));
	this_player()->add_prop(PLAYER_AS_DIRPATH, paths);
    }
    else
    {
	if (strlen(arg) == 0)
	{
	    if (sizeof(paths) == 0)
	    {
		notify_fail("No other directory.\n");
		return 0;
	    }
	    the_dir = paths[0];
	    paths -= ({ paths[0] });
	    paths = ({ RPATH(this_player()->query_path()) }) + paths;
	    this_player()->set_path(FTPATH(this_player()->query_path(), the_dir));
	    this_player()->add_prop(PLAYER_AS_DIRPATH, paths);
	}
	else
	{
	    if (file_size(FTPATH(this_player()->query_path(), arg)) != -2)
	    {
		notify_fail("No such directory.\n");
		return 0;
	    }
	    paths = ({ RPATH(this_player()->query_path()) }) + paths;
	    this_player()->set_path(FTPATH(this_player()->query_path(), arg));
	    this_player()->add_prop(PLAYER_AS_DIRPATH, paths);
	}
    }

    dirs();
    return 1;
}

/* **************************************************************************
 * pwd - print working directory
 */
int
pwd(string str)
{
    CHECK_SO_WIZ;

    if (str == "-r")
	write(RPATH(this_interactive()->query_path()) + "\n");
    else
	write(this_interactive()->query_path() + "\n");
    return 1;
}

static mixed *old_rank = 0;
static int old_time = -1;
#define MIN_DIFF_TIME 300

/* **************************************************************************
 * ranking - Print a ranking list of the domains
 */
int
ranking(string dom)
{
    string *doms, sg;
    mixed *mems; /* Array of array of members */
    int il, q, c, s, cmd, aft;

    CHECK_SO_WIZ;

    if (!sizeof(old_rank) || (time() - old_time) > MIN_DIFF_TIME)
    {
	doms = SECURITY->query_domain_list();
	for (mems = ({}), il = 0; il < sizeof(doms); il++)
	{
	    mems += ({ ({ doms[il] }) +
			   SECURITY->query_domain_members(doms[il]) });
	}

	for (mems = ({}), il = 0; il < sizeof(doms); il++)
	{
	    q = SECURITY->query_domain_qexp(doms[il]);
	    c = SECURITY->query_domain_cexp(doms[il]);
	    cmd = SECURITY->query_domain_commands(doms[il]);
#ifdef DOMAIN_RANKWEIGHT_FORMULA
	    s = DOMAIN_RANKWEIGHT_FORMULA(q, c);
#else
	    s = 100 + (q / 25) + ((-c) / 10000);
#endif
	    mems += ({ ({ doms[il], (cmd * s) / 100,
			  s, cmd, q, c }) });
	}

	mems = sort_array(mems, "rank_sort", this_object());
    }
    else
	mems = old_rank;

    if (stringp(dom))
    {
	if (sscanf(dom, "%d", aft) != 1)
	{
	    dom = capitalize(dom);
	    for (aft = (sizeof(mems) - 5), il = 0; il < sizeof(mems); il++)
		if (dom == mems[il][0])
		    aft = il;
	}
    }


#define abs(x) ((x) < 0 ? -(x) : (x))
#define sgn(x) ((x) < 0 ? -1 : ((x) == 0) ? 0 : 1)

    write("Ranking of domains:     Rank#     Weight   Base Rank    Quest   Combat\n-------------------\n");

    for (il = sizeof(mems) -1; il >= 0; il--)
    {
	q = sizeof(mems) - il;
	if ((!stringp(dom) && (q < 11)) ||
	    (stringp(dom) && (abs(il - aft) < 3)) ||
	    dom == "All")
	{
	    sg = (mems[il][2] < 0 ? "-" + abs(mems[il][2] / 100) : " " +
		abs(mems[il][2] / 100));
	    write(sprintf("%2d: %-12s     %8d     %4s.%02d   %8d %8d %8d\n",
			  q,
			  mems[il][0], mems[il][1],
			  sg, abs(mems[il][2] % 100),
			  mems[il][3], mems[il][4], mems[il][5]));
	}
    }
    old_rank = mems;
    old_time = time();

    return 1;
}

int
rank_sort(mixed *elem1, mixed *elem2)
{
    if (elem1[1] < elem2[1]) return -1;
    if (elem1[1] == elem2[1]) return 0;
    return 1;
}

/* **************************************************************************
 * regret - regret an application to a domain
 */
int
regret(string dom)
{
    CHECK_SO_WIZ;

    write((string)SECURITY->regret_application(dom) + "\n");

    return 1;
}

/* **************************************************************************
 * review - review move messages
 */
int
review(string str)
{
    object tp;

    CHECK_SO_WIZ;

    if (!strlen(str))
	tp = this_interactive();
    else
	tp = find_player(str);

    if (!tp)
    {
	notify_fail("No such player: " + str +".\n");
	return 0;
    }

    write("mout:\t" + tp->query_m_out() +
	  "\nmin:\t" + tp->query_m_in() +
	  "\nmmout:\t" + tp->query_mm_out() +
	  "\nmmin:\t" + tp->query_mm_in() + "\n");
    return 1;
}

/* **************************************************************************
 * rsupport - show what another mud support
 */
int
rsupport(string str)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_support(str);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * rwho - show people on other muds
 */
int
rwho(string str)
{
    CHECK_SO_WIZ;
#ifdef UDP_MANAGER
    return UDP_MANAGER->cmd_rwho(str);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * sanction - sanction actions
 */
int
sanction(string who)
{
    mapping sanctioned, spec;
    int i, j, len, level, flag;
    string dom, lord, name, ps;
    string *s_index, *sp_index, *argv;

    CHECK_SO_WIZ;
    name = lower_case(((string)this_interactive()->query_real_name()));

    sanctioned = this_interactive()->query_sanctioned();
    if (!mappingp(sanctioned))
        sanctioned = ([]);
    len = m_sizeof(sanctioned);

    if (!who)
    {
        if (!len)
        {
            write("You haven't sanctioned anyone.\n");
            return 1;
        }
	s_index = m_indexes(sanctioned);
        for (i = 0 ; i < sizeof(s_index) ; i++)
            write(capitalize(s_index[i]) + ": " + sanctioned[s_index[i]] + "\n");
        return 1;
    }

    argv = explode(who, " ");
    dom = SECURITY->query_wiz_dom(name);
    level = SECURITY->query_wiz_level(name);
    lord = SECURITY->query_domain_lord(capitalize(argv[0]));

    switch(sizeof(argv))
    {
    case 1:
	/*
	 * Possible cases:	Removal of a personal sanctioning.
	 *			Listing of a domain sanctioning.
	 */

	/*
	 * Remove a personal sanctioning.
	 */
	if (strlen(sanctioned[argv[0]]) || argv[0] == "clear")
	{
	    if (argv[0] == "clear")
	    {
		sanctioned = ([]);
		write("You removed all your sanctionings.\n");
	    }
	    else
	    {
		sanctioned = m_delete(sanctioned, lower_case(argv[0]));
		write("You removed " + capitalize(argv[0]) + " from your sanction list.\n");
	    }
	    SECURITY->set_sanctioned(name, sanctioned);
	}
	/*
	 * List a domain sanctioning.
	 */
	else if (strlen(lord))
	{
	    sanctioned = SECURITY->query_domain_sanction(capitalize(argv[0]));
	    spec = SECURITY->query_special_sanction(argv[0]);
	    if (!m_sizeof(sanctioned) && !m_sizeof(spec))
		write("The domain " + capitalize(argv[0]) + " hasn't sanctioned anyone.\n");
	    else
	    {
		if (m_sizeof(sanctioned))
		{
		    s_index = m_indexes(sanctioned);
		    for (i = 0 ; i < sizeof(s_index) ; i++)
			write(capitalize(s_index[i]) + ": " + sanctioned[s_index[i]] + "\n");
		}

		if (m_sizeof(spec))
		{
		    s_index = m_indexes(spec);
		    for (i = 0 ; i < sizeof(s_index) ; i++)
		    {
			sp_index = m_indexes(spec[s_index[i]]);
			for (j = 0 ; j < sizeof(sp_index) ; j++)
			    write(capitalize(s_index[i]) + ": " + spec[s_index[i]][sp_index[j]] + " {" + sp_index[j] + "}\n");
		    }
		}
	    }
	}
	else 
	    write("You haven't sanctioned " + argv[0] + ".\n");
	break;

    case 2:
	/*
	 * Possible cases:	Adding of a personal sanctioning.
	 *			Removal of a domain sanctioning.
	 */

	/* 
	 * Add a personal sanctioning.
	 */
	flag = 0;
	if (SECURITY->query_wiz_level(lower_case(argv[0])))
	{
	    for (j = 0 ; j < strlen(argv[1]) ; j++)
	    {
		switch(argv[1][j])
		{
		case 'r':
		case 's':
		    break;

		case 'w':
		    if (WIZ_TYPE(SECURITY->query_wiz_level(argv[0])) < WIZ_NORMAL)
		    {
			write("The sanctioned player must be a full wizard.\n");
			flag = 1;
		    }
		    break;
		    
		default:
		    write("Illegal sanctioning: '" + argv[1][j..j] + "'\n");
		    flag = 1;
		    break;
		}
	    }
	    if (!flag)
	    {
		ps = sanctioned[lower_case(argv[0])];
		sanctioned += ([ lower_case(argv[0]):argv[1] ]);
		SECURITY->set_sanctioned(name, sanctioned);
		if (!strlen(ps))
		    write("You added " + capitalize(argv[0]) + " [" + argv[1] + "] to your sanction list.\n");
		else
		    write("You changed " + capitalize(argv[0]) + " [" + ps + "->" + argv[1] + "] in your sanction list.\n");
	    }
	}
	/* 
	 * Remove a domain sanctioning.
	 */
	else if (strlen(lord))
	{
	    if (lord != name && WIZ_TYPE(level) < WIZ_ARCH)
		write("You are not the lord of that domain!\n");
	    else
	    {
		if (argv[1] == "clear")
		    write("You removed all sanctionings in " + capitalize(argv[0]) + "'s sanction list.\n");
		else
		    write("You removed " + capitalize(argv[1]) + " from " + capitalize(argv[0]) + "'s sanction list.\n");
		SECURITY->set_domain_sanction(argv[1], "", argv[0]);
	    }
	}
	else
	    write("No such wizard in the game: " + capitalize(argv[0]) + ".\n");
	break;

    case 3:
	/*
	 * Possible cases:	Adding of a domain sanctioning.
	 */
	if (lord != name && WIZ_TYPE(level) < WIZ_ARCH)
	    write("You are not the lord of that domain!\n");
	else if (!strlen(lord))
	    write("The domain " + capitalize(argv[0]) + " does not exist.\n");
	else
	{
	    flag = 0;
	    for (j = 0 ; j < strlen(argv[2]) ; j++)
	    {
		switch(argv[2][j])
		{
		case 'r':
		case 'R':
		    break;

		case 'w':
		    if (WIZ_TYPE(SECURITY->query_wiz_level(argv[1])) < WIZ_NORMAL &&
			SECURITY->query_domain_number(capitalize(argv[1])) < 0)
		    {
			write("The sanctioned player must be a full wizard.\n");
			flag = 1;
		    }
		    break;

		case 'W':
		    if (WIZ_TYPE(SECURITY->query_wiz_level(argv[1])) < WIZ_NORMAL &&
			SECURITY->query_domain_number(capitalize(argv[1])) < 0)
		    {
			write("The sanctioned player must be a full wizard.\n");
			flag = 1;
		    }
		    if (WIZ_TYPE(level) >= WIZ_ARCH)
			break;

		    /* else fall into next case */
		    
		default:
		    write("Illegal sanctioning: '" + argv[2][j..j] + "'\n");
		    flag = 1;
		    break;
		}
	    }
	    if (!flag)
	    {
		sanctioned = SECURITY->query_domain_sanction(argv[0]);
		ps = sanctioned[lower_case(argv[1])];
		SECURITY->set_domain_sanction(argv[1], argv[2], argv[0]);
		if (!strlen(ps))
		    write("You added " + capitalize(argv[1]) + " [" + argv[2] + "] to " + capitalize(argv[0]) + "'s sanction list.\n");
		else
		    write("You changed " + capitalize(argv[1]) + " [" + ps + "->" + argv[2] + "] in " + capitalize(argv[0]) + "'s sanction list.\n");
	    }
	}
	break;

    case 4:
	/* 
	 * Possible cases: Arch adding special access.
	 */
	if (WIZ_TYPE(level) < WIZ_ARCH)
	    write("You are not priviliged to do this.\n");
	else if (!strlen(lord))
	    write("The domain " + capitalize(argv[0]) + " does not exist.\n");
	else
	{
	    flag = 0;
	    for (j = 0 ; j < strlen(argv[2]) ; j++)
	    {
		switch(argv[2][j])
		{
		case 'R':
		case 'W':
		    if (WIZ_TYPE(SECURITY->query_wiz_level(argv[1])) < WIZ_NORMAL &&
			SECURITY->query_domain_number(capitalize(argv[1])) < 0)
		    {
			write("The sanctioned player must be a full wizard.\n");
			flag = 1;
		    }
		    break;

		default:
		    write("Illegal sanctioning: '" + argv[2][j..j] + "'\n");
		    flag = 1;
		    break;
		}
	    }
	    if (!flag)
	    {
		if (argv[3] == "clear")
		{
		    SECURITY->set_special_sanction(argv[1], argv[2], argv[0], argv[3]);
		    write("You removed " + capitalize(argv[1]) + "'s special sanctioning in " + capitalize(argv[0]) + "'s sanction list.\n");
		    break;
		}

		spec = SECURITY->query_special_sanction(argv[0]);
		if (!mappingp(spec[lower_case(argv[1])]) ||
		    !stringp(spec[lower_case(argv[1])][argv[3]]))
		    write("You added " + capitalize(argv[1]) + " [" + argv[2] + "] {" + argv[3] + "} to " + capitalize(argv[0]) + "'s sanction list.\n");
		else
		{
		    ps = spec[lower_case(argv[1])][argv[3]];
		    if (ps != argv[2])
			write("You changed " + capitalize(argv[1]) + " [" + ps + "->" + argv[2] + "] {" + argv[3] + "} in " + capitalize(argv[0]) + "'s special sanction list.\n");
		    else
			write("You removed " + capitalize(argv[1]) + " {" + argv[3] + "} in " + capitalize(argv[0]) + "'s sanction list.\n");
		}
		SECURITY->set_special_sanction(argv[1], argv[2], argv[0], argv[3]);
	    }
	}
	break;

    default:
        write("Strange number of arguments.\n");
        break;
    }
    return 1;
}

/* **************************************************************************
 * second - note someone as second.
 */
int
second(string what)
{
    string *args;
    int rval = 1;

    if (!strlen(what))
	what = "l l";

    args = explode(what, " ");

    if (sizeof(args) == 1)
	args = ({ "l" }) + args;

    switch (args[0])
    {
    case "a":
    case "add":
	rval = this_player()->add_second(args[1]);
	break;

    case "r":
    case "remove":
	rval = this_player()->remove_second(args[1]);
	break;

    case "l":
	write("Currently listed seconds: " + implode(this_player()->query_seconds(), " ") + "\n");
	break;
    }

    if (rval)
	write("Ok.\n");
    else
	write("Operation failed, either the named second was not mortal or did not exist.\n");
    return 1;
}

/* **************************************************************************
 * setmout - set move out message.
 */
int
set_m_out(string m)
{
    CHECK_SO_WIZ;

    this_interactive()->set_m_out(m);
    return 1;
}

/* **************************************************************************
 * setmin - set move in message.
 */
int
set_m_in(string m)
{
    CHECK_SO_WIZ;

    this_interactive()->set_m_in(m);
    return 1;
}

/* **************************************************************************
 * setmmout - set teleport out message.
 */
int
set_mm_out(string m)
{
    CHECK_SO_WIZ;

    this_interactive()->set_mm_out(m);
    return 1;
}

/* **************************************************************************
 * setmmin - set teleport in message.
 */
int
set_mm_in(string m)
{
    CHECK_SO_WIZ;

    this_interactive()->set_mm_in(m);
    return 1;
}

/* **************************************************************************
 * sman - source manual, give information on functions in the standard mudlib
 *
 * Working on this 920602 /JnA
 */
int
sman(string entry)
{
    CHECK_SO_WIZ;

    if (!entry)
    {
        notify_fail("Syntax error, for instructions on usage, do 'help sman'.\n"
);
	return 0;
    }

    return lman(entry, "/doc/sman");
}

/* **************************************************************************
 * start - set start point.
 */
int
start(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	write("Your default starting location: " +
	    (string)this_interactive()->query_default_start_location() + "\n");
	return 1;
    }

    if (str == "valid")
    {
	str = implode((mixed *) SECURITY->query_list_def_start(), ", ");
	write("Available starting locations:\n" + break_string(str, 76, 3)
	      + "\n");
	return 1;
    }

    if (str == "here")
	str = file_name(environment(this_interactive()));

    if (!this_interactive()->set_default_start_location(str))
    {
	write("<" + str + "> is not a valid starting location.\n");
	return start("-valid");
    }
    write("Ok.\n");
    return 1;
}

#define STAT_PLAYER      0x1
#define STAT_LIVING      0x2
#define STAT_OBJECT      0x4
#define LEFT(y, x) write((w = "" + (y)ob->x()) + extract(half, 0, 24 - strlen(w)))
int show_stat(object ob, int what);
int show_skills(object ob);
/* **************************************************************************
 * stat - displays stats of something
 */
int
stat(string str)
{
    object ob;

    CHECK_SO_WIZ;

    if (!strlen(str))
	return show_stat(this_interactive(), STAT_PLAYER | STAT_LIVING);

    if (ob = find_player(str))
	return show_stat(ob, STAT_PLAYER | STAT_LIVING);

    if ((ob = parse_list(str)) ||
	(ob = present(str, environment(this_interactive()))))
    {
	if (living(ob))
	    return show_stat(ob, STAT_LIVING);
	return show_stat(ob, STAT_OBJECT);
    }

    if (ob = find_living(str))
    {
	return show_stat(ob, STAT_LIVING);
    }

    if (ob = present(str, this_interactive()))
	return show_stat(ob, STAT_OBJECT);

    if (ob = SECURITY->finger_player(str))
	return show_stat(ob, STAT_PLAYER | STAT_LIVING);

    notify_fail("No such thing.\n");
    return 0;
}

int
skillstat(string str)
{
    object ob;

    CHECK_SO_WIZ;

    if (!strlen(str))
	return show_skills(this_interactive());

    if (ob = find_player(str))
	return show_skills(ob);

    if (ob = present(str, environment(this_interactive())))
    {
	if (living(ob))
	    return show_skills(ob);
    }
    if (ob = find_living(str))
    {
	return show_skills(ob);
    }
    notify_fail("No such living thing.\n");
    return 0;
}

int
show_skills(object ob) {
    string str;

    str = "";
    str += ob->skill_living();
    write(str);
    return 1;
}

int
show_stat(object ob, int what)
{
    string tmp, str;

    str = "";
    /*
    if (what & STAT_PLAYER)
	str += ob->stat_playervars();
     */
    if (what & STAT_LIVING)
	str += ob->stat_living();
    if (what & STAT_OBJECT)
	str += ob->stat_object();
    write(str);
    return 1;
}

/* **************************************************************************
 * tail - display the end of a file.
 */
int
tail_file(string path)
{
    CHECK_SO_WIZ;

    if (!path)
    {
	notify_fail("What file do you want to get the tail of.\n");
	return 0;
    }

    path = FTPATH((string)this_interactive()->query_path(), path);
    if (!strlen(path))
    {
	notify_fail("Bad file name format.\n");
	return 0;
    }
    if (!tail(path))
    {
	notify_fail("No such file: " + path + "\n");
	return 0;
    }
    return 1;
}

/* **************************************************************************
 * title - change the title
 */
int
title(string t)
{
    CHECK_SO_WIZ;

    if (!strlen(t))
    {
	write("Your title is '" + (string)this_interactive()->query_title() +
	    "'.\n");
	return 1;
    }
    this_interactive()->set_title(t);
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * tell - tell something to someone
 */
int
tell(string str)
{
    object ob, ti;
    string who, it, msg, pron;
    int busy;

    CHECK_SO_WIZ;

    if (!strlen(str) || sscanf(str, "%s %s", who, msg) != 2)
    {
	notify_fail("Tell what ?\n");
	return 0;
    }
    it = lower_case(who);
    ob = find_living(it);
    ti = this_interactive();
    if (!ob)
    {
	notify_fail("No player with that name.\n");
	return 0;
    }
    if (!query_ip_number(ob))
    {
	notify_fail(QCTNAME(ob) + " is link dead right now.\n");
	return 0;
    }
    busy = (int)ob->query_prop(WIZARD_I_BUSY_LEVEL);
    if (busy & 8 || busy & 16 || busy & 128)
    {
	pron = (string)ob->query_pronoun();
	write(capitalize(pron) +
	    " seems to be busy at the moment. If you want to be contacted\n" +
	    "when " + pron +
	    " is available again, use the 'audience' command.\n");
	return 1;
    }

    if (!(busy & 128))
    {
        ob->add_prop("_reply_wizard", ti->query_real_name());
	if (ob->query_wiz_level())
	    tell_object(ob, capitalize(ti->query_real_name()) +
			" tells you: " + msg + "\n");
	else if (ti->check_seen(ob))
	{
	    pron = capitalize(ti->query_pronoun());
	    if (environment(ob) != environment(ti))
	    {
		ob->catch_msg("An apparition of " + QNAME(ti) +
			  " appears.\n" + pron +
			  " tells you: " + msg + "\n" + pron +
			  " then disappears.\n", ti);
	    }
	    else
	    {
		ob->catch_msg(QCTNAME(ti) + " tells you: " + msg + "\n", ti);
	    }
	} else
	    ob->catch_msg("A blurred apparition of someone appears.\n" +
		"It tells you: " + msg + "\n" + "Then it disappears.\n");
    }

    if (ti->query_get_echo())
	ti->catch_msg("You tell " + QNAME(ob) + ": " + msg + "\n");
    else
        write("Ok.\n");

    return 1;
}

/* **************************************************************************
 * tradestat - Stat the trade in a room
 */
int
tradestat()
{
    if (!function_exists("stat_trade", environment(this_player())))
    {
	notify_fail("No obvious trade in this room.\n");
	return 0;
    }

    write(environment(this_player())->stat_trade());
    return 1;
}

/* **************************************************************************
 * unmetflag - Set the unmetflag, a wizard can decide: met/unmet with everyone
 */
int
unmetflag(string str)
{
    int flag;

    CHECK_SO_WIZ;

    flag = 1;
    if (str == "npc")
	flag = 2;

    this_player()->set_wiz_unmet(flag);
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * wiz - gozzip on the wizline
 */
int
wiz(string str)
{
    object          *list;
    int             i, busy;
    string          name;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Wiz what?\n");
	return 0;
    }

    name = capitalize((string) this_interactive()->query_real_name());
    list = filter(users(), "is_wiz", this_object());
    str = implode(explode(str, "@@"), "#");
    for (i = 0; i < sizeof(list); i++)
    {
	busy = (int)(list[i]->query_prop(WIZARD_I_BUSY_LEVEL));
	if (!((busy & 1) || (busy & 128)))
	{
	    if (list[i] == this_interactive())
	    {
		if (this_interactive()->query_get_echo())
	    	    list[i]->catch_msg("@ " + name + ": " + str + "\n");
	    	else
		    write("Ok.\n");
	    } else
		list[i]->catch_msg("@ " + name + ": " + str + "\n");
	}
    }

    return 1;
}

int
is_wiz(object ob)
{
    return (int)ob->query_wiz_level();
}

/* ***************************************************************************
 * wize - Emote on the wizline
 */
int
wize(string str)
{
    object          *list;
    int             i, busy;
    string          name;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
        notify_fail("Wize what?\n");
        return 0;
    }

    name = capitalize((string) this_interactive()->query_real_name());
    list = filter(users(), "is_wiz", this_object());
    str = implode(explode(str, "@@"), "#");
    for (i = 0; i < sizeof(list); i++)
    {
        busy = (int)(list[i]->query_prop(WIZARD_I_BUSY_LEVEL));
        if (!((busy & 1) || (busy & 128)))
	{
	    if (list[i] == this_interactive())
	    {
		if (this_interactive()->query_get_echo())
		    list[i]->catch_msg("@ " + name + " " + str + "\n");
		else
		    write("Ok.\n");
	    } else
                list[i]->catch_msg("@ " + name + " " + str + "\n");
	}
    }

    return 1;
}

/* **************************************************************************
 * mapdomains - Give information on the domains linked to the map
 */
int
mapdomains(string arg)
{
    string *domains;
    mixed link;
    int il, f;

    CHECK_SO_WIZ;

    domains = SECURITY->query_maplink_domains();

    for (f = 0, il = 0; il < sizeof(domains); il++)
    {
	link = SECURITY->query_maplink(domains[il]);
	if (sizeof(link) == 3)
	{
	    write(domains[il] + " (" + link[0] + "):\n" +
		  break_string((link[1] ? "   " + link[1] + ", " : "") +
			       implode(link[2], ", "), 70, 3) + "\n");
	    f = 1;
	}
    }
    if (!f)
	write("No domains properly linked to the map.\n");
    return 1;
}

int
mmap(string arg) {
    int x, y, radius;

    CHECK_SO_WIZ;

    if (!arg)
	return 0;

    if (sscanf(arg, "%d %d %d", x, y, radius) != 3)
    {
	notify_fail("wrong number of arguments\n");
	return 0;
    }

    MAP_OBJ->draw_area_around(x, y, radius);
    return 1;
}

int
map_display(string arg)
{
    int x, y, range, y_width;

    CHECK_SO_WIZ;

    if (!arg)
	return 0;

    if (sscanf(arg, "%d %d %d %d", x, y, range, y_width) != 4 &&
       sscanf(arg, "%d %d %d", x, y, range) != 3)
    {
	notify_fail("wrong number of arguments\n");
	return 0;
    }

    if (range < 1 || range > 100 || y_width > 100)
    {
	notify_fail("Illegal range\n");
	return 0;
    }

    if (!y_width)
	y_width = range;

    MAP_OBJ->draw_area_from_corner(x, y, range, y_width);
    return 1;
}

int
fetch(string arg)
{
    int x, y;

    CHECK_SO_WIZ;

    if (!arg)
	return 0;

    if (sscanf(arg, "%d %d", x, y) != 2)
    {
	notify_fail("wrong number of arguments\n");
	return 0;
    }

    MAP_OBJ->get_one_square(x, y);
    return 1;
}

