/*
 * wiz_cmd_arch.c
 *
 * This object holds the commands reserved for archwizards. (level 30+)
 *
 */
#pragma save_binary

inherit "/cmd/std/command_driver";

#include <macros.h>
#include <log.h>
#include <stdproperties.h>
#include "/secure/std.h"

#define CHECK_SO_ARCH 	if (WIZ_CHECK < WIZ_ARCH) return 0; \
			if (this_interactive() != this_player()) return 0

/* **************************************************************************
 * Return a list of which souls are to be loaded.
 * The souls are listed in order of command search.
 */
string *
get_soul_list()
{
    return ({ WIZ_CMD_ARCH,
	      WIZ_CMD_LORD,
	      WIZ_CMD_NORMAL,
	      WIZ_CMD_PILGRIM,
	      WIZ_CMD_APPRENTICE });
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return WIZ_LEV_NAME(WIZLEV_ARCH);
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "all_spells":"all_spells",
	     "arch":"arch",

	     "block":"block",

	     "draft":"draft",

	     "global":"global",

	     "lockout":"lockout",

	     "mkdomain":"mkdomain",
	     "mudstatus":"mudstatus",

	     "pingmud":"pingmud",

	     "rmapp":"rmapp",
             "rmdomain":"rmdomain",

	     "startloc":"startloc",
	     "storemuds":"storemuds",

	     "trace":"set_trace",
	     "traceprefix":"set_traceprefix",

	     "xpclear":"xpclear",
	 ]);
}


/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/

/* **************************************************************************
 * all_spells - list all active spells
 */
int
all_spells()
{
    CHECK_SO_ARCH;

    SECURITY->list_spells();
    
    return 1;
}

/* **************************************************************************
 * arch - send a message on the archline
 */
int
arch(string str)
{
    object          *list;
    int             i, busy;
    string          name;

    CHECK_SO_ARCH;

    if (!strlen(str))
    {
	notify_fail("Arch what?\n");
	return 0;
    }

    name = capitalize((string) this_interactive()->query_real_name());
    list = filter(users(), "is_arch", this_object());
    str = implode(explode(str, "@@"), "#");
    for (i = 0; i < sizeof(list); i++)
    {
	if (list[i] == this_interactive())
	{
	    if (this_interactive()->query_get_echo())
	    	list[i]->catch_msg("@Arch@ " + name + ": " + str + "\n");
	    else
		write("Ok.\n");
	} else
	    list[i]->catch_msg("@Arch@ " + name + ": " + str + "\n");
    }

    return 1;
}

int
is_arch(object ob)
{
    return ob->query_wiz_level() > 40;
}

/* **************************************************************************
 * block - arch managing of sites that are totally blocked
 */
int
block(string str)
{
    CHECK_SO_ARCH;
    
    if (!strlen(str))
    {
	notify_fail("Block what?\n");
	return 0;
    }
    return (int)SECURITY->block_site(str);
}

/* **************************************************************************
 * draft - add someone to a domain
 */
int
draft(string str)
{

    string          pname,
                    dname,
                    err;

    CHECK_SO_ARCH;

    if (!strlen(str) || sscanf(str, "%s %s", dname, pname) != 2)
    {
	notify_fail("Usage: draft Domain player\n");
	return 0;
    }

    write(SECURITY->domain_add_wizard(dname, pname) + "\n");

    return 1;
}


/* **************************************************************************
 * global - add/remove people from the global list
 */
int
global(string str)
{
    string *cmds, *wnames, *comment;
    int i;
    mapping gread;

    CHECK_SO_ARCH;

    if (!strlen(str))
    {
	gread = SECURITY->query_global_read();
	wnames = m_indexes(gread);
	if (!sizeof(wnames))
	    write("There are no wizards with global read rights.\n");
	{
	    for (i = 0 ; i < sizeof(wnames) ; i++)
	    write(sprintf("%-10s added by %-10s", capitalize(wnames[i]), capitalize(gread[wnames[i]][0])) + " (" + gread[wnames[i]][1] + ")\n");
	}
	return 1;
    }

    cmds = explode(str, " ");
    comment = cmds[2..sizeof(cmds)];

    if (cmds[0] == "add")
	write(SECURITY->add_global_read(cmds[1], implode(comment, " ")) + "\n");
    else if (cmds[0] == "remove")
	write(SECURITY->remove_global_read(cmds[1]) + "\n");
    else
    {
	notify_fail("Do which 'global' operation?\n");
	return 0;
    }
    return 1;
}

/* **************************************************************************
 * lockout - arch managing of sites from which new players are locked out
 */
int
lockout(string str)
{
    CHECK_SO_ARCH;
    
    if (!strlen(str))
    {
	notify_fail("lockout what?\n");
	return 0;
    }
    return (int)SECURITY->lockout_site(str);
}


/* **************************************************************************
 * mkdomain - make a new domain
 */
int
mkdomain(string arg)
{
    string dname, lord;

    CHECK_SO_ARCH;

    if (!strlen(arg) || sscanf(arg, "%s %s", dname, lord) != 2)
    {
	notify_fail("Usage: mkdomain domain_name lord\n");
	return 0;
    }

    write(SECURITY->make_domain(dname, lord) + "\n");
    
    return 1;
}

/* **************************************************************************
 * mudstatus - Turn on and off /MUDstatistics
 */
int
mudstatus(string arg)
{
    int ev, ti;
    string fl;

    CHECK_SO_ARCH;

    
    if (!strlen(arg) || sscanf(arg,"%s %d %d", fl, ev, ti) != 3)
    {
	if (fl != "off")
	{
	    notify_fail("SYNTAX: mudstatus on/off eval_limit time_limit(ms)\n");
	    return 0;
	}
	ev = 0; 
	ti = 0;
    }

    SECURITY->do_debug("mudstatus", fl, ev, ti / 10);
    write("Ok.\n");
    return 1;
}
    
/* **************************************************************************
 * pingmud - send a udp ping to another mud
 */
int
pingmud(string arg)
{
    CHECK_SO_ARCH;
#ifdef UDP_MANAGER    
    return UDP_MANAGER->cmd_ping(arg);
#endif
    notify_fail("No udp manager active.\n");
    return 0;
}

/* **************************************************************************
 * rmapp - remove an old application
 */
int
rmapp(string arg)
{
    string str1, str2;

    CHECK_SO_ARCH;

    if (!strlen(arg) || sscanf(arg, "%s %s", str1, str2) != 2)
    {
	notify_fail("Usage: rmapp wname dname\n");
	return 0;
    }

    write(SECURITY->rm_application(str1, str2) + "\n");
    return 1;
}

/* **************************************************************************
 * rmdomain - remove an old domain
 */
int
rmdomain(string arg)
{
    CHECK_SO_ARCH;

    if (!strlen(arg))
    {
	notify_fail("Usage: rmdomain domain_name\n");
	return 0;
    }

    write(SECURITY->remove_domain(arg) + "\n");
    
    return 1;
}

/* **************************************************************************
 * trace - trace the mud
 */
int
set_trace(string str)
{
    int             n;
    int             o;

    CHECK_SO_ARCH;

    if (str && sscanf(str, "%d", n) == 1)
    {
	o = SECURITY->do_debug("trace", n);
	write("Trace was " + o + ", now " +
	      SECURITY->do_debug("trace", n) + "\n");
    }
    else
	write("Bad argument to trace.\n");
    return 1;
}

/* **************************************************************************
 * traceprefix - set the trace prefixes
 */
int
set_traceprefix(string str)
{
    string          o;

    CHECK_SO_ARCH;

    if (str)
	o = SECURITY->do_debug("traceprefix", str);
    else
	o = SECURITY->do_debug("traceprefix");
    write("Trace prefix was " + o + "\n");
    return 1;
}

/* **************************************************************************
 * startloc - handle starting locations
 */
int
startloc(string str)
{
    string *sstr;
    int what;

    CHECK_SO_ARCH;

    if (!stringp(str))
    {
	notify_fail("Wrong number of arguments to startloc.\n");
	return 0;
    }

    sstr = explode(str, " ");

    switch(sstr[0])
    {
    case "list":
	if (sizeof(sstr) < 2)
	    what = 3;
	else
	{
	    switch (sstr[1])
	    {
	    case "default":
	    case "def":
		what = 1;
		break;

	    case "temporary":
	    case "temp":
		what = 2;
		break;

	    default:
		notify_fail("I don't know of any '" + sstr[1] + "' start location.\n");
		return 0;
		break;
	    }
	}

	if (what == 1 || what == 3)
	{
	    write("Default start locations:\n");
	    write(sprintf("%-*#s\n", 76, 
		  implode(SECURITY->query_list_def_start(str), "\n")) + "\n");
	}

	if (what == 2 || what == 3)
	{
	    write("Temporary start locations:\n");
	    write(sprintf("%-*#s\n", 76, 
		  implode(SECURITY->query_list_temp_start(str), "\n")) + "\n");
	}
	break;
	
    case "add":
	if (sizeof(sstr) < 3)
	{
	    notify_fail("Add what?\n");
	    return 0;
	}

	switch(sstr[1])
	{
	case "default":
	case "def":
	    SECURITY->add_def_start_loc(sstr[2]);
	    break;

	case "temporary":
	case "temp":
	    SECURITY->add_temp_start_loc(sstr[2]);
	    break;

	default:
	    notify_fail("I don't know of any '" + sstr[1] + "' start location.\n");
	    return 0;
	    break;
	}
	break;

    case "remove":
    case "rem":
	if (sizeof(sstr) < 3)
	{
	    notify_fail("Remove what?\n");
	    return 0;
	}

	switch(sstr[1])
	{
	case "default":
	case "def":
	    SECURITY->rem_def_start_loc(sstr[2]);
	    break;

	case "temporary":
	case "temp":
	    SECURITY->rem_temp_start_loc(sstr[2]);
	    break;

	default:
	    notify_fail("I don't know of any '" + sstr[1] + "' start location.\n");
	    return 0;
	    break;
	}
	break;

    default:
	notify_fail("Do what?\n");
	return 0;
    }

    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * storemuds - Store the mudlist currently in the UDP_MANAGER
 */
int
storemuds(string arg)
{
#ifdef UDP_MANAGER    
    CHECK_SO_ARCH;

    if (UDP_MANAGER->update_masters_list())
    {
	write("Ok.\n");
	return 1;
    }
    else 
	notify_fail("Master did not allow the store.\n");
#endif
    return 0;
}

/* **************************************************************************
 * xpclear - Clear the xp counters in master for a specific domain
 *           NOTE
 *		The 'ranking' wont be affected until it is remade, each 5min
 */
int
xpclear(string dom)
{
    CHECK_SO_ARCH;

    if (SECURITY->query_domain_number(dom) < 0)
    {
	notify_fail("No such domain: " + dom + "\n");
	return 0;
    }
    SECURITY->domain_clear_xp(dom);
    write("Ok.\n");
    return 1;
}
