/*
 * application_player.c
 * 
 * Blocked site player application procedures.
 */

#include <config.h>
#include <std.h>
#include <macros.h>

void quit(int abort);

#ifdef RELAY_HOST
static string relay_host;

public nomask string
query_real_host() { return relay_host; }
#endif
string result = "", *info;
int pos;

void write_info();

void
enter_game()
{
    string data;
    int bl;

    set_screen_width(80);
#ifdef RELAY_HOST
    relay_host = previous_object()->query_relay();
#endif

    bl = SECURITY->check_newplayer(query_ip_number(this_object()));
    if (bl == 0)
    {
	write_socket("Your site isn't blocked. Log in as usual.\n");
	call_out("quit", 1, 0);
	return;
    }
    else if (bl == 1)
    {
	write_socket("Your site has been banned, not just simply blocked.\n" +
		     "No one will be admitted from your site.\n");
    }
    
    data = read_file(APPLICATION_INFO);
    if(!data)
    {
	write_socket("Panic! Couldn't read: " + APPLICATION_INFO + ".\n");
	call_out("quit", 1, 0);
	return;
    }

    info = explode(data, "##");

    enable_commands();
    pos = 0;
    write_info();
}

void
write_info()
{
    string *part;
    object ob;

    remove_call_out("time_out");

    if (pos >= sizeof(info))
    {
	catch(LOAD_ERR(APPLICATION_BOARD_LOC));
	ob = present("application board", find_object(APPLICATION_BOARD_LOC));
	if (!ob)
	    write_socket("\nThe application could not be saved. Please try again at\n" +
			 "a later date when the bug has been found and fixed.\n");
	else
	{
	    result = "Application from " + query_ip_number(this_object()) + " at " + ctime(time()) + "\n" + result;
	    ob->insert_message("Application (Root, " + ctime(time())[4..9] + ")", result);
	    write_socket("\nYour application has been filed.\n");
	}
	call_out("quit", 1, 0);
	return;
    }
    
    call_out("time_out", 10*60);

    part = explode(info[pos++], "&&");

    write_socket(part[2] + "\nApplication entry> ");
    if (explode(part[0], "\n")[0] == "m")
    {
	result += explode(part[1], "\n")[0];
	input_to("get_mline");
    }
    else if (explode(part[0], "\n")[0] == "-")
    {
	input_to("get_nline");
    }
    else
    {
	result += explode(part[1], "\n")[0];
	input_to("get_sline");
    }
    return;
}

void
get_sline(string str)
{
    if (str == "quit")
	call_out("quit", 1, 1);
    
    result += str + "\n";
    write_info();
}

void
get_nline(string str)
{
    if (str == "quit")
	call_out("quit", 1, 1);
    write_info();
}

void
get_mline(string str)
{
    if (str == "quit")
	call_out("quit", 1, 1);

    if (str != "**")
    {
	result += str + "\n\t\t";
	write_socket("Application entry> ");
	input_to("get_mline");
	return;
    }
    write_info();
}

void
time_out()
{
    write_socket("Application time out.\n");
    call_out("quit", 1, 1);
    return;
}

static void
quit(int abort)
{
    if (abort)
	write_socket("\nAborting application.\n");
    destruct();
}

string
query_real_name()
{
    return "application";
}
