/*
  /secure/login.c    Ver 2.0

  This is the object called from the Game Driver to log people into
  the Game. This object decides which player object is to be used by
  the player and swaps the socket to that object.

  New players and conversion ar not managed here but in the specific login-
  player object.

*/
#pragma save_binary

inherit "/std/object";

#include <const.h>
#include <ss_types.h>
#include <macros.h>
#include <stdproperties.h>
#include <std.h>
#include "/config/login/login.h"

/*
 * Prototypes
 */
int valid_name(string str);
void start_player();
void start_player2(object ob);
int enter_quota(string name);
#ifdef LOCKOUT_START_TIME
int is_lockout(string name);
#endif

/* These are the neccessary variables stored in the save file
*/
string          name;           /* The real name of the player */
string          password;       /* The password of the player */
string          player_file;    /* The racefile to use for the player */
int             link_dead;      /* True if player is loggin back after LD */
#ifdef CONVERT_PLAYERS
int		old_char;       /* True if we have an old character */
#endif
#ifdef RELAY_HOST
string		relay_host;	/* Real host name of user */
#endif

static void
create_object()
{
    set_name("logon");
    call_out("clean_up", 120);
}

void
clean_up()
{
    if (!query_ip_number(this_object()))
	destruct();
    else
	call_out("clean_up", 120);
}

string
query_pl_name() {  return name; }

int
logon()
{
    set_screen_width(80);

    if (!query_ip_number(this_object()))
    {
	remove_object();
	return 0;
    }

    if (SECURITY->check_newplayer(query_ip_number(this_object())) == 1)
    {
	write("\nYour site is blocked due to repeated offensive behaviour\n"+
	      "by users from your site.\n\n");
	remove_object();
	return 0;
    }

#ifdef RELAY_HOST
    if (member_array(efun::query_ip_name(this_object()), RELAY_HOST) >= 0)
    {
#ifdef RELAY_QUERY
	write(RELAY_QUERY);
#endif
	input_to("get_relay");
    }
    else
    {
#ifdef RELAY_FORCE
	if (member_array(lower_case(efun::query_ip_name(this_object())),
			 RELAY_FORCE) < 0)
	{
	    write("Direct connection disallowed.\n");
	    remove_object();
	    return 0;
	}
	else
	{
	    input_to("get_name");
	    relay_host = efun::query_ip_name(this_object());
	}
#else
	input_to("get_name");
	relay_host = efun::query_ip_name(this_object());
#endif
    }
#else
    input_to("get_name");
#endif

    player_file = 0; 
    seteuid(creator(this_object()));
    cat(LOGIN_FILE_WELCOME);
    write("Gamedriver version:\t" +
	  SECURITY->do_debug("version") + "\nMudlib version:\t\t" + 
	  MUDLIB_VERSION + "\n\n");
    write("What is your name: ");
    call_out("time_out", 120);

    return 1;
}

#ifdef RELAY_HOST
static void
get_relay(string str)
{
    relay_host = str;
    if (!sizeof(explode(str, ".")))
	input_to("get_relay");
    else
	input_to("get_name");
}

public string
query_relay() { return relay_host; }
#endif

static void
get_name(string str)
{
    object g_info, a_player;
    string *offensive;
    int i;

    if (!str || str == "")
    {
	remove_object();
	return;
    }
    str = lower_case(str);
    if (!valid_name(str))
    {
	input_to("get_name");
	write("Give name again: ");
	return;
    }
    if (file_size("/syslog/banish/" + extract(str, 0, 0) + "/" + str) >= 0)
    {
	write("That name is reserved.\n");
	input_to("get_name");
	write("Give name again: ");
	return;
    }
    if (SECURITY->query_domain_number(capitalize(str)) >= 0)
    {
	write("You can't have a name of a domain.\n");
	input_to("get_name");
	write("Give name again: ");
	return;
    }

    /*
     * General offensiveness check. The list can be added to as you
     * like. Just do not make it too long. Also, remember that you
     * should use banish for individual names.
     */
    offensive = ({ ".*fuck.*", ".*dick.*", ".*cunt.*", ".*arse.*", 
		   ".*shit.*", ".*pussy.*", ".*slut.*", ".*rape.*",
		   ".*fag.*", ".*penis.*", ".*hell.*", ".*bitch.*",
	       	   ".*suck.*", ".*clit.*", ".*peck.*", ".*cock.*",
		   ".*fart.*" });

#ifdef LOCKOUT_START_TIME
    /*
     * Check if the Mud is closed.
     */
    if (is_lockout(str))
    {
        write("Sorry, the game is closed now. It is open from " +
              LOCKOUT_END_TIME + ":00 hours\nuntil " + LOCKOUT_START_TIME +
              ":00 hours, daily. Please come back then!\n" +
              "Local time is " + ctime(time()) + ".\n");
        remove_object();

        return;
    }
#endif

    for (i = 0 ; i < sizeof(offensive) ; i++)
    {
	if (sizeof(regexp(({ str }), offensive[i])))
	{
	    write("\nName caught in offensiveness filter.\n" +
		  "Come up with something better or play somewhere else.\n");
	    remove_object();
	    return;
	}
    }

    /*
     * Check enter quota.
     */
    if (enter_quota(str) == 0)
    {
	write("Sorry, the game is full at the moment. Try again later.\n");
	remove_object();
	return;
    }

    if (str == GAMEINFO_LOGIN) /* Does own cleanup */
    {
	g_info = clone_object("/secure/gameinfo_player");
	exec(g_info, this_object());
	g_info->enter_game();
	remove_object();
	return;
    }

    if (str == APPLICATION_LOGIN) /* Does own cleanup */
    {
	a_player = clone_object("/secure/application_player");
	exec(a_player, this_object());
	a_player->enter_game();
	remove_object();
	return;
    }

    if (!restore_object("players/" + extract(str, 0, 0) + "/" + str))
    {
#ifdef CONVERT_PLAYERS
	old_char = 0;
	if ((file_size("/op/" + extract(str, 0, 0) + "/" + str + ".o") > 0) ||
	    (file_size("/op/" + str + ".o") > 0))
	{
	    write("Old character, give your old password.\n");
	    player_file = 0;
	    old_char = 1;
	}
	else
	{
	    if (SECURITY->check_newplayer(query_ip_number(this_object())) == 2)
	    {
		write("\nYour site is blocked due to repeated offensive\n" +
		      "behaviour by users from your site.\n\n" +
		      "You may still apply for a character by logging\n" +
		      "in as 'application'.\n\n");
		remove_object();
		return;
	    }

	    write("New character.\n");
	    player_file = 0;
	}
#else
	if (SECURITY->check_newplayer(query_ip_number(this_object())) == 2)
	{
	    write("\nYour site is blocked due to repeated offensive\n" +
		  "behaviour by users from your site.\n\n" +
		  "You may still apply for a character by logging\n" +
		  "in as 'application'.\n\n");
	    remove_object();
	    return;
	}

	write("New character.\n");
	player_file = 0;
#endif
    }
    
    /*
     * Do not do this before the restore !
     */
    name = str;			/* Must be here for a new player. */

    if (player_file)
    {
	input_to("check_password", 1);
    }
    else
    {
	if (SECURITY->query_wiz_level(name))
	{
	    write("\nOoops, this name was once used by an old wizard who " +
		"was deleted in a bad way.\nPlease contact an Arch about " +
		"this error, he can fix so you can use this name\nif you " +
		"want to also.\n");
	    destruct();
	    return;
	}
	input_to("new_password", 1);
    }
    write("Password: ");
    if (name == "guest")
	write("(just CR) ");
    return;
}

/*
 * Give a new password to a player.
 */
static void
new_password(string p)
{
    write("\n");
#ifdef CONVERT_PLAYERS
    if ((!p || p == "") && !old_char)
    {
	write("Try again another time then.\n");
	remove_object();
	return;
    }
#else
    if (!p || p == "")
    {
	write("Try again another time then.\n");
	remove_object();
	return;
    }
#endif
#ifdef CONVERT_PLAYERS
    if ((strlen(p) < 6) && !old_char)
#else
    if (strlen(p) < 6)
#endif
    {
	write("The password must have at least 6 characters.\n");
	input_to("new_password", 1);
	write("Password: ");
	return;
    }
    if (password == 0)
    {
	password = p;
#ifdef CONVERT_PLAYERS
	if (!old_char)
	{
	    input_to("new_password", 1);
	    write("Password: (again) ");
	}
	else
	{
	  object pold;
	  string seed, tmp;
	  
	    if (file_size(CONVERT_OLD) > 0)
	      {
		tmp = read_file(CONVERT_OLD);
		pold = clone_object(tmp);
		if (pold)
		  {
		    seteuid("root");
		    export_uid(pold);
		    if (!pold->enter_here(name))
		      {
			write("Couldn't restore your old character.\n");
			pold->remove_object();
		      }
		    seteuid(creator(pold));
		    export_uid(pold);
		    if (pold->q_password())
			seed = extract(pold->q_password(),0,1);
		  }
	      }
	      
	    if (strlen(password))
	      {
		if (seed)
		  password = crypt(password, seed);  /* Use old seed. */
		else
		  password = crypt(password, 0);  /* Generate new seed. */
	      }
	    else
		password = 0;
	    
	    start_player();
	    return;
	}
#else
	input_to("new_password", 1);
	write("Password: (again) ");
#endif	
	return;
    }
    remove_call_out("time_out");
    if (password != p)
    {
	write("You changed!\n");
	remove_object();
	return;
    }
    password = crypt(password, 0);	/* Generate new seed. */

    start_player();
}

static void
check_password(string p)
{
    write("\n");
    remove_call_out("time_out");

    if (password == 0)
	write("You have no password ! Set it with the 'password' cmd.\n");
    else if (name != "guest" && crypt(p, password) != password)
    {
	write("Wrong password!\n");
	remove_object();
	return;
    }
    start_player();
}

void
time_out()
{
    write("Time out\n");
    remove_object();
}

/*
 * Function name: start_player
 * Description:   Initializes the player and prepares for game entry
 */
static void
start_player()
{
    object          ob,
                    other_copy;
    string          cr_pfile;

    /*
     * See if we are already playing.
     */
    other_copy = find_player(name);

    if (other_copy)
    {
	write("You are already playing !\n");
	write("Throw the other copy out ? ");
	input_to("try_throw_out");
	return;
    }
    
    /*
     * Now we can enter the game, find the player file
     */
    if (player_file)
    {
	ob = clone_object(player_file);
	if ((ob) && (function_exists("enter_game", ob)) != "/std/player_sec")
	{
	    player_file = 0;
	    ob->remove_object();
	}

	if (!ob)
	{
	    write("I can't find your body, you must choose a new.\n");
	    player_file = 0;
	}
    }

    /* 
     * There can be three different reasons for not having a player_file:
     * 
     *    1 - If this is a new character, let the login player object
     *        manage the creation / conversion / process.
     *    2 - The players racefile is not loadable, a new body must be
     *        choosen.
     *    3 - The players racefile is not a legal playerfile, a new body
     *        must be choosen.
     */
    if (!player_file || player_file == LOGIN_NEW_PLAYER)
    {
	ob = clone_object(LOGIN_NEW_PLAYER);
	ob->open_player(); 

	seteuid(SECURITY->get_bb_uid());
	export_uid(ob); 
	ob->set_trusted(1); 
	exec(ob, this_object());
	ob->enter_new_player(name, password);
	remove_object();
	return;
    }

    ob->open_player(); 

    if (SECURITY->query_wiz_level(name))
	seteuid(name);
    else
	seteuid(SECURITY->get_bb_uid());

    export_uid(ob); 
    ob->set_trusted(1); 
    start_player2(ob);
}


/*
 * Function name: start_player2
 * Description:   Swapsocket to player object and if we are not already
 *                in the game enter it.
 * Arguments:	  ob: Player object to be.
 */
static void
start_player2(object ob)
{
    object          dump;
    int             lev,
                    oklev;
    string 	tst;

#ifdef STATUE_WHEN_LINKDEAD
    int old_was_live;
    old_was_live = 0;
#endif

    cat(LOGIN_FILE_NEWS);

    if (query_ip_number(ob))
    {				/* Must lose old socket first */
	dump = clone_object(DEFAULT_PLAYER);
	exec(dump, ob);		/* Swap old socket to dummy player */
	dump->remove_object();
#ifdef STATUE_WHEN_LINKDEAD
	old_was_live = 1;
#endif
    }

    if (ob)
	exec(ob, this_object());/* Swap interactive object */
    else
	start_player();		/* We lost the other copy while asking */

    SECURITY->log_public("ENTER", ctime(time()) + " " + name + " from " + query_ip_name(ob) + "\n");

    /*
     * If we are not in the game, enter it.
     */
    if (!environment(ob))
    {
	if (!ob->enter_game(name))
	{
	    write("Illegal playerfile.\n");
	    ob->remove_object();
	}
    }
#ifdef STATUE_WHEN_LINKDEAD
    else if (! old_was_live) {
	ob->set_linkdead(0);
	ob->fixup_screen();
#ifdef OWN_STATUE
	OWN_STATUE->revive(ob);
#else	
	say(QCTNAME(ob) + " " + STATUE_TURNS_ALIVE + ".\n");
#endif
    }
#endif
    else
	ob->fixup_screen();

    SECURITY->notify(ob, name, 1, link_dead);
 
    /*
     * Make sure the level is the right one.
     */
    oklev = (int)SECURITY->query_wiz_level(name);
    lev = (int)ob->query_wiz_level();
    if (lev != oklev)
    {
	write("Your level has changed during your absence.\n");
	ob->set_wiz_level(oklev);
    }

    /* Do not count commands for wizards
    */
    if (oklev || sscanf(ob->query_real_name(), "%sjr", tst) == 1)
	SECURITY->do_debug("set_wizard", ob);
    ob->update_hooks();
    remove_object();
}


/*
 * Function name: try_throw_out
 * Description:   Gets response on throw out other copy question
 * Arguments:	  str: Response
 */
static void
try_throw_out(string str)
{
    object          ob;
    if (str == "" || (str[0] != 'y' && str[0] != 'Y'))
    {
	write("Welcome another time then !\n");
	remove_object();
	return;
    }
    link_dead = 1;
    ob = find_player(name);
    start_player2(ob);
}

/*
 * Check that a player name is valid. Only allow
 * lowercase letters.
 */
int
valid_name(string str)
{
    int             i,
                    length;

    if (str == "logon" || strlen(str) == 1)
    {
	write("Invalid name\n");
	return 0;
    }
    length = strlen(str);
    if (length > 11)
    {
	write("Too long name.\n");
	return 0;
    }
    i = 0;
    while (i < length)
    {
	if (str[i] < 'a' || str[i] > 'z')
	{
	    write("Invalid characters in name:" + str + "\n");
	    write("Character number was " + (i + 1) + ".\n");
	    return 0;
	}
	i += 1;
    }
    return 1;
}

string
valid_read(string str)
{
    string          file;

    if (sscanf(str, "/%s", file) == 1)
	return file;
    else
	return str;
}

#ifdef LOCKOUT_START_TIME
/*
 * Function name: is_lockout
 * Description:   This function determines if the game is open to players
 *                now. The mud will be open to wizards and their test
 *                characters above the LOCKOUT_LEVEL.
 * Arguments:     pl_name - The name of the player attempting to
 *                          enter the game.
 * Returns:       True if the mud is closed to this player, false
 *                otherwise.
 */
int
is_lockout(string pl_name)
{
    int d, h, ob_type;
    string day, mon, wiz_name;

    /*
     * Determine if we are in the lockout period.
     */
    sscanf(ctime(time()), "%s %s %d %d:%d:%d %d", day, mon, d, h, d, d, d);
    if ((h >= LOCKOUT_START_TIME) && (h < LOCKOUT_END_TIME) &&
        (day != "Sat") && (day != "Sun"))
    {
        ob_type = WIZ_TYPE(SECURITY->query_wiz_level(pl_name));
        if (ob_type >= LOCKOUT_LEVEL)
            return 0;
        /*
         * Here we assume there are no `old players', i.e., that
         * names ending in `jr' are wizard's test characters.
         */
        if (extract(pl_name, strlen(pl_name) - 2, strlen(pl_name)) == "jr")
        {
            wiz_name = extract(pl_name, 0, strlen(pl_name) - 3);
            ob_type = WIZ_TYPE(SECURITY->query_wiz_level(wiz_name));
            if (ob_type >= LOCKOUT_LEVEL)
                return 0;
        }
        /*
         * Everyone else is locked out
         */
        return 1;
    }
    return 0;
}
#endif

/*
 * Function name: enter_quota
 * Description:   This function determines whether a certain player
 *		  may enter the game or not.
 * Arguments:	  pl_name - The player in questions name
 * Returns:	  True or false.
 */
int
enter_quota(string pl_name)
{
   object *list;
   int left, arch, arch_res, wiz, frac, i, ob_type;

   /*
    * Begin by getting rid of idlers. This is done EVERY time anyone
    * tries to log in. Some can be idle longer than others.
    */
   list = users();
   for (i = 0 ; i < sizeof(list) ; i++)
   {
       ob_type = WIZ_TYPE(SECURITY->query_wiz_level(list[i]->query_real_name()));

       if (query_idle(list[i]) > MAX_IDLE_TIME * (1 + ob_type))
       {
	   tell_object(list[i], "You have been idle too long. You are logged out.\n");
	   list[i]->save_character();
	   list[i]->remove_object();
       }
   }
   ob_type = WIZ_TYPE(SECURITY->query_wiz_level(pl_name));
   list = users();

   /*
    * The game is FULL.
    */
   if (sizeof(list) >= MAX_PLAY)
       return 0;

   /*
    * Calculate the slots left, the number of wizards and arches.
    */
   left = MAX_PLAY - sizeof(list);
   for (i = 0 ; i < sizeof(list) ; i++)
   {
       if (list[i] && list[i]->query_wiz_level())
       {
	   wiz++;
	   if (WIZ_TYPE((int)list[i]->query_wiz_level()) >= WIZ_ARCH)
	       arch++;
       }
   }

   /*
    * Determine the wizard ratio and the arch reserve.
    */
   frac = 100 * wiz / MAX_PLAY;
   arch_res = ARCH_SLOTS > arch ? ARCH_SLOTS - arch : 0;

   /*
    * Anyone can enter as long as there is room. 
    */
   if (left > ARCH_SLOTS + MIN_SLOTS)
       return 1;

   /*
    * Is he an Arch+? In that case, enter no matter what.
    */
   if (ob_type >= WIZ_ARCH)
       return 1;

   /*
    * Is he a mortal && space left (arches accounted for)?
    */
   if (ob_type == WIZ_MORTAL && left > arch_res)
       return 1;

   /*
    * Is he a wizard and the ratio ok?
    */
   if (frac < WIZ_RATIO)
       return 1;
   
   return 0;
}

query_race()
{
    return "logon";
}

public void
catch_tell(string msg)
{
    write_socket(msg);
}
