/*
  /secure/std.h

  This file holds all global constants and macros relevant to game security

*/

#ifndef SECURE_DEFINED
#define SECURE_DEFINED

/*
 * MUDLIB_VERSION
 *
 * This is the name of the version of the mudlib.
 *
 */
#define MUDLIB_VERSION "CD.00.30"

/*
 *  PLAYER_FILE
 *
 * This is the macro for finding the filename for saving the player object.
 *
 */
#define PLAYER_FILE(name) ("/players/" + extract(name,0,0) + "/" + name)

/*
 * This object holds all function used for test certain functions of the game.
 * For example: 
 *               Allowed player files
 *               Allowed starting locations.
 *               Allowed guilds
 */
#define SECURITY "/secure/master"
#define CALLED_BY_SECURITY (previous_object() == find_object(SECURITY))


/*
 * Wizard levels
 *
 * These are the max values on each level
 */
#define WIZLEV_MORTAL     0
#define WIZLEV_APPRENTICE 1
#define WIZLEV_PILGRIM    8
#define WIZLEV_RETIRED    9
#define WIZLEV_NORMAL     19
#define WIZLEV_MAGE       29
#define WIZLEV_LORD       39
#define WIZLEV_ARCH       49
#define WIZLEV_KEEPER     50


/* 
 * Midlevels
 */
#define WIZMID(x) ((x == WIZLEV_ARCH) ? 45 : \
		     (x == WIZLEV_LORD) ? 35 : \
		     (x == WIZLEV_MAGE) ? 25 : \
		     (x == WIZLEV_NORMAL) ? 15 : \
		     (x == WIZLEV_RETIRED) ? 9 : \
		     (x == WIZLEV_PILGRIM) ? 5 : \
		     (x == WIZLEV_APPRENTICE) ? 1 : 0)


/* 
 * These are the possible types of wizards
 *
 */
#define WIZ_MORTAL     0
#define WIZ_APPRENTICE 1
#define WIZ_PILGRIM    2
#define WIZ_RETIRED    3
#define WIZ_NORMAL     4
#define WIZ_MAGE       5
#define WIZ_LORD       6
#define WIZ_ARCH       7
#define WIZ_KEEPER     8

#define GETLEV (query_wiz_level(geteuid(previous_object())))
#define GETTYPE (WIZ_TYPE(GETLEV))
/*
 * WIZ_TYPE
 *
 * This macro gives you the wizard type as a function of level
 */
#define WIZ_TYPE(x) ((x > WIZLEV_ARCH) ? WIZ_KEEPER : \
		     (x > WIZLEV_LORD) ? WIZ_ARCH : \
		     (x > WIZLEV_MAGE) ? WIZ_LORD : \
		     (x > WIZLEV_NORMAL) ? WIZ_MAGE : \
		     (x > WIZLEV_RETIRED) ? WIZ_NORMAL : \
		     (x > WIZLEV_PILGRIM) ? WIZ_RETIRED : \
		     (x > WIZLEV_APPRENTICE) ? WIZ_PILGRIM : \
		     (x > WIZLEV_MORTAL) ? WIZ_APPRENTICE : \
		     WIZ_MORTAL)

/*
 * WIZ_LEV_NAME
 *
 * This macro gives you the wizard type as a function of level
 */
#define WIZ_LEV_NAME(x) ((x > WIZLEV_ARCH) ? "keeper" : \
			 (x > WIZLEV_LORD) ? "arch" : \
			 (x > WIZLEV_MAGE) ? "lord" : \
			 (x > WIZLEV_NORMAL) ? "mage" : \
			 (x > WIZLEV_RETIRED) ? "wizard" : \
			 (x > WIZLEV_PILGRIM) ? "retired" : \
			 (x > WIZLEV_APPRENTICE) ? "pilgrim" : \
			 (x > WIZLEV_MORTAL) ? "apprentice" : \
			 "mortal")

/* GAAAAHH!!! I've never seen such an inefficient way of getting the
** level!!  Just check out how WIZ_LEV_NAME is called!
** Macro argument should never be used more than once!
** It's also horribly dangerously coded (x is not bracked)!
**  /LA
** there seems to be a bug in query_gender!
*/
/* x is an object, get the gender and level */
#define WIZ_LEVEL_NAME(x) (\
     ({ "mortal", \
	"apprentice", \
	"pilgrim", \
	"retired", \
	"wizard", \
	"mage", \
	"liege", \
	"arch", \
	"keeper", \
     })[WIZ_TYPE((x)->query_wiz_level())])
#define WIZ_DOMAIN_RULER_NAME(x) "liege"

/*
 * WIZ_T
 * WIZ_N
 *
 * This macro gives you the wizard types (name and number) in an array
 */
#define WIZ_T ({ "apprentice", "pilgrim", "retired", "wizard", "mage", \
		 "liege", "arch", "keeper" })

#define WIZ_N ({ WIZ_APPRENTICE, WIZ_PILGRIM, WIZ_RETIRED, WIZ_NORMAL, WIZ_MAGE, \
		WIZ_LORD, WIZ_ARCH, WIZ_KEEPER })

/*
 * WIZ_CHECK
 *
 * This macro is used for terminating functions if a wizards level is
 * not sufficiently high.
 *
 */
#define WIZ_CHECK (this_interactive() && WIZ_TYPE(this_interactive()->query_wiz_level()))

/*
 * WIZ_CMD_ ....
 *
 * These are the files containing the definition of specific wizard commands.
 *
 */
#define WIZ_CMD_MORTAL     "/cmd/wiz_cmd_mortal"
#define WIZ_CMD_APPRENTICE "/cmd/wiz_cmd_apprentice"
#define WIZ_CMD_PILGRIM    "/cmd/wiz_cmd_pilgrim"
#define WIZ_CMD_RETIRED    "/cmd/wiz_cmd_retired"
#define WIZ_CMD_NORMAL     "/cmd/wiz_cmd_normal"
#define WIZ_CMD_LORD       "/cmd/wiz_cmd_lord"
#define WIZ_CMD_MAGE       "/cmd/wiz_cmd_mage"
#define WIZ_CMD_ARCH       "/cmd/wiz_cmd_arch"
#define WIZ_CMD_KEEPER     "/cmd/wiz_cmd_keeper"

#define TRACER_TOOL_SOUL   "/cmd/std/tracer_tool"
 
/*
 * WIZ_SOUL
 *
 * This macro gives the wizard command file to use for a certain wizard level.
 *
 */   
#define WIZ_SOUL(level) ((level > WIZLEV_ARCH) ? WIZ_CMD_KEEPER : \
			 (level > WIZLEV_LORD) ? WIZ_CMD_ARCH : \
			 (level > WIZLEV_MAGE) ? WIZ_CMD_LORD : \
			 (level > WIZLEV_NORMAL) ? WIZ_CMD_MAGE : \
			 (level > WIZLEV_RETIRED) ? WIZ_CMD_NORMAL : \
			 (level > WIZLEV_PILGRIM) ? WIZ_CMD_RETIRED : \
			 (level > WIZLEV_APPRENTICE) ? WIZ_CMD_PILGRIM : \
			 (level > WIZLEV_MORTAL) ? WIZ_CMD_APPRENTICE : \
			 WIZ_CMD_MORTAL) 

/* And now some prototypes. */
string wiz_domain(string name);

/*
 * DOCMAKER
 *
 * Path to the file that implements the documentation making functionality
 */
#define DOCMAKER "/secure/docmake"

/*
 * MUDLIST_SERVER
 *
 * Address to the mudlist server. /secure/master will send a message to
 * this address on startup. Other interaction is possible through a
 * special protocol using the udp messages.
 *
 * The default server is defined as Genesis below. You can change this in
 * your /config/sys/local.h by undefining and redefining this symbol.
 */
#define MUDLIST_SERVER ({ "milou.cd.chalmers.se", 2500 })

#include "/secure/config.h"

#endif SECURE_DEFINED 

