/* Gedit_parse.h for Gedit by Groo 91-06-13 */
/* Last modified : 91-11-06 */
/* Version 1.11 */

#ifndef NO_GEDIT_DEFS
#define GEDIT_COMMAND_PRINT   1
#define GEDIT_COMMAND_APPEND  2
#define GEDIT_COMMAND_INSERT  3
#define GEDIT_COMMAND_DELETE  4
#define GEDIT_COMMAND_HELP    5
#define GEDIT_COMMAND_QUIT    6
#endif

#define LCUR 		1
#define RCUR		2
#define LPLUS		4
#define RPLUS		8
#define LMINUS		16
#define RMINUS		32
#define LDOLLAR		64
#define RDOLLAR		128
#define COMMA		256

mixed *
parse_line(string str, int current_index, int last_index)
{
    int tmp;
    int left, right, leftval, rightval;
    int start, end, cmd, flags;
    
    if (strlen(str) == 1)
    {
	switch (str)
	{
	case "d": 
	    return ({ current_index, current_index, GEDIT_COMMAND_DELETE });
	case "a": 
	    return ({ current_index, current_index, GEDIT_COMMAND_APPEND });
	case "i": 
	    return ({ current_index, current_index, GEDIT_COMMAND_INSERT });
	case ".":
	case "p": 
	    return ({ current_index, current_index, GEDIT_COMMAND_PRINT });
	case "h":
	case "?":
	    return ({ 0, 0, GEDIT_COMMAND_HELP });
	case "q":
	case "Q":
	    return ({ 0, 0, GEDIT_COMMAND_QUIT });
	}
    }
    
    tmp = strlen(str) - 1;
    switch (str[tmp])
    {
    case 'a':
	cmd = GEDIT_COMMAND_APPEND;
	sscanf(str, "%sa", str);
	break;
    case 'd':
	cmd = GEDIT_COMMAND_DELETE;
	sscanf(str, "%sd", str);
	break;
    case 'i':
	cmd = GEDIT_COMMAND_INSERT;
	sscanf(str, "%si", str);
	break;
    case 'p':
	cmd = GEDIT_COMMAND_PRINT;
	sscanf(str, "%sp", str);
	break;
    default:
	cmd = GEDIT_COMMAND_PRINT;
    }
    
    while (strlen(str))
    {
	if (str[0] == '.')
	{
	    sscanf(str, ".%s", str);
	    if (!(flags & COMMA))
	    {
		if (flags & LDOLLAR) return 0;
		left += (current_index + 1);
		flags |= LCUR;
	    }
	    else
	    {
		if (flags & RDOLLAR) return 0;
		right += (current_index + 1);
		flags |= RCUR;
	    }
	}
	else if (str[0] == '$')
	{
	    sscanf(str, "$%s", str);
	    if (!(flags & COMMA)) {
		if ((flags & LCUR) || (flags & LPLUS)) return 0;
		left += (last_index + 1);
		flags |= LDOLLAR;
	    }
	    else
	    {
		if ((flags & RCUR) || (flags & RPLUS)) return 0;
		right += (last_index + 1);
		flags |= RDOLLAR;
	    }
	}
	else if (str[0] == '-')
	{
	    sscanf(str, "-%s", str);
	    if (!(flags & COMMA))
	    {
		if (flags & LPLUS) return 0;
		flags |= LMINUS;
	    }
	    else
	    {
		if (flags & RPLUS) return 0;
		flags |= RMINUS;
	    }
	}
	else if (str[0] == '+')
	{
	    sscanf(str, "+%s", str);
	    if (!(flags & COMMA)) {
		if ((flags & LMINUS) || (flags & LDOLLAR)) return 0;
		flags |= LPLUS;
	    }
	    else
	    {
		if ((flags & RMINUS) || (flags & RDOLLAR)) return 0;
		flags |= RPLUS;
	    }
	}
	else if (str[0] == ',')
	{
	    sscanf(str, ",%s", str);
	    if (flags & COMMA) return 0;
	    flags |= COMMA;
	}
	else if (leftval || (flags & COMMA))
	{
	    if (rightval || !(flags & COMMA) || 
		sscanf(str, "%d%s", rightval, str) != 2)
		return 0;
	    else if (rightval < 0)
		rightval = -rightval;
	}
	else if ((flags & COMMA) || sscanf(str, "%d%s", leftval, str) != 2)
            return 0;
	else if (leftval < 0)
	    leftval = -leftval;
    }
    if (!left)
    {
	if (leftval)
	{
	    if (flags & LMINUS)
		left = current_index - leftval + 1;
	    else if (flags & LPLUS)
		left = current_index + leftval + 1;
	    else left = leftval;
	}
	else if (flags & LMINUS)
	    left = current_index;
	else if (flags & LPLUS)
	    left = current_index + 2;
    }
    else if (leftval)
    {
	if (flags & LMINUS)
	    left -= leftval;
	else if (flags & LPLUS)
	    left += leftval;
    }
    if (!right)
    {
	if (rightval)
	    right = rightval;
	else right = left;
    }
    else
    {
	if (flags & RPLUS)
	    right += rightval;
	else if (flags & RMINUS)
	    right -= rightval;
    }
    if (!left || !right)
	return 0;
    return ({ left - 1, right - 1, cmd });
}

/* Gedit by Groo 91-06-13 */
/* Genesis mailreader modifications 91-10-07 */
/* Genesis bulletin board modifications 91-11-03 */
/* Last modified 91-11-06 */

/* Thanks go to the bugfinders : 
Survivorii (who found the nasty EOB-bug..)
    Survivorii who did it again when he found a small bug.
    Survivorii (yes, him again) who noticed that linenumbers > MAXINT
    would cause an illegal index value error because of
    sscanf() exception.
    */
    
#define CPROMPT "Gedit : "
    
#define LAST_MODIFIED "91-11-03"
#define VERSION "1.1"
    
#ifndef NO_GEDIT_DEFS
#define GEDIT_COMMAND_MODE    0
#define GEDIT_APPEND_MODE     1
#define GEDIT_INSERT_MODE     2
#define GEDIT_SETPREF_MODE    3
#define GEDIT_SETPREF_CPROMPT 4
#endif	

#define GDISPLAY(X) tell_object(editor, X)
#define GPROMPT() tell_object(editor, cprompt)

static object ged_user;
static string *ed_it_buffer, *cur_buf, current_file;
static int current_index, last_index; 
static int mode;
string cprompt;
int print_line_numbers, iaprompt;

/*
 * Prototypes
 */
void setpref_menu();
void gedit_read_file(string file);
void gedit_write_file(string file);
void display_lines(int start, int end);
void delete_lines(int start, int end);
void help();
void quit_function();

void init_ed_it(object the_editor)
{
    ged_user = the_editor;
    ed_it_buffer = ({});
    current_index = 0;
    last_index = -1;
    cprompt = "Command : ";
#ifdef CPROMPT
    cprompt = CPROMPT;
#endif
    mode = GEDIT_COMMAND_MODE;
    current_file = "";
}
	
string *
get_ed_it_buffer()
{
    return ed_it_buffer;
}
	
mixed
ed_it(string str)
{
    string part1, part2;
    int start, end, cmd;
    mixed tmp, tmp2;
    switch (mode)
    {
    case GEDIT_COMMAND_MODE:
	if (str == "")
	{
	    if (current_index >= last_index)
	    {
		current_index = last_index;
		GDISPLAY("<EOB>\n");
		GPROMPT();
		return 0;
	    }
	    GDISPLAY(ed_it_buffer[++current_index] + "\n");
	    GPROMPT();
	    return 0;
	}   
	else if (str == "prefs")
	{
	    GDISPLAY("The preferences have been excluded in the " +
		     "bulletin board version\nof Gedit, unfortunately.\n");
	    return 0;
	    mode = GEDIT_SETPREF_MODE;
	    setpref_menu();
	    return 0;
	}
	else if (str[0] == 'w' || str[0] == 'W')
	{
	    if (ged_user->query_wiz_level() < 2)
	    {
		GDISPLAY("Only wizards may use this command. Sorry!\n");
		return 0;
	    }
	    if (sscanf(str, "%s %s", tmp2, tmp) != 2)
	    {
		if (current_file == "")
		    GDISPLAY("No filename.\n");
		else 
		    gedit_write_file(current_file);
	    }
	    else
		gedit_write_file(tmp);
	    GPROMPT();
	    return 0;
	}
	else if (str[0] == 'r' || str[0] == 'R')
	{
	    if (ged_user->query_wiz_level() < 2)
	    {
		GDISPLAY("Only wizards may use this nice command. Sorry.\n");
		return 0;
	    }
	    if (sscanf(str, "%s %s", tmp2, tmp) != 2)
		GDISPLAY("Usage : r(ead) <file>\n");
	    else
		gedit_read_file(tmp);
	    GPROMPT();
	    return 0;
	}
	else if (str == "=")
	{
	    if (current_file != "")
		GDISPLAY("Current file : \"" + current_file + "\"\n");
	    else 
		GDISPLAY("Current file : [none]\n");
	    GDISPLAY("Edit buffer = " + (last_index+1) + " lines\n" +
		     "Current line = " + (current_index+1) + "\n");
	    GPROMPT();
	    return 0;
	}
	else if (str == "help")
	{
	    help();
	    GPROMPT();
	    return 0;
	}
	tmp = parse_line(str, current_index, last_index);
	if (!pointerp(tmp))
	{
	    GDISPLAY("Error or bad command.\n");
	    GPROMPT();
	    return 0;
	}
	start = tmp[0];
	if ((end = tmp[1]) > last_index)
	    end = last_index;
	cmd = tmp[2];
	if (!cmd)
	{
	    if (!random(10))
		GDISPLAY("Insufficient user intelligence error..\n");
	    else
		GDISPLAY("Unknown command...try again.\n");
	    GPROMPT();
	}
	else
	{
	    switch (cmd)
	    {
	    case GEDIT_COMMAND_PRINT:
		display_lines(start, end);
		break;
	    case GEDIT_COMMAND_APPEND:
		mode = GEDIT_APPEND_MODE;
		cur_buf =  ({});
		current_index = start;
		if (iaprompt)
		    GDISPLAY("A>");
		break;
	    case GEDIT_COMMAND_INSERT:
		mode = GEDIT_INSERT_MODE;
		cur_buf = ({});
		current_index = start;
		if (iaprompt)
		    GDISPLAY("I>");
		break;
	    case GEDIT_COMMAND_DELETE:
		delete_lines(start, end);
		break;
	    case GEDIT_COMMAND_HELP:
		help();
		break;
	    case GEDIT_COMMAND_QUIT:
		return quit_function();
	    default:
		if (!random(10))
		    GDISPLAY("User too stupid error.\n");
		else
		    GDISPLAY("Unknown command..try again.\n");
		GPROMPT();
	    }
	}
	break;
    case GEDIT_APPEND_MODE:
    	if (str == ".")
	{
	    mode = GEDIT_COMMAND_MODE;
	    if (tmp = sizeof(cur_buf))
	    {
		tmp2 = sizeof(ed_it_buffer);
		if (current_index >= tmp2 - 1)
		    ed_it_buffer = ed_it_buffer + cur_buf;
		else
		    ed_it_buffer = slice_array(ed_it_buffer, 0, current_index) +
			cur_buf + slice_array(ed_it_buffer, current_index + 1, 
					      tmp2 - 1);
		if (!tmp2)
		    tmp--;
		current_index += tmp;
		last_index = sizeof(ed_it_buffer) - 1;
	    }
	    GPROMPT();
	}
	else
	{
	    cur_buf = cur_buf + ({ str });
	    if (iaprompt)
		GDISPLAY("A>");
	}
	break;
    case GEDIT_INSERT_MODE:
	if (str == ".")
	{
	    mode = GEDIT_COMMAND_MODE;
	    if (tmp = sizeof(cur_buf))
	    {
		tmp2 = sizeof(ed_it_buffer);
		if (current_index == 0)
		    ed_it_buffer = cur_buf + ed_it_buffer;
		else
		    ed_it_buffer = slice_array(ed_it_buffer, 0, current_index-1) +
			cur_buf + slice_array(ed_it_buffer, current_index, 
					      tmp2 - 1);
	    }
	    if (!tmp2)
		tmp--;
	    last_index = sizeof(ed_it_buffer) - 1;
	    current_index += tmp;
	    GPROMPT();
	}
	else
	{
	    cur_buf = cur_buf + ({ str });
	    if (iaprompt)
		GDISPLAY("I>");
	}
	break;
    case GEDIT_SETPREF_MODE:
   	switch (str)
	{
	case "1":
	    mode = GEDIT_SETPREF_CPROMPT;
	    GDISPLAY("Enter the new command prompt : ");
	    break;
	case "2":
	    if (iaprompt)
	    {
		iaprompt = 0;
		GDISPLAY("No insert/append prompt will be displayed.\n");
	    }
	    else
	    {
		iaprompt = 1;
		GDISPLAY("The insert/append prompt will be displayed.\n");
	    }
	    setpref_menu();
	    break;
	case "3":
	    if (print_line_numbers)
	    {
		print_line_numbers = 0;
		GDISPLAY("Line numbers will not be displayed.\n");
	    }
	    else
	    {
		print_line_numbers = 1;
		GDISPLAY("Line numbers will be displayed.\n");
	    }
	    setpref_menu();
	    break;
	case "":
	    mode = GEDIT_COMMAND_MODE;
	    GDISPLAY("\n");
	    GPROMPT();
	    break;
	default:
	    GDISPLAY("Wrong. Try another key.\n");
	    setpref_menu();
	}
	break;
    case GEDIT_SETPREF_CPROMPT:
	cprompt = str;
	mode = GEDIT_SETPREF_MODE;
	setpref_menu();
	break;
    default:
	GDISPLAY("Bug-in-program error.\n");
    }
}


int
display_lines(int start, int end)
{
    if (start < 0)
    {
	GDISPLAY("<Buffer empty>\n");
	GPROMPT();
	return 0;
    }
    if (start > end || start > last_index || end > last_index)
    {
	GDISPLAY("Incorrect line number(s) given.\n");
	GPROMPT();
	return 0;
    }
    while (start <= end)
    {
	if (print_line_numbers)
	    GDISPLAY((start+1) + ":\t");
	GDISPLAY(ed_it_buffer[start] + "\n");
	start++;
    }
    current_index = end;
    GPROMPT();
}

int
delete_lines(int start, int end)
{
    if (end > last_index)
	end = last_index;
    if (start > end) {
	GDISPLAY("Incorrect line number(s) given.\n");
	GPROMPT();
	return 0;
    }
    if (start == 0)
    {       
	current_index = 0;
	if (end == last_index) 
	    ed_it_buffer = ({});
	else 
	    ed_it_buffer = slice_array(ed_it_buffer, end+1, last_index);       
    }
    else if (end == last_index)
    {
	ed_it_buffer = slice_array(ed_it_buffer, 0, start - 1);
	current_index = start - 1;
    }
    else
    {
	ed_it_buffer = slice_array(ed_it_buffer, 0, start - 1) +
	    slice_array(ed_it_buffer, end + 1, last_index);
	current_index = end;
    }
    last_index = sizeof(ed_it_buffer) - 1;
    GPROMPT();
    return 1;
}

string *
quit_function()
{
    return ed_it_buffer;
}

void
help()
{
    GDISPLAY("\nGEDIT V" + VERSION + " by Groo " + LAST_MODIFIED + " :\n");
    GDISPLAY("\nThis editor works (or should work) as ED does only it\n" +
	     "is less powerful (yet..;).\n" +
	     "Supported commands are : \n" +
	     "'a' - Append\n'd' - Delete\n'i' - Insert\n'p' - Print\n" +
	     "'=' - Info about current file and edit buffer\n" +
	     "'q' = Quit Gedit (When finished editing. Text will be saved)\n" +
	     "'h', '?' - This help\n" +
	     "'prefs' - Preferences menu\n" +
	     "'w [file]' - Write buffer to file\n" +
	     "'r <file>' - Read file to buffer\n" +
	     "\n\n");
    GPROMPT();
}

void
setpref_menu()
{
    GDISPLAY("\nGEDIT Preferences Menu\n\n");
    GDISPLAY("1. \tCommand prompt : \"" + cprompt + "\"\n");
    GDISPLAY("2. \tDisplay prompt for each line when inserting/appending : ");
    if (iaprompt)
	GDISPLAY("On\n");
    else
	GDISPLAY("Off\n");
    GDISPLAY("3. \tDisplay line numbers during print : ");
    if (print_line_numbers)
	GDISPLAY("On\n");
    else
	GDISPLAY("Off\n");
    GDISPLAY("\nChoose one of the above to change or return to exit prefs : ");
}

void
gedit_read_file(string fname)
{
    string *tmp_buf;
    int x, y, z;
    if (fname[0] != '/')
	fname = ged_user->query_path() + "/" + fname;
    if ((x = file_size(fname)) == -1)
	return GDISPLAY("\"" + fname + "\" doesn't exist. " +
			"Load failed.\n");
    else if (x == -2)
	return GDISPLAY("\"" + fname + "\" is a directory. " +
			"Load failed.\n");
    if (x > 49999)
	GDISPLAY("\"" + fname + "\" is too big (" + x + " bytes)" +
		 " - only partly loaded.\n");
    tmp_buf = explode(read_file(fname), "\n");
    GDISPLAY("\"" + fname + "\" - " + x + " bytes, " + 
	     (y = sizeof(tmp_buf)) + " lines.\n");
    current_file = fname;
    if (y + (last_index + 1) > 1000)
    {
	GDISPLAY("Concatenated text too large for buffer. " +
		 (1000 - (last_index+1)) + " lines added.\n");
	if (1000 - (last_index+1))
	{
	    ed_it_buffer += 
		slice_array(tmp_buf, 0, 1000 - (last_index+2));
	    current_index = last_index;
	    last_index = sizeof(ed_it_buffer) - 1;
	}
    }
    else
    {
	ed_it_buffer += tmp_buf;
	current_index = last_index;
	last_index = sizeof(ed_it_buffer) - 1;
    }
}

void
gedit_write_file(string fname)
{
    string tmp_buf;
    int x;
    if (fname[0] != '/')
	fname = ged_user->query_path() + "/" + fname;
    if ((x = file_size(fname)) == -2)
	return GDISPLAY("\"" + fname + "\" is a directory. " +
			"Save failed.\n");
    else if (x >= 0)
	GDISPLAY("\"" + fname + "\" already existed (" +
		 x + " bytes). Buffer appended to end of file.\n");
    tmp_buf = implode(ed_it_buffer, "\n");
    if (write_file(tmp_buf, fname))
    {
	GDISPLAY("Wrote "); 
	current_file = fname;
    }
    else 
	GDISPLAY("Failed to write ");
    GDISPLAY((last_index + 1) + " lines, " + strlen(tmp_buf) +
	     " bytes to \"" + fname + "\".\n");
}

varargs int
load_file_to_buffer(string fname, mixed buf) /* Hmm, what is buf for? */
{
    int x;
    if ((x = file_size(fname)) < 0)
	return x;
    ed_it_buffer = explode(read_file(fname), "\n");
    current_index = -1;
    last_index = sizeof(ed_it_buffer)-1;
    return x;
}

int
save_file_in_buffer(string fname)
{
    return (write_file(implode(ed_it_buffer, "\n"), fname));
}
  
