/* Groo 91-01-31 */
/**********************************
 * The new, great bulletin board! *
 * (The manic wanderer - version) *
 * (with the nasty bug #1 fixed.) *
 **********************************/

/* Fixed for inheritance by Tintin 91-02-06 */
/* Fixed for 3.0 by Commander 91-08-25 */
/* Types added by Tintin 91-10-12 */
/* Gedit editor included by Groo 91-11-03 */
/* Backup of removed messages added by Groo 91-11-18 
   'look under/behind board', 'read/mread old <msg_number>' */

/* Last time modified : 91-11-28 by Groo */

/* Example of use:
 * inherit "std/board";
 * 
 * create_board()
 * {
 *	set_board_name("w/tintin/my_bulletin");
 *	set_num_notes(10);
 *	set_anonymous(1);
 *	set_silent(0);
 *	set_show_lvl(0);
 *	set_remove_lvl(10);
 *	set_remove_str("Sorry, you can't do that");
 * }
 *
 *	After setting it up this way you just need to clone it and move it
 *	to the location where you want it.
 *
 *	The following options can be set:
 *	set_num_notes(n)	Set max number of notes (default 30)
 *
 *	set_silent(n)		0 = Tell room about read and examine of the 
 *				board 1 = silent read & examine,
 *				(default 1)
 *	set_anonymous(n)	0 = No anonymous posts, 1 = make articles
 *				made by invisible persons come out as written
 *				by 'Someone' (default 0)
 *	set_board_name(name)	Path to the save_file for the board
 *				(no default, this variable MUST be set)
 *	set_remove_lvl(n)	Minimum level required to remove other
 *				players notes (default 24)
 *	set_remove_str(str)	The string to send to the player if a
 *				remove failed. (default 'Only an Archwizard 
 *				can remove other peoples notes')
 *	set_err_log(str)	Name of the log to where error messages
 *				should go. (default 'BOARDS', but you
 *				MUST change it since only /obj and /room
 *				objects can write to this file)
 *	set_show_lvl(n)		0 = Don't show level in note header,
 *				1 = show level of writer in note header
 *				(default 1)
 *	set_use_gedit(n)	0 = Don't use the built-in editor Gedit.
 *				1 = Use the built-in editor Gedit (default 0)
 *	set_keep_discarded(n)	Keep 'n' number of removed notes always.
 *				('look under/behind board' lists old notes
 *				 and 'read old <n>' reads old note number 'n')
 */

inherit "/std/object";

#include "/sys/macros.h"
#include "/sys/stdproperties.h"
#include "/sys/bb_gedit.h"

string tmp_head, tmp_text, new_head, new_text;
mixed *old_messages;
int msg_num;

static string  *messages, *headers;
static int line, looked_at, keep_discarded;
static object curr_writer;
static string board_name, remove_str, err_log;
static int notes, silent, anonymous, remove_lvl, show_lvl, fuse, use_gedit;

/*
 * Prototypes
 */
void error_log(string str);
static void discard_message(string header, string message);
static int no_special_fellow();
string mk_header();
int save_board();
void mread_msg(string message);

/*
 * These functions are made for setting up the board in the file
 * inheriting it.
 */
void
set_num_notes(int n) 
{ 
    if (!fuse) 
      notes = n; 
}

void	
set_silent(int n)
{ 
    if (fuse)
	return;

    silent = (n ? 1 : 0);
}

void
set_anonymous(int n)
{
    if (fuse)
	return;

    anonymous = (n ? 1 : 0); 
}

void
set_board_name(string str)
{
    if (fuse)
	return;

    board_name = str; 
}

void
set_remove_lvl(int n) 
{
    if (fuse)
	return;

    remove_lvl = n; 
}

void
set_remove_str(string str)
{
    if (fuse)
	return;

    remove_str = str; 
}

void
set_err_log(string str)
{
    err_log = str; 
}

void
set_show_lvl(int n) 
{
    if (fuse)
	return;
 
    show_lvl = (n ? 1 : 0);
}

void
set_fuse() 
{ 
    fuse = 1; 
}

void
set_use_gedit(int n) 
{
    use_gedit = (n ? 1 : 0);
}
void
set_keep_discarded(int n)
{
    keep_discarded = (n >= 0 ? n : -n);
}


/* 
 * These functions are hooks that can be replaced with custom functions
 * for selecting who may read, write or remove messages in an inherited
 * board.
 */
varargs static int check_reader() { return 0; }
varargs static int check_writer() { return 0; }
varargs static int check_remove() { return 0; }

public mixed
long() 
{
    string str;
    int ind;
    str = "This is a bulletin board.\n";
    str+="Usage : note <headline>, remove <message number>\n";
    str+="        store <message number> <file name>\n";
    str+="        read/mread [old] <message number> \n";
    if (sizeof(old_messages))
	if (sizeof(old_messages) == 1)
	    str +=
		"You notice an old note that has been left under the board.\n";
	else str +=
	    "You notice some old notes that have been left under the board.\n";
    if (!msg_num) 
	return str +"The board is empty.\n";
    
    str += "The bulletin board contains " + msg_num +
	((msg_num==1)?" note":" notes") + " :\n\n";
    
    if (!silent)
	say(QCTNAME(this_interactive()) + " studies the bulletin board.\n");
    ind = 0;
    while (ind < msg_num) {
	str+= ind + 1 + ":\t" + headers[ind] + "\n";
	ind++;
    }
    return str;
}

public int
no_get()
{
    write("It is firmly secured to the ground.\n");
    return 1;
}

public void
init() 
{
    add_action("new_msg", "note");
    add_action("read_msg", "read");
    add_action("read_msg", "mread");
    add_action("remove_msg", "remove");
    add_action("store_msg", "store");
    add_action("heres_looking_at_you_kid", "look");
    if (!looked_at) {
	int i;
	string *arr;
	messages = allocate(notes);
	headers = allocate(notes);
	looked_at = 1;
	if (!restore_object(board_name)) {
	    error_log("Unable to restore saved board");
	    write("Board : unable to restore saved messages.\n");
	    return;
	}
	arr = explode(new_head, "\n**\n");
	i = 0;
	while(i < sizeof(arr)) {
	    headers[i] = arr[i];
	    i++;
	}
	arr = explode(new_text, "\n**\n");
	i = 0;
	while(i < sizeof(arr)) {
	    messages[i] = arr[i];
	    i++;
	}
	new_text = "";
	new_head = "";
    }
    return;
}

nomask void
reset_object()
{
    if (!random(5)) 
    {
	say("A small gnome appears and secures some " +
	    "notes on the board that were loose.\n");
	say("The gnome leaves again.\n");
    }
    return;
}

nomask void
create_object() 
{
    ::create_object();
    
    notes = 30;
    board_name = "";
    silent = 1;
    anonymous = 0;
    show_lvl = 1;
    remove_lvl = 24;
    keep_discarded = 10;
    use_gedit = 0;
    remove_str = "Only an Archwizard can remove other peoples notes.\n";
    err_log = "BOARDS";
    old_messages = ({ });
    set_name(({"board", "bulletin board", "bulletinboard" }));
    add_prop(OBJ_I_NO_GET, "@@no_get");
    set_short("bulletin board");
    this_object()->create_board();
    call_out("set_fuse",1);
}

static int
new_msg(string msg_head) 
{
    line = 1;
    if (check_writer())
	return 1;
    if (!msg_head)
	return 0;
    if (curr_writer && environment(curr_writer) == environment(this_object())) 
    {
	write(curr_writer->query_name() + " is busy writing.\n");
	return 1;
    }
    if (msg_num == notes) 
    {
	write("You have to remove an old message first.\n");
	return 1;
    }
    if (strlen(msg_head) > 50) 
    {
	write("Message header too long. Try again.\n");
	return 1;
    }
    curr_writer = this_interactive();
    say(QCTNAME(curr_writer) + " starts writing a note.\n");
    tmp_head = msg_head;
    tmp_text = "";
    if (use_gedit) 
    {
	init_ed_it(this_interactive());
	write("Entering Gedit. Type 'help' or '?' for help.\n");
	input_to("gedit_msg");
	GPROMPT();
	return 1;
    }
    input_to("get_msg");
    write("Enter message text. End with '**', abort with '~q'.\n");
    write("1>>");
    return 1;
}

void
gedit_msg(string str) 
{
    string *ret_buf;
    if ((ret_buf = ed_it(str))) 
    {
	if (sizeof(ed_it_buffer) == 0 || !strlen(ed_it_buffer[0])) 
	{
	    tell_object(curr_writer, "No text entered. " +
			"Note aborted.\n");
	    say(QCTNAME(curr_writer) + " aborts writing a note.\n");
	}
	else 
	{
	    tell_object(curr_writer, "Ok.\n");
	    say(QCTNAME(curr_writer) + " has completed " +
		"a note : " + tmp_head + "\n");
	    headers[msg_num] = tmp_head + mk_header();
	    messages[msg_num] = implode(ret_buf, "\n") + "\n";
	    msg_num++;
	    save_board();
	}
	curr_writer = 0;
	init_ed_it(0);
    }
    else
	input_to("gedit_msg");
}

get_msg(string str) 
{
    if (str == "~q") 
    {
	say(QCTNAME(curr_writer) + " aborts writing a note.\n");
	write("Note aborted.\n");
	curr_writer = 0;
	tmp_head = "";
	tmp_text = "";
	return;
    }
    if (str == "**") 
    {
	if (line == 1) 
	{
	    write("No text entered. Message discarded.\n");
	    say(QCTNAME(curr_writer) + " quits writing.\n");
	    curr_writer = 0;
	    tmp_head = "";
	    tmp_text = "";
	    return;
	}
	say(QCTNAME(curr_writer) + " has completed a note : " +
	    tmp_head + "\n");
	headers[msg_num] = tmp_head + mk_header();
	messages[msg_num] = tmp_text + "\n";
	msg_num++;
	save_board();
	write("Ok.\n");
	curr_writer = 0;
	return;
    }
    tmp_text = tmp_text + str + "\n";
    line++;
    write(line + ">>");
    input_to("get_msg");
}

static string
mk_header()
{
    string lvl_text, name;
    lvl_text = "";
    if(show_lvl)
	lvl_text = ", " + this_interactive()->query_wiz_level();
    if (anonymous)
	name = this_interactive()->query_name();
    else
	name = capitalize(this_interactive()->query_real_name());
    
    return "(" + name + ", " + extract(ctime(time()), 4, 9) + 
	lvl_text + ")";
}

public int
read_msg(string what_msg) 
{
    int note, read_discarded;
    if (check_reader())
	return 1;
    if (!what_msg) 
    {
	write("You read a while but as you can't decide exactly\n");
	write("what to read, you don't find much of interest.\n");
	return 1;
    }
    if (sscanf(what_msg, "old %s", what_msg))
	read_discarded = 1;
    if (!sscanf(what_msg, "%d", note))
	if (!sscanf(what_msg, "note %d", note))
	    return 0;
    if (read_discarded) 
    {
	if (note > sizeof(old_messages) || note < 1) {
	    write("You can't find that message.\n");
	    return 1;
	}
    }
    else if (note < 1 || note > msg_num) 
    {
	write("Not that many messages.\n");
	return 1;
    }
    note--;
    if (!silent) 
    {
	if (read_discarded)
	    say(QCTNAME(this_interactive()) + " picks up an old " +
		"note from under the board and reads it.\n" +
		"It is titled '" + old_messages[note][0] + "'.\n");
	else
	    say(QCTNAME(this_interactive()) + " reads a note titled '" +
		headers[note] + "'.\n");
    }
    
    if (read_discarded)
	write("The note is titled '" + old_messages[note][0] +
	      "':\n\n");
    else
	write("The note is titled '" + headers[note] + "':\n\n");
    if (query_verb() == "mread") 
    {
	if (read_discarded)
	    mread_msg(old_messages[note][1]);
	else
	    mread_msg(messages[note]);
    }
    else
	if (read_discarded)
	    write(old_messages[note][1]);
	else
	    write(messages[note]);
    return 1;
}

static void
mread_msg(string message) 
{
    object ob;
    ob = clone_object("/std/board_more");
    ob->move(this_interactive(), 1);
    ob->more(message);
}

static int
remove_msg(int what_msg) 
{
    string player, title, date;
    int note, ind;
    if (check_remove())
	return 1;
    if (!sscanf(what_msg, "%d", note)) 
	if (!sscanf(what_msg, "note %d", note))
	    return 0;
    if (note < 1 || note > msg_num) 
    {
	write("That message doesn't exist.\n");
	return 1;
    }
    note--;
    if (sscanf(headers[note], "%s(%s,%s", title, player, date) != 3) 
    {
	error_log("Header error");
	write("Board : error - header corrupt.\n");
	return 1;
    }
    if ((this_interactive()->query_real_name() != lower_case(player) &&
	 no_special_fellow() &&
	 (this_interactive()->query_real_name() != geteuid(this_object())) &&
	 this_interactive()->query_wiz_level() < remove_lvl) ||
	!query_ip_number(this_interactive())) 
    {
	write(remove_str);
	say(QCTNAME(this_interactive()) + " failed to remove " +
	    "a note.\n");
	return 1;
    }
    say(QCTNAME(this_interactive()) + " removes a note titled '" +
	headers[note] + "'.\n");
    ind = note;
    discard_message(headers[ind], messages[ind]);
    while (ind < msg_num - 1) 
    {
	messages[ind] = messages[ind + 1];
	headers[ind] = headers[ind + 1];
	ind++;
    }
    messages[ind] = 0;
    headers[ind] = 0;
    msg_num--;
    save_board();
    write("Ok.\n");
    return 1;
}

static int
store_msg(string str) 
{
    int note;
    string file;
    
    write("Sorry, storing hasn't been implemented yet.\n");
    return 1; /* This is insecure in 3.0 */
    
    if (this_interactive()->query_wiz_level() < 10) 
    {
	write("Sorry, you can't save messages.\n");
	return 1;
    }
    if (check_reader())
	return 1;
    if (!str) 
	return 0;
    if (sscanf(str, "%d %s", note, file) != 2)
	if (sscanf(str, "note %d %s", note, file) != 2)
	    return 0;
    if (note < 1 || note > msg_num) 
    {
	write("That message doesn't exist.\n");
	return 1;
    }
    note--;
    file += ".note";
    write_file(file, headers[note] + "\n" + messages[note] + "\n");
    return 1;
}

static int
save_board() 
{
    new_head = implode(headers, "\n**\n") + "\n**\n";
    new_text = implode(messages, "\n**\n") + "\n**\n";
    save_object(board_name);
    new_head = "";
    new_text = "";
    return 1;
}


static void
error_log(string str) 
{
    tell_room(environment(this_object()), "Board says '" + str + "'.\n");
    log_file(err_log, "Board : " + str);
    return;
}

heres_looking_at_you_kid(string where) 
{
    int i, sz;
    if (where == "under board" || where == "behind board") {
	say(QCTNAME(this_interactive()) + " crawls under the board " +
	    "in a desperate search for more notes!\n");
	if ((sz = sizeof(old_messages)) == 0)
	    write("There are no discarded messages. Sorry.\n");
	else {
	    write("You see " + sz);
	    if (sz > 1) write(" discarded messages :\n");
	    else write(" single, discarded message :\n");
	    for (i = 0; i < sz; i++) {
		write((i+1) + ":\t" + old_messages[i][0] + "\n");
	    }
	}
	return 1;
    }
}

static void 
discard_message(string header, string message) 
{
    int sz;
    if (!keep_discarded) 
    {
	old_messages = ({ });
	return;
    }
    sz = sizeof(old_messages);
    if (!sz)
	old_messages = ({ ({ header, message }) });
    else if (sz >= keep_discarded) 
    {
	old_messages = 
	    slice_array(old_messages, sz - (keep_discarded - 1), sz) +
		({ ({ header, message }) });
    }
    else
	old_messages += ({ ({ header, message }) });
}

/* 
 * Some people can always remove messages
 */
static int
no_special_fellow()
{
    string name, euid;
    
    name = this_interactive()->query_real_name();
    euid = geteuid(this_object());
    
    /* The board is mine */
    if (name == euid)  
	return 0;
    
    /* The board is my domain's */
    if ("/secure/master"->query_domain_lord(euid) == name)
	return 0;

    return 1;
}
    

