/*
 * This is a decaying corpse. It is created automatically
 * when a player or monster die.
 */
#pragma save_binary

inherit "std/container";
#include <stdproperties.h>
#include <composite.h>
#include <macros.h>
#include <ss_types.h>
#include <wa_types.h>
#include <cmdparse.h>
#include <language.h>

#define DECAY_TIME	10

int             decay;
string          met_name, nonmet_name, state_desc, pstate_desc;
mixed		leftover_list;

void
init()
{
    add_action("get_leftover", "tear");
    add_action("get_leftover", "cut");
    /* I remove this silly eat thing now. When someone add real eat code
       it can be reopened. Nick 
    add_action("eat_it", "eat");
    */
}

void
create_corpse()
{
    ::create_container();
}

nomask void
create_container()
{
    add_prop(OBJ_I_SEARCH_TIME, 2);
    add_prop(OBJ_S_SEARCH_FUN, "search_for_leftovers");

    create_corpse();

    decay = DECAY_TIME;
}

void
reset_corpse()
{
    ::reset_container();
}

nomask void
reset_container()
{
    reset_corpse();
}

void
set_name(string n)
{
    met_name = lower_case(n);
    nonmet_name = (string)previous_object()->query_nonmet_name();
    ::set_name("corpse");
    add_name("corpse of " + met_name);
    add_name("remains");
    add_name("remains of " + met_name);
    if (nonmet_name)
    {
	add_name("corpse of " + nonmet_name);
	add_name("remains of " + nonmet_name);
    }
    set_short("@@short_func");
    set_pshort("@@pshort_func");
    set_long("@@long_func");
    change_prop(CONT_I_WEIGHT, 50000);
    change_prop(CONT_I_VOLUME, 50000);
    call_out("decay_fun", 60);
    state_desc = "corpse of ";
    pstate_desc = "corpses of ";
}

short_func()
{
    object pob;

    pob = vbfc_caller();
    if (!pob || !query_ip_number(pob) || pob == this_object())
	pob = this_player();
    if (pob && pob->query_real_name() == lower_case(met_name))
	return state_desc + "yourself";
    else if (pob && pob->query_met(met_name))
	return state_desc + capitalize(met_name);
    else
	return state_desc + LANG_ADDART(nonmet_name);
}

pshort_func()
{
    object pob;

    pob = vbfc_caller();
    if (!pob || !query_ip_number(pob) || pob == this_object())
	pob = this_player();
    if (pob && pob->query_real_name() == lower_case(met_name))
	return pstate_desc + "yourself";
    else if (pob && pob->query_met(met_name))
	return pstate_desc + capitalize(met_name);
    else
	return pstate_desc + LANG_PWORD(nonmet_name);
}

long_func()
{
    object pob;

    pob = vbfc_caller();
    if (!pob || !query_ip_number(pob) || pob == this_object())
	pob = this_player();
    if (pob->query_real_name() == lower_case(met_name))
	return "This is your own dead body.\n";
    if (pob->query_met(met_name))
	return "This is the dead body of " + capitalize(met_name) + ".\n";
    else
	return "This is the dead body of " + nonmet_name + ".\n";
}

void
decay_fun()
{
    object *ob, obj;
    string desc;
    int i, j, flag;

    call_out("decay_fun", 60);

    if (--decay == 3)
    {
	ob = filter(all_inventory(this_object()), "move_out",
			  this_object());
	desc = COMPOSITE_DEAD(ob);
	if (desc)
	{
	    tell_room(environment(this_object()),
	        "As the " + QSHORT(this_object()) + " rapidly decays " +
			   desc + " appears from it.\n");
	}
	state_desc = "heap with decayed remains of ";
	pstate_desc = "heaps with decayed remains of ";
	add_name("heap");
    }
    
    if (decay > 0)
    {
	call_out("decay_out", 60);
	return;
    }

    for (i = 0; i < sizeof(leftover_list); i++)
    {
	if (leftover_list[i][4])
	{
	    for (j = 0 ; j < leftover_list[i][2] ; j++)
	    {
		obj = clone_object(leftover_list[i][0]);
		obj->leftover_init(leftover_list[i][1], 
				   query_prop(CORPSE_S_RACE));
		obj->move(environment(this_object()), 0);
		flag = 1;
	    }
	}
    }

    if (flag)
        tell_room(environment(this_object()),
              "The " + QSHORT(this_object()) +
                 " rapidly decays, leaving some sad remains behind.\n");

    map(all_inventory(this_object()), "move_out", this_object());
    remove_object();
}

static int
move_out(object ob)
{
    if (ob->move(environment(this_object())))
	return 0;
    else
	return 1;
}

string
query_race()
{
    return query_prop(CORPSE_S_RACE);
}

int
eat_it(string what)
{
    notify_fail("You wouldn't want to eat that.\n");
    if (id(what))
    {
        say( QCTNAME(this_player()) + " eats the " + 
	    QSHORT(this_object()) + "\n");

        if (query_race() == (string) this_player()->query_race_name())
	{
	    write("You feel a bit uneasy doing this, but...\n");
	    this_player()->add_prop("cannibal", 1);
	}
	write("You devour the " + what + ".\n");
	
	if (decay < 2)
	{
	    /* Fix some kind of food disease here later */
	}
	remove_call_out("decay_fun");
	call_out("remove_object", 1);
	return 1;
    }
    return 0;
}

int
appraise_object(int num)
{
    int skill, value, volume, weight, seed;

    if (num)
        skill = num;
    else
        skill = this_player()->query_skill(SS_APPR_OBJ);

    volume = query_prop(CONT_I_VOLUME);
    weight = query_prop(CONT_I_WEIGHT);

    sscanf(OB_NUM(this_object()), "%d", seed);
    skill = 1000 / (skill + 1);
    skill = random(skill, seed);

    weight = cut_sig_fig(weight + (skill % 2 ? -skill % 70 : skill) *
                        weight / 100);
    volume = cut_sig_fig(volume + (skill % 2 ? -skill % 70 : skill) *
                        volume / 100);

    write("\n" + this_object()->long(this_player()) + "\n");
    write(break_string("You appraise that the weight is " + weight +
        " grams and you guess its volume is about " + volume +
        " milliliters.\n", 75));
}

/*
 * Function name: add_leftover
 * Description:   Add leftovers (at death) to the body.
 * Arguments:	  leftover - The leftover list to use.
 */
varargs public void
add_leftover(mixed list)
{
    leftover_list = list;
}

/*
 * Function name: query_leftover
 * Description:   Return the leftover list. If an organ is specified, that
 *		  actual entry is looked for, otherwise, return the entire
 *		  list.
 *		  The returned list contains the following entries:
 *		  ({ objpath, organ, nitems, vbfc, hard, cut})
 * Arguments:	  organ - The organ to search for.
 */
varargs public mixed
query_leftover(string organ)
{
    int i;

    if (!sizeof(leftover_list))
	return ({ });

    if (!strlen(organ))
	return leftover_list;

    for (i = 0 ; i < sizeof(leftover_list) ; i++)
	if (leftover_list[i][1] == organ)
	    return leftover_list[i];
}

/*
 * Function name: remove_leftover
 * Description:   Remove a leftover entry from a body.
 * Arguments:	  organ - Which entry to remove.
 * Returns:       1 - Ok, removed, 0 - Not found.
 */
public int
remove_leftover(string organ)
{
    int i;

    if (!sizeof(leftover_list))
	return 0;

    for (i = 0 ; i < sizeof(leftover_list) ; i++)
	if (leftover_list[i][1] == organ)
	    leftover_list = leftover_list[0..(i - 1)] +
		leftover_list[(i + 1)..(sizeof(leftover_list))];
}

/*
 * Function name: get_leftover
 * Description:   Get leftovers from the body.
 */
public int
get_leftover(string arg)
{
    mixed	corpses, leftovers;
    object 	*found, *weapons, theorgan;
    int		i, slash;
    string	organ, vb, fail;
    
    if (this_player()->query_prop(TEMP_STDCORPSE_CHECKED))
	return 0;

    vb = query_verb(); 
    
    notify_fail(capitalize(vb) + " what from what?\n");  /* access failure */
    if (!arg)
	return 0;

    if (!parse_command(arg, environment(this_player()), "%s 'from' %i",
		organ, corpses))
	return 0;

    found = VISIBLE_ACCESS(corpses, "find_corpse", this_object());
    
    if (sizeof(found) == 1)
    {
	switch(vb)
	{
	case "cut":
	    slash = 0;
	    weapons = this_player()->query_weapon(-1);
	    for (i = 0 ; i < sizeof(weapons) ; i++)
		if (weapons[i]->query_dt() & W_SLASH)
		    slash = 1;
	    break;

	case "tear":
	default:
	    slash = 0;
	    break;
	}

	if (vb == "cut" && slash == 0)
	{
	    notify_fail("@@get_fail:" + file_name(this_object()) +
		"|Better find something sharper than _that_ to cut with.\n@@");
	    return 0;
	}

	leftovers = query_leftover(organ);
	if (!sizeof(leftovers) || leftovers[2] == 0)
	{
	    notify_fail("@@get_fail:" + file_name(this_object()) +
		"|There's no such thing to get from the corpse.\n@@");
	    return 0;
	}

	if (leftovers[5] && slash == 0)
	{
	    notify_fail("You can't just tear this loose, use a knife or " +
			"something...\n");
	    return 0;
	}

	if (strlen(leftovers[3]))
	    fail = check_call(leftovers[3]);

	if (strlen(fail))
	{
	    notify_fail("@@get_fail:" + file_name(this_object()) + "|" +
		fail + ".\n@@");
	    return 0;
	}

	if(leftovers[2]-- == 0)
	    remove_leftover(leftovers[1]);

	theorgan = clone_object(leftovers[0]);
	theorgan->leftover_init(leftovers[1], query_prop(CORPSE_S_RACE));
	theorgan->move(this_player(), 0);

	say(QCTNAME(this_player()) + " " + vb + "s " + LANG_ADDART(organ) +
	    " from " + QSHORT(found[0]) + ".\n");
	write("You " + vb +" " + LANG_ADDART(organ) + " from a corpse.\n");
	return 1;
    }

    call_out("tp_remove_temp", 1, this_player());
    this_player()->add_prop(TEMP_STDCORPSE_CHECKED, 1);
    if (sizeof(found))
	notify_fail("@@get_fail:" + file_name(this_object()) +
	    "|Please try to be more specific with what corpses you choose.\n@@");
    else
	notify_fail("@@get_fail:" + file_name(this_object()) +
	    "|There is no such corpse lying around here.\n@@");
    return 0;
}

void
tp_remove_temp(object tp)
{
    tp->remove_prop(TEMP_STDCORPSE_CHECKED);
    remove_call_out("tp_remove_temp");
}

string
get_fail(string fault)
{
    tp_remove_temp(this_player());
    return fault;
}

public int
find_corpse(object ob)
{
    if (file_name(ob)[0..10] == "/std/corpse" &&
	(environment(ob) == this_player() ||
	 environment(ob) == environment(this_player())))
	return 1;

    return 0;
}


/*
 * Function name: search_for_leftovers
 * Description:   This function is called when someone searches the corpse
 *		  as set up in create_container()
 * Arguments:	  player - The player searching
 *		  str    - If the player specifically said what to search for
 * Returns:       A string describing what, if anything the player found.
 */
string
search_for_leftovers(object player, string str)
{
    mixed left;
    string *found;
    int i;

    left = query_leftover();
    found = ({});

    for (i = 0; i < sizeof(left); i++)
    {
	if (left[i][2] == 1)
	    found += ({ LANG_ADDART(left[i][1]) });
	else if (left[i][2] > 1)
	    found += ({ LANG_WNUM(left[i][2]) + " " + LANG_PWORD(left[i][1]) });
    }

    if (sizeof(found) > 0)
	return break_string("After searching the corpse you believe you " +
		"can tear out or cut off " + COMPOSITE_WORDS(found) + ".\n", 76);

    return "";
}

