/*
  domain_link.c

  This is the standard code called when rebooting the Game for each domain.
  Inherit this object for your domain link object.

  Note that there is now two ways to link your domain to the world, either
  as a description and direction in someones room, or as a direct link in the
  global map. Terrain 'rooms' should inherit /std/mapsquare.c or some 
  of its descendants, and use 'exceptions' to link normal rooms to the map.
  Map managers should inherit /std/map/manager

  Note also that you should use add_preload() for preloading of any files
  in the domain, such as wizardfiles. This is so that userid will get set
  correctly.

  Typical usage in your preload file:

     inherit "/std/domain_link";

     create_domain()
     {
	 |* Use this to link the domain as an exit from another room
	 *|
         set_place("/d/Standard/void");   
         set_desc("There is a small path to the south.\n");
         set_dir("south");
         set_entry("/d/Domain/common/entrance");

	 |* Or you might want a link to the map
	 *|
         set_mapmanager("/d/Domain/common/maplink");

	 |* And you might want to preload some wizard files
	 *|
	 add_preload("/d/Domain/wizard1/preload");
	 add_preload("/d/Domain/wizard2/preload");
     }


 */
#pragma save_binary

inherit "/std/object";  

#include "/secure/std.h"

static   string   dom_place,         /* From where you enter the domain */
                  dom_desc,          /* The added description to that place */
                  dom_dir,           /* The command given to move */
                  dom_entry,         /* Which room to move the player to */
		 *dom_preload;	     /* What wizard files to load */

static string map_file;

set_place(str) { dom_place = str; }
set_desc(str) { dom_desc = str; }
set_dir(str) { dom_dir = str; }
set_entry(str) { dom_entry = str; }

set_mapmanager(string file) { map_file = file; }

add_preload(file) { dom_preload += ({ file }); }

public nomask
create_object()
{
    int il;

    seteuid(getuid(this_object()));

    dom_preload = ({});
    create_domain();

    if ((dom_place) && (dom_dir) && (dom_entry)) 
    {
	dom_place->add_exit(dom_entry,dom_dir,0);
	if (dom_desc)
	{
	    dom_entry->teleledningsanka();
	    dom_place->add_my_desc(dom_desc, dom_entry);
	}
    }
    if (strlen(map_file))
	SECURITY->set_map_manager(map_file);
	
    if (sizeof(dom_preload))
	for (il = 0; il < sizeof(dom_preload); il++) 
	{
	    seteuid(SECURITY->creator_file(dom_preload[il]));
	    write("\tPreloading: " + dom_preload[il] + ".c\n");
	    dom_preload[il]->teleledningsanka();
	}
    call_out("remove_object",1); 
	
}

/* Default */
public create_domain() {}
