/*
   /std/food.c
   
   This is the standard object used for any form of eatable things


 Usage:

 	inherit "/std/food";

	create_food()
	{
	    set_amount(amount_in_gram);
	    set_name("name of food");
	    set_short(...);
	    set_long(....);
	}
*/
#pragma save_binary

inherit "/std/heap";

#include <cmdparse.h>
#include <composite.h>
#include <macros.h>
#include <stdproperties.h>

static	int	food_amount;
static	object	*gFail;

/*
 * Prototypes
 */
void consume_them(object *arr);

public void
create_food()
{
    set_name("food");
}

public nomask void
create_heap()
{
    set_heap_size(1);

    create_food();
}

public void
set_long(string long)
{
    ::set_long(long);

    if (!query_prop(HEAP_S_UNIQUE_ID))
        add_prop(HEAP_S_UNIQUE_ID, long);
}

/*
 * Function name:	set_amount
 * Description:		sets the amount of food in this food (in grams)
 * Arguments:		a: The amount of food
 */
public void
set_amount(int a) 
{ 
    food_amount = a; 
    add_prop(OBJ_I_VOLUME, a / 10);
    add_prop(OBJ_I_WEIGHT, a);
}

/*
 * Function name:	query_amount
 * Description:		Gives the amount of food in this food
 * Returns:		Amount as int (in grams)
 */
public int
query_amount() { return food_amount; }

public void
init()
{
    add_action("eat_it", "eat");
}

/*
 * Function name: consume_text
 * Description:   Here the eat message is written. You may redefine it if
 *		  you wish.
 * Arguments:     arr - Objects being consumed
 *		  vb  - The verb player used to consume them
 */
void
consume_text(object *arr, string vb)
{
    string str;

    write("You " + vb + " " + (str = COMPOSITE_DEAD(arr)) + ".\n");
    say(QCTNAME(this_player()) + " " + vb + "s " + str + ".\n");
}

/*
 * Function name:	eat_it
 * Description:		Eats the objects described as parameter to 'eat'. It
 *			uses command parsing to find which objects to eat.
 * Arguments:		str: The trailing command after 'eat ...'
 * Returns:		True if command successfull
 */
public int
eat_it(string str)
{
    object 	*a, *foods;
    int		il;
    string str2, vb;
    
    if (this_player()->query_prop(TEMP_STDFOOD_CHECKED))
	return 0;

    gFail = ({ });
    vb = query_verb(); 
    
    notify_fail(capitalize(vb) + " what?\n");  /* access failure */
    
    a = CMDPARSE_ONE_ITEM(str, "eat_one_thing", "eat_access");
    if (sizeof(a) > 0)
    {
	consume_text(a, vb);
	for (il = 0; il < sizeof(a); il++)
	    a[il]->destruct_object();
	return 1;
    }
    else
    {
	call_out("tp_remove_temp", 1, this_player());
	this_player()->add_prop(TEMP_STDFOOD_CHECKED, 1);
	if (sizeof(gFail))
	    notify_fail("@@eat_fail:" + file_name(this_object()));
	return 0;
    }
}

void
tp_remove_temp(object tp)
{
    tp->remove_prop(TEMP_STDFOOD_CHECKED);
    remove_call_out("tp_remove_temp");
}

string
eat_fail()
{
    string str;

    str = "You try to eat " + COMPOSITE_DEAD(gFail) + " but fail.\n";
    say(QCTNAME(this_player()) + " tries to eat " + QCOMPDEAD +
	    " but fails.\n");
    tp_remove_temp(this_player());
    return str;
}

int
eat_access(object ob)
{ 
    if ((environment(ob) == this_player()) &&
	(function_exists("create_heap", ob) == "/std/food") &&
	(ob->query_short()))
	return 1;
    else
	return 0;
}

int
eat_one_thing(object ob)
{
    int am, num, i;

    am = (int) ob->query_amount();
    num = (int) ob->num_heap();

    for (i = 1; i <= num; i++)
    {
        if (!this_player()->eat_food(am))
        {
	    if (i == 1)
	    {
		ob->split_heap(1);
		write(capitalize(ob->short()) + " is too much for you.\n");
		gFail += ({ ob });
	    	return 0;
	    }
	    ob->split(i - 1);
	    return 1;
	}
    }

    return 1;
}

void
destruct_object()
{
    if (leave_behind > 0)
	set_heap_size(leave_behind);
    else
    	call_out("remove_object", 1);
}

/*
 * Function name: stat_object
 * Description:   This function is called when a wizard wants to get more
 *                information about an object.
 * Returns:       str - The string to write..
 */
string
stat_object()
{
    string str;

    str = ::stat_object();

    str += "Food: " + food_amount + " (grams).\n";

    return str;
}

