/*
    /std/living/gender.c

    This is a subpart of living.c
    All gender processing routines are defined here.

    This file is included into living.c
*/

#include "/sys/macros.h"
#include "/sys/const.h"

static nomask void
gender_reset()
{
    /* Illegal default value to force a change. 
    */
    PROFILE("gender_reset");
    this_object()->set_gender(-1); 
}

/*
 * Function name:   query_gender_string
 * Description:     Gives back a string that contains the gender of a living
 * Returns:         The string
 */
public string
query_gender_string()
{
    PROFILE("query_gender_string");
    return LD_GENDER_MAP[query_gender()];
}

/*
 * Function name:   query_pronoun
 * Description:     Returns the pronoun that goes with the gender of this
 *                  living.
 * Returns:         "he", "she" or "it", depending on gender.
 */
public string
query_pronoun()
{
    PROFILE("query_pronoun");
    return LD_PRONOUN_MAP[query_gender()];
}

/*
 * Function name:   query_possessive
 * Description:     Returns the possessive that goes with the gender of this
 *                  living.
 * Returns:         "his", "her" or "its", depending on gender.
 */
public string
query_possessive()
{
    PROFILE("query_possessive");
    return LD_POSSESSIVE_MAP[query_gender()];
}

/*
 * Function name:   query_objective
 * Description:     Returns the objective that goes with the gender of this
 *                  living.
 * Returns:         "him", "her" or "it", depending on gender.
 */
public string
query_objective()
{
    PROFILE("query_objective");
    return LD_OBJECTIVE_MAP[query_gender()];
}
