/*
    /std/living/heart_beat.c

    This is a subpart of living.c
    All heart_beat relevant routines are defined here.

    This file is included into living.c
*/

/* You can disable heart_beat completely with this define. */
#undef HEART_NEEDED 

#include <config.h>             /* for STATUE_WHEN_LINKDEAD */
#include <macros.h>

static int stop_time;		/* Time when heart_beat was stopped */

static void heart_beat(int num);

/*
 * Function name:   start_heart
 * Description:     Starts the heart beat for the living object and sees to
 *                  it that it heals as much as it shall due to lost time.
 */
static void
start_heart()
{
#ifdef HEART_NEEDED
    PROFILE("start_heart");
    if (stop_time)
	heart_beat(F_NUM_BEATS(time() - stop_time));
    stop_time = 0;
    set_heart_beat(1);
#endif
}

/*
 * Function name: stop_heart
 * Description:   Stop the heart beat for the living object and mark
 *                the time for later reference.
 */
static void
stop_heart()
{
#ifdef HEART_NEEDED
    PROFILE("stop_heart");
    if (!stop_time)
	stop_time = time();
    set_heart_beat(0);
#endif
}


/*
 * Function name:   heart_beat
 * Description:     Perform all heart_beat routines.
 * Arguments:       num: Number of lost heart beats, sent as argument
 *			 If num == 0, it's a normal heart_beat
 */
static void
heart_beat(int num)
{
    PROFILE("heart_beat");
#ifdef HEART_NEEDED

        call_out("restart_heart",1);
/* Call any heart_beat functions here. */
        remove_call_out("restart_heart");
    }
#endif
}

void restart_heart() 
{ 
#ifdef HEART_NEEDED
    set_heart_beat(1); 
#endif
}
