/*
 * Clone and move this object to a player if you want to paralyze him.
 *
 * Made by Nick
 */

inherit "/std/object";

/*
 * Variables
 */
string	stop_verb,	/* What verb to stop this paralyze ? */
	stop_fun,	/* What function to call when stopped */
	fail_message,   /* Message to write when command failed */
	stop_message;	/* Message to write when paralyze stopped */
int	remove_time;	/* Shall it go away automatically? */
object	stop_object;	/* Object to call stop_fun in when stopped */

/*
 * Prototypes
 */
void set_standard_paralyze(string str);

/*
 * Function name: create_paralyze
 * Description:	  Set up standard paralyze
 */
void
create_paralyze()
{
    set_standard_paralyze("paralyze");
}

/*
 * Function name: create_object
 * Description:   The standard create routine.
 */
nomask void
create_object() { create_paralyze(); }

/*
 * Function name: init
 * Description:   Called when meeting an object
 */
void
init()
{
    if (remove_time)
	call_out("stop_paralyze", remove_time);

    add_action("stop", "", 1);
}

/*
 * Function name: stop
 * Description:   Here all commands the player gives comes.
 * Argument:	  str - The extra string the player sent.
 */
varargs int
stop(string str)
{
    /* Only paralyze our environment */
    if (environment(this_object()) != this_player())
	return 0;

    if (stop_verb && query_verb() == stop_verb)
    {
    /* If a stop_fun is defined it MUST return 1 to not stop the paralyze */
	if (stop_object && call_other(stop_object, stop_fun, str))
	    return 1;

	call_out("remove_object", 1);

	if (stop_message)
	    this_player()->catch_msg(stop_message);

	return 1;
    }

    if (fail_message)
	this_player()->catch_msg(fail_message);

    if (!this_player()->query_wiz_level())
	return 1;

    write("Since you are a wizard this paralyze won't affect you.\n");
    return 0;
}

/*
 * Function name: dispel_magic
 * Description:   If this paralyze should be able to lift magical, define this
 *		  function.
 * Arguments:     i - A number indicating how strong the dispel spell is.
 * Returns:	  1 if dispelled otherwise 0.
 */

/*
 * Description: Set the verb to stop paralyze, if possible.
 */
void set_stop_verb(string verb) { stop_verb = verb; }

/*
 * Description: Return the stopping verb.
 */
string query_stop_verb() { return stop_verb; }

/*
 * Description: Set function to call when paralyze stops.
 */
void set_stop_fun(string fun) { stop_fun = fun; }

/*
 * Description: Returns the function to call when paralyze stops
 */
string query_stop_fun() { return stop_fun; }

/*
 * Description: Set which object to call the stop function in
 */
void set_stop_object(object ob) { stop_object = ob; }

/*
 * Description: Returns which object to call the stop function in.
 */
object query_stop_object() { return stop_object; }

/*
 * Description: Set the fail message when player tries to do something
 */
void set_fail_message(string message) { fail_message = message; }

/*
 * Description: returns the fail message when player tries to do something
 */
string query_fail_message() { return fail_message; }

/*
 * Description: Set how long time player should be paralyzed.
 */
void set_remove_time(int time) { remove_time = time; }

/*
 * Description: Returns the paralyze time
 */
int query_remove_time() { return remove_time; }

/*
 * Description: Set the message written when paralyze stops.
 */
void set_stop_message(string message) { stop_message = message; }

/*
 * Description: Returns the message written when paralyze stops.
 */
string query_stop_message() { return stop_message; }

/*
 * Function name: set_standard_paralyze
 * Description:   Set up standard settings for a paralyze
 * Argumnets:	  str - String to be used when calculate standard setting
 */
void
set_standard_paralyze(string str)
{
    set_stop_verb("stop");
    set_stop_fun("stop_paralyze");
    set_stop_object(previous_object());
    set_stop_message("You stop " + str + ".\n");
    set_fail_message("You are busy with other things right now. You must " +
		"'stop' to do something else.\n");
}

/*
 * Function name: stop_paralyze
 * Description:	  This function is called if the paralyze shall stop due to 
 *		  the time running out.
 */
varargs void
stop_paralyze()
{
    if (stop_object && stop_fun)
	call_other(stop_object, stop_fun, environment(this_object()));
    else if (stop_message && environment(this_object()))
	environment(this_object())->catch_msg(stop_message);

    remove_object();
}

/*
 * Description: Function called when wiz tries to stat this object.
 */
string
stat_object()
{
    string str;

    str = ::stat_object();

    if (strlen(stop_verb))
	str += "Stop verb: " + stop_verb + "\n";
    if (strlen(stop_fun))
	str += "Stop fun:  " + stop_fun + "\n";
    if (strlen(stop_message))
	str += "Stop mess: " + stop_message + "\n";
    if (strlen(fail_message))
  	str += "Fail mess: " + fail_message + "\n";
    if (remove_time)
	str += "Duration:  " + remove_time + "\n";
    if (stop_object)
	str += "Stop obj:  " + file_name(stop_object) + "\n";

    return str;
}

