/*
  /std/player/savevars_sec.c

  This is a subpart of player_sec.c

  All the variables that are to be saved to the players save file
  are defined here.

*/

#include <macros.h>
#include <std.h>
#include <language.h>

int notmet_me(object ob);
string query_nonmet_name();

private string	name,			/* Name of the player */
                password,        	/* Password of the player */
	        player_file,		/* Name of file used as player file */
  
                path,			/* Current directory path */
                mailaddr,		/* Email adress of the player */
		adj_desc,		/* The adjectives describing player */
                default_start_location,	/* Start location of player */
                temp_start_location,	/* Temp start location */
                domain,			/* The name of the domain */
		login_from,		/* Host wherefrom last logged in */
		*auto_load,             /* Automatically loaded objects */
		*auto_shadow,           /* Our loyal shadows */
		*recover_list,		/* The recover list */
		*bit_savelist,		/* Saved bits */
		*seconds;		/* Second characters */
private mixed	saved_props;		/* List of saved properties */
private int     brief,			/* Flag deciding short or long desc. */
                tot_value,		/* Accumulated value of inventory */
		get_echo,		/* Wants to get echoes of say and such */
		notify,			/* Notify status */
		login_time,		/* Last time logging in */
		history_len;		/* The history length */

private static string
		cap_name;		/* Capitalized name of the player */
static int	*bit_wizlist,		/* The runtime bit wizardlist */
                *bit_bitlist;		/* The runtime bit bitlist */

private mapping m_sanctioned,		/* Sanctioned wizards */
		m_remember_name,	/* Names of players we have met */
		m_alias_list;		/* Aliases for the quicktyper */

nomask public int remove_autoshadow(mixed shadowfile);

/* 
 * Name of the player.
 * This can only be called from player_sec.
 */
static /* private */ nomask void
set_name(string n)
{
    name = n;
    cap_name = capitalize(n);

    ::set_name(name);
}

static void
add_name(mixed name)
{
    ::add_name(name);
}

/* 
 * Function name:   master_set_name
 * Description:     /secure/master needs to be able to do set_name. This
 *                  function can only be called from that object.
 * Arguments:       n: name to set
 */
nomask void
master_set_name(string n)
{
    if (previous_object() != find_object(SECURITY))
	return;

    name = n;
    cap_name = capitalize(n);

    ::set_name(name);
}

/*
 * Function name:   remove_name
 * Description:     Remove a certain name
 * Arguments:       n: A string or a pointer of strings of names
 */
nomask void
remove_name(mixed n) { ::remove_name(n); }

/*
 * Function name:   set_adj
 * Description:     'go through function' for storing the adjectives. Also
 *                  sets the adjectives to 'adj_desc' if no arg given.
 *                  No more than 2 adjectives of a total length of 35
 *                  characters is allowed.
 * Arguments:       str: Adjectives to pass to ::set_adj() or 0.
 * Returns:         1 if setting was succesfull, 0 otherwise
 */
public int
set_adj(string *arr)
{
    string *adj;
    int i, len;

    if (arr) 
    {
	adj = query_adj(1);

	if (sizeof(adj) + sizeof(arr) > 2)
	    return 0;

	for (i = 0; i < sizeof(adj); i++)
	    len += strlen(adj[i]);
	for (i = 0; i < sizeof(arr); i++)
	    len += strlen(arr[i]);
	if (len > 33)
	    return 0;

	::set_adj(arr);
	adj_desc = query_adj(1);
	return 1;
    }
    else 
	::set_adj(adj_desc);
    
    return 1;
}

/*
 * Function name:   remove_adj
 * Description:     'go through function' for removing the adjectives.
 * Arguments:       str: The adjective string to remove.
 */
public
remove_adj(string str)
{
    ::remove_adj(str);
    adj_desc = query_adj(1);
}

/*
 * Function name:   query_real_name
 * Description:     Gives back the real name of a player, e.g. "fatty"
 * Returns:         The lowercased name string
 */
nomask public string
query_real_name()
{
    return name;
}

/*
 * Password
 */

/*
 * Function name:   login_set_password
 * Description:     Set the password of a player. This function may only be
 *                  called by /secure/login.c
 * Arguments:       p: The new password string
 */
nomask void       
login_set_password(string p)
{
    string fname;
    if (sscanf(file_name(previous_object()), "/secure/login#%s", fname) != 1)
	return;
    password = p;
}

/*
 * Function name:   set_password
 * Description:     Set the password of a player
 * Arguments:       p: The new password string
 */
nomask static /* private */ void     
set_password(string p)
{
    password = p;
}

/*
 * Function name:   query_password
 * Description:     Gives back the password of a player
 * Returns:         The password string
 */
nomask public string 
query_password()
{
    return password;
}

/*
 * Function name:   set_player_file
 * Description:     Set the playerfile that is loaded when the player enters
 *                  the game. The lagality of the file is checked against
 *		    legal_player in the 'login_new_player' file defined in
 *		    <config.h>
 * Arguments:       f: The filename-string.
 */
nomask public void
set_player_file(string f)
{
    object ob;
    
    catch(f->teleledningsanka());

    ob = find_object(f);
    if (ob && LOGIN_NEW_PLAYER->legal_player(ob))
	player_file = f;
}

/*
 * Function name:   query_player_file
 * Description:     Gives back the playerfile that is loaded when the player
 *                  enters the game.
 * Returns:         The string with the filename
 */
nomask public string
query_player_file()
{
    return player_file;
}

/*
 * Function name:   query_auto_load
 * Description:     Gives back the array of strings with objects to autoload
 *                  when the player enters the game.
 * Returns:         An array of strings describing what objects to load.
 */
nomask string *
query_auto_load()
{
    return auto_load;
}

/*
 * Function name:   query_recover_list
 * Description:     Gives back the array of strings with objects to recover
 *                  when the player enters the game.
 * Returns:         An array of strings describing what objects to recover,
 *		    along with the 
 */
nomask string *
query_recover_list()
{
    return recover_list;
}

/*
 * Function name:   query_path
 * Description:     Gives back the path of the current directory.
 * Returns:         The path string
 */
public string
query_path()
{
    if (path) return path;
    else if (query_wiz_level())
	path = (string) SECURITY->query_wiz_path(query_real_name());
    return path;
}

/*
 * Function name:   query_mailaddr
 * Description:     Gives back the Email address of a player.
 * Returns:         The Email address string
 */
public string
query_mailaddr()
{
    return mailaddr;
}

/*
 * Function name:   query_domain
 * Description:     Gives back the name of the domain the player is a member
 *                  of. Only wizard can be member of a domain.
 * Returns:         The name of the domain (if any).
 */
public string
query_domain()
{
    return (string)SECURITY->query_wiz_dom(query_real_name()); /* domain; */
}

/*
 * Function name:   query_cap_name
 * Description:     Gives back the capitalized name of a player.
 * Returns:         The capitalized name string
 */
public string
query_cap_name()
{
    return cap_name;
}

/*
 * Function name:   query_default_start_location
 * Description:     Gives back the default start location of a player when
 *                  she enters the game.
 * Returns:         The string with the filename of the startup-room.
 */
public string
query_default_start_location()
{
    return default_start_location;
}

/*
 * Function name:   query_temp_start_location
 * Description:     Gives back the temporary start location of a player when
 *                  she enters the game.
 * Returns:         The string with the filename of the startup-room.
 */
public string
query_temp_start_location()
{
    return temp_start_location;
}

/*
 * Function name:   query_remember_name
 * Description:     Gives back a mapping with all names that a player has
 *                  remembered.
 * Returns:         The mapping with all names.
 */
public mapping
query_remember_name()
{
    if (mappingp(m_remember_name))
	return m_remember_name;
    else
	return ([ ]);
}

/*
 * Function name:   query_brief
 * Description:     Returns the state of the brief-flag.
 * Returns:         true if brief,
 *                  false if verbose.
 */
public int
query_brief()
{
    return brief;
}

/*
 * Function name:   query_tot_value
 * Description:     Gives back the total value of all that is carried
 * Returns:         the total carried value
 */
public int
query_tot_value()
{
    return tot_value;
}

/*
 * Function name:   set_auto_load
 * Description:     Sets the array with autoload strings. Auytoload strings
 *                  look like "<file>:<arg>".
 * Arguments:       arr: An array with autoload strings.
 */
set_auto_load(string *arr)
{
    auto_load = arr;
}

/*
 * Function name:   set_recover_list
 * Description:     Sets the array with recover strings. Recover strings
 *                  look like "<file>:<arg>".
 * Arguments:       arr: An array with autoload strings.
 */
set_recover_list(string *arr)
{
    recover_list = arr;
}

/*
 * Function name:   set_path
 * Description:     Sets the current path of a player.
 * Arguments:       str: Pathstring
 */
public void
set_path(string str)
{
    path = str;
}

/*
 * Function name:   set_mailaddr
 * Description:     Sets the Email address of a player
 * Arguments:       addr: The Email address string
 */
public void
set_mailaddr(string addr)
{
    mailaddr = addr;
}

/*
 * Function name:   set_domain
 * Description:     Set the domain of a player. This can only be called by
 *                  /secure/master.c
 * Arguments:       dom: The string of the new domain.
 * Returns:         1 upon success, 0 upon failure
 */
public int
set_domain(string dom)
{
    if (previous_object() != find_object(SECURITY))
	return 0;
    domain = dom;
    return 1;
}

/*
 * Function name:   set_cap_name
 * Description:     Sets the capitalized name of a player. This is derived
 *                  from query_real_name(), which returns a lowercase name.
 */
public void
set_cap_name()
{
    cap_name = capitalize(query_real_name()); 
}

/*
 * Set the default place where to start when you log on.
 *  Only one per major guild.
 */

/*
 * Function name:   set_default_start_location
 * Description:     Sets a new default startup location for a player. The
 *                  default startup-location must have been approved of by
 *                  an archwizard or keeper.
 * Arguments:       str: the startup room's filename string
 * Returns:         0 if the string was not an accepted location,
 *                  1 when set.
 */
public int
set_default_start_location(string str)
{
    if (!query_wiz_level() && 
	SECURITY->check_def_start_loc(str) < 0)
	return 0;

    default_start_location = str;
    return 1;
}

/*
 * Set the place to start at next time you log in only. May only be used after
 * applying to use it.
 */

/*
 * Function name:   set_temp_start_location
 * Description:     Sets a new temporary startup location for a player. The
 *                  next time the player logs in, she will enter the game at
 *                  the set temporary location, which is then discarded. The
 *                  temporary startup-location must have been approved of by
 *                  an archwizard or keeper.
 * Arguments:       str: the startup room's filename string
 * Returns:         0 if the string was not an accepted location,
 *                  1 when set.
 */
public int
set_temp_start_location(string str)
{
    if (str && SECURITY->check_temp_start_loc(str) < 0)
	return 0;
    temp_start_location = str;
    return 1;
}

/*
 * Function name:   set_remember_name
 * Description:     Sets the people who are remembered by a player.
 * Arguments:       nlist: A mapping with names of remembered players
 */
public void
set_remember_name(mapping nlist)
{
    m_remember_name = nlist;
}

/*
 * Function name:   set_brief
 * Description:     Sets the brief flag.
 * Arguments:       0 = set verbose mode, otherwise set brief mode.
 */
public void
set_brief(int br)
{
    brief = br;
}

/*
 * Function name:   set_get_echo
 * Description:	    Sets the echo flag
 * Arguments:	    0 = Ok. 1 = echo message with say and tell and such
 */
public void
set_get_echo(int flag)
{
    get_echo = flag;
}

/*
 * Function name:   query_get_echo
 * Description:     Query the echo flag
 * Returns:	    The flag (int)
 */
public int
query_get_echo()
{
    return get_echo;
}

/*
 * Function name:   set_notify
 * Description:     Set notify status
 * Arguments:	    flag - The current status
 */
public void
set_notify(int flag)
{
    notify = flag;
}

/*
 * Function name:    query_notify
 * Description:	     Query the notify status
 * Returns:	     The setting
 */
public int
query_notify()
{
    return notify;
}

/*
 * Function name:   set_tot_value
 * Description:     Sets the total value carried by a player
 * Arguments:       val: The carried value
 */
static nomask void
set_tot_value(int val)
{
    tot_value = val;
}

/*
 * Function name:   set_login_time
 * Description:     sets the time of the login
 */
static nomask void
set_login_time() 
{ 
    login_time = time(); 
}

/*
 * Function name:   query_login_time
 * Description:     Gives back the login-time.
 * Returns:         The login-time.
 */
public int
query_login_time() 
{ 
    return login_time; 
}

/*
 * Function name:   set_login_from
 * Description:     Sets from which site the player is logged in.
 */
public void
set_login_from() 
{ 
    login_from = query_ip_name(this_object()); 
}

/*
 * Function name:   query_login_from
 * Description:     shows from which site the player is logged in
 * Returns:         A string with the site name.
 */
public string
query_login_from() 
{ 
    return login_from; 
}

/*************************************************************************
 * 
 * Sanctioning routines.
 *
 */

/*
 * Function name:   set_sanctioned
 * Description:     Set the 'sanctioned' field in the player. This function
 *                  may only be called from the /secure/master
 * Arguments:       arr - The array to set.
 */
nomask public
set_sanctioned(mapping map)
{
    if (previous_object() != find_object(SECURITY))
	return;

    m_sanctioned = map;
}

/*
 * Function name:   query_sanctioned
 * Description:	    Gives back the 'sanctioned' field in the player.
 * Returns:         The sanctioned mapping
 */
nomask public mapping
query_sanctioned()
{
    if (!mappingp(m_sanctioned))
	m_sanctioned = ([]);

    if (SECURE_ARRAYS)
	return ([]) + m_sanctioned;
    else
	return m_sanctioned;
}

/*
 * Function name:   query_sanctioning
 * Description:	    Gives back how a player has sanctioned another player.
 *                  A player can be sanctioned "r", "w", "s" or a combination
 *                  of those.
 * Arguments:       s_taker - The name of the sanctioned player
 * Returns:         A string with what the sanctioned player can do, eg. "rw"
 */
nomask public string
query_sanctioning(string s_taker)
{
    if (!mappingp(m_sanctioned))
	m_sanctioned = ([]);

    return "" +  m_sanctioned[s_taker];
}

/*************************************************************************
 * 
 * Auto shadow routines.
 *
 */

/*
 * Function name:   add_autoshadow
 * Description:	    Add a shadow to the shadow list.
 * Arguments:       shadowfile: the shadow-object or the filename of the
 *                              shadow-object.
 * Returns:         1 if the adding was successfull,
 *                  0 otherwise.
 */
nomask public int
add_autoshadow(mixed shadowfile)
{
    string *sh;
/* I temporarily remove this because of guild style changes, I'll put this check
back in in time /Nick 
    if (query_wiz_level())
    {
	if (geteuid(previous_object()) != geteuid(this_object()) &&
	    geteuid(previous_object()) != (string)SECURITY->get_root_uid())
	    return 0;
    }
*/

    if (objectp(shadowfile))
	shadowfile = MASTER_OB(shadowfile);

    sh = explode(shadowfile, ":");
    if (sizeof(sh) > 1 && strlen(sh[1]) > 20)
	return 0;

    /*
     * There can only be one.
     */
    while(remove_autoshadow(shadowfile));

    if (!sizeof(auto_shadow))
	auto_shadow = ({ shadowfile });
    else
	auto_shadow += ({ shadowfile });
    return 1;
}

/*
 * Function name:   remove_autoshadow
 * Description:	    Remove a shadow from the shadow list
 * Arguments:       shadowfile: The shadow-object or the filename of the
 *                              shadow_object.
 * Returns:         1 if the removing was successfull,
 *                  0 otherwise
 */
nomask public int
remove_autoshadow(mixed shadowfile)
{
    string *sh;
    int i;

    if (objectp(shadowfile))
	shadowfile = MASTER_OB(shadowfile);

    sh = explode(shadowfile, ":");

    for (i = 0 ; i < sizeof(auto_shadow) ; i++)
    {
	if (sh[0] == explode(auto_shadow[i], ":")[0])
	{
	    auto_shadow = exclude_array(auto_shadow, i, i);
	    return 1;
	}
    }
    return 0;
}

/*
 * Function name:   query_autoshadow_list
 * Description:	    Gives back the list of autoshadow object filenames.
 * Returns:         The autoshadow list
 */
nomask public string *
query_autoshadow_list()
{
    if (SECURE_ARRAYS)
	return (auto_shadow ? auto_shadow + ({}) : 0);
    else
	return auto_shadow;
}


/*************************************************************************
 * 
 * Bit handling routines.
 *
 */

/*
 * Function name:   set_bit
 * Description:     Set a given bit in a given group. The effective userid of
 *                  the object calling this function is used to find which
 *                  domain is setting the bits.
 * Arguments:       group: An integer 0-4
 *                  bit:   An integer 0-19
 * Returns:         1 if the bit was successfully set, 0 otherwise.
 */
public int
set_bit(int group, int bit)
{
    int index, num;
    string euid;

    if (group < 0 || group > 4 || bit < 0 || bit > 19)
	return 0;

    euid = geteuid(previous_object());
	
    num = (int)SECURITY->query_domain_number(euid); /* If euid == domain */
    if (num < 0)
	num = SECURITY->query_domain_number(SECURITY->query_wiz_dom(euid));
    if (num < 0)
	return 0;

    num = num * 5 + group;
    index = member_array(num, bit_wizlist);
    if (index < 0)
    {
	bit_wizlist += ({ num });
	bit_bitlist += ({ 0 });
	index = sizeof(bit_wizlist) - 1;
    }
    bit_bitlist[index] |= (1 << bit);
    return 1;
}

/*
 * Function name:   clear_bit
 * Description:     Clear a given bit in a given group. The effective userid
 *                  of the object calling this function is used to find which
 *                  domain is clearing the bits.
 * Arguments:       group: An integer 0-4
 *                  bit:   An integer 0-19
 * Returns:         1 if the bit was successfully cleared, 0 otherwise.
 */
public int
clear_bit(int group, int bit)
{
    int index, num;
    string euid;

    if (group < 0 || group > 4 || bit < 0 || bit > 19)
	return 0;
    euid = geteuid(previous_object());
	
    num = (int)SECURITY->query_domain_number(euid); /* If euid == domain */
    if (num < 0)
	num = SECURITY->query_domain_number(SECURITY->query_wiz_dom(euid));
    if (num < 0)
	return 0;

    num = num * 5 + group;
    index = member_array(num, bit_wizlist);
    if (index < 0)
	return 1;

    bit_bitlist[index] &= (0xFFFFFFFF - (1 << bit));
    return 1;
}

/*
 * Function name:   test_bit
 * Description:     Test a given bit in a given group for a given domain. 
 * Arguments:       dom:   Domain which bits are to be tested
 *                  group: An integer 0-4
 *                  bit:   An integer 0-19
 * Returns:         1 if the bit was set, 0 if unset or failed to test.
 */
public int
test_bit(string dom, int group, int bit)
{
    int index, num;

    if (group < 0 || group > 4 || bit < 0 || bit > 19)
	return 0;
    num = (int)SECURITY->query_domain_number(dom);
    if (num < 0)
	return 0;
    num = num * 5 + group;
    index = member_array(num, bit_wizlist);
    if (index < 0)
	return 0;

    if(bit_bitlist[index] & (1 << bit))
	return 1;
    else
	return 0;
}


/*
 * Function name:   num_bit
 * Description:     Do nothing.
 */
public int
num_bit() {}


/*
 * Function name:   pack_bits
 * Description:     Stuff the used bits into an array to save memory.
 *                  This MUST be nomask for security reasons
 */
public nomask void
pack_bits()
{
    int i;

    bit_savelist = ({});
    for (i = 0; i < sizeof(bit_wizlist); i++)
	bit_savelist += ({ bit_wizlist[i] | (bit_bitlist[i] << 12) });
}

/*
 * Function name:   unpack_bits
 * Description:     Get the stuffed bits out of their array and put them
 *                  in the correct arrays.
 */
public nomask void
unpack_bits()
{
    if (!sizeof(bit_savelist))
    {
	bit_savelist = ({});
	bit_wizlist = ({});
	bit_bitlist = ({});
    }
    else
    {
	bit_wizlist = map(bit_savelist, "extract_name", this_object());
	bit_bitlist = map(bit_savelist, "extract_bit", this_object());
    }
}

/*
 * Function name:   extract_name
 * Description:     Called by unpack_bits.
 */
static int
extract_name(int comp)
{
    return comp & 0xFFF;
}

/*
 * Function name:   extract_bit
 * Description:     Called by unpack_bits.
 */
static int
extract_bit(int comp)
{
    return comp >> 12;
}

/*
 * Function name:   set_bit_array
 * Description:     Try to set the bit array. Only the ghost_player-file is
 *                  allowed to call this function.
 */
public void
set_bit_array(string *arr)
{
    if (file_name(previous_object()) != LOGIN_NEW_PLAYER)
	return;
    bit_savelist = arr;
    unpack_bits();
}

/*
 * Function name: query_bit_array
 * Description:   Gives back the global bit array of a player
 * Returns:       An array with packed bitstrings.
 */
public string *
query_bit_array()
{
    pack_bits();
    return bit_savelist + ({ });
}

/*
 * Function name:
 * Description:     Obvious (Will be documented later, use man -s)
 */
public mapping
query_aliases()
{
    if (mappingp(m_alias_list))
	return m_alias_list;
    else
	return ([ ]);
}

/*
 * Function name:
 * Description:     Obvious (Will be documented later, use man -s)
 */
public void
set_aliases(mapping alist)
{
    m_alias_list = alist;
}

/*
 * Function name:	add_second
 * Description:		Add a second to the player list.
 */
public nomask int
add_second(string second)
{
    if (!strlen(second))
	return 0;

    if (!sizeof(seconds))
	seconds = ({});

    second = lower_case(second);
    
    if (SECURITY->query_wiz_level(second))
	return 0;

    if (file_size("/players/" + second[0..0] + "/" + second + ".o") < 0)
	return 0;

    if (member_array(second, seconds) < 0)
	seconds += ({ second });

    return 1;
}

/*
 * Function name:	remove_second
 * Description:		Remove a second from the player list.
 */
public nomask int
remove_second(string second)
{
    if (!strlen(second))
	return 0;

    if (!sizeof(seconds))
	return 1;

    second = lower_case(second);
    
    if (member_array(second, seconds) < 0)
	return 1;

    seconds -= ({ second });

    return 1;
}


/*
 * Function name:	query_seconds
 * Description:		Return the seconds array
 */
public nomask string *
query_seconds()
{
    if (!sizeof(seconds))
	seconds = ({});

    if (SECURE_ARRAYS)
	return seconds + ({ });
    else
	return seconds;
}


