/*
 * A standard object that adds resistance to its carrier
 */
 
inherit "/std/object";
#include "/sys/ss_types.h"
#include "/sys/macros.h"
#include "/sys/stdproperties.h"

mixed	res_type;	/* The type of resistance */
int	time,		/* How long time it will be in effect, if 0 always. */
	strength;	/* How strong it will be */


/*
 * Function name: create_resistance
 * Description:   Create the resistance
 */
void
create_resistance()
{
    set_name("Resistance_Object");
    add_prop(OBJ_M_NO_DROP, 1);
    add_prop(OBJ_I_WEIGHT, 0);
    add_prop(OBJ_I_VOLUME, 0);
    add_prop(OBJ_I_VALUE, 0);
    set_no_show();
}

/*
 * Function name: create_object
 * Description:   The standard create
 */
nomask void
create_object()
{
    strength = 5;
    res_type = MAGIC_I_RES_MAGIC;
    time = 0;
    create_resistance();
}

/*
 * Function name: init
 * Description:   Called when this object 'gets in touch' with another object
 */
void
init()
{
    ::init();

    add_prop(res_type, ({ strength, 1 }) );
    if (time)
	call_out("stop_res", time);
}

/*
 * Function name: enter_env
 * Description:   Called each time this object enters another environment
 * Arguments:     to   - The object this object enters
 *		  from - The object this object leaves
 */
void
enter_env(object to, object from)
{
    if (living(to))
	to->add_prop(res_type, to->query_prop(res_type) + 1);

    ::enter_env(to, from);
}

/*
 * Function name: leave_env
 * Description:   Called when this object is moved from another object
 * Arguments:     from - The object this object is moved from
 *		  to   - The object to which this object is being moved
 */
void
leave_env(object from, object to)
{
    if (living(from))
      	from->add_prop(res_type, from->query_prop(res_type) - 1);

    ::leave_env(from, to);
}

/*
 * Function name: stop_res
 * Description:   If the resistance is time dependent this function will
 *		  called when it's time to remove this resistance
 */
void
stop_res()
{
    object ob;

    if ((ob = environment(this_object())) && living(ob))
	tell_object(ob, "You feel less resistant.\n");
    remove_object();
}

/*
 * Function name: set_res_type
 * Description:   Set the type of resistance of this object
 */
void set_res_type(string str) { res_type = str; }

/*
 * Function name: query_res_type
 * Description:   Returns what type of resistance this object helps
 */
string query_res_type() { return res_type; }

/*
 * Function name: set_strength
 * Description:   Set how strong this resistance is
 */
void set_strength(int i) { strength = i; }

/*
 * Function name: query_strength
 * Description:   Query how strong this resistance is
 */
int query_strength() { return strength; }

/*
 * Function name: set_time
 * Description:   How long this will hold
 */
void set_time(int i) { time = i; }

/*
 * Function name: query_time
 * Description:   How long will we be resistive
 */
int query_time() { return time; }

/*
 * Function name:
 * Description:
 * Arguments:
 * Returns:
 */
