/* 

   /std/room/move.c

   Room with handling of terrain exits.
   ----------------------------------------------------------------------
   Defined functions and variables:


   ----------------------------------------------------------------------
*/

static  string	  room_dircmd;   /* Command given after the triggering verb */

/*
* Function name:   load_room
* Description:     Finds a room object for a given array index
* Arguments:       index of the file name entry in the room_exits array
* Return value:    object pointer to the room corresponding to the argument
*                  or 0 if not found
*/
object
load_room(int i)
{
    mixed droom;
    string err;
    object ob;
    
    droom = check_call(room_exits[i]);
    
    if (objectp(droom))
	return droom;

    /* Handle linkrooms that get destructed, bad wizard... baa-aad wizard...
    */
    if (!stringp(droom))
    {
	remove_exit(room_exits[i + 1]);
	this_player()->move_living("X", query_link_master());
	return 0;
    }
    
    ob = find_object(droom);
    if (ob)
	return ob;
    
    if (err = LOAD_ERR(droom))
    {
	SECURITY->log_loaderr(droom,
			      environment(this_object()), 
			      room_exits[i + 1],
			      this_object(), err);
	write("err in load:" + err + " <" + droom + ">\n");
	return 0;
    }
    return find_object(droom);
}

/*
 * Function name: query_dircmd
 * Description:   Gives the rest of the command given after move verb.
 *                This can be used in blocking functions (third arg add_exit)
 * Returns:       The movecommand as given. 
 */
public string
query_dircmd() { return room_dircmd; }

/*
 * Function name: set_dircmd
 * Description:   Set the rest of the command given after move verb
 *		  This is mainly to be used from own move_living() commands
 *		  where you want a team to be able to follow their leader.
 * Arguments:     rest - The rest of the move command
 */
public void
set_dircmd(string rest)
{
    room_dircmd = rest;
}

/*
 * Function name: unq_move
 * Description:   Routine called when a direction command id given. Moves
 *                the player. 'value by function call' are applied to the
 *		  first and third parameter as given to add_exit(). The third
 *		  is a 'delay value' as described in add_exit(). Observe that
 *		  if you have a direction command that uses its trailing
 *		  test as in 'enter portal' and fails if the second word
 *		  is not 'portal', your block function should return 2 as you
 *		  want the rest of the dir commands to be tested.
 * Arguments:	  str: String given after dir command
 * Returns:       1/0
 */
public int
unq_move(string str)   /* Must have a unique name */
{
    int i, wd, carry, own, own_weight, tired, tmp;
    object d;

    room_dircmd = str;
    for (i = 0; i < sizeof(room_exits); i += 3)
	if (query_verb() == room_exits[i + 1])
	{
	    if ((wd = check_call(room_exits[i + 2])) > 0)
	    {
		if (wd > 1)
		    continue;
		else
		    return 1;
	    }
	    if (!(d = load_room(i)))
		return 1;

    /* Young players younger than 2 hours or people carrying less than
     * 20 % of their capability doesn't get tired from walking around in
     * the world... */

	    if (this_player()->query_age() > 3600 &&
			(tmp = this_player()->query_encumberance_weight()) > 20)

		if (this_player()->query_fatigue() >=
			(tired = check_call(tired_exits[i / 3])))
		    this_player()->add_fatigue(tmp > 80 ? -tired * 2 : -tired);
		else {
		    notify_fail("You are too tired to walk in that direction.\n");
		    return 0;
		}

	    if ((wd == 0) || (!transport_to(room_exits[i + 1], d, -wd)))
		this_player()->move_living(room_exits[i + 1], d);
	    return 1;
	}
    return 0; /* We get here if a 'block' function stopped a move */
}


/*
 * Function name:
 * Description:     Obvious (Will be documented later, use man -s)
 */
public int
unq_no_move()
{
    notify_fail("There is no obvious exit " + query_verb() + ".\n");
    return 0;
}

