/*
 * scroll.c      
 *
 * A standard object that can be both read, and mread :)
 *
 * set_file(string file) - Will connect a file to the read command
 * 
 * set_autoload()        - Will make this object autoload
 *
 */
#pragma save_binary

inherit "/std/object";

#include <macros.h>
#include <stdproperties.h>
#include <config.h>
#include <cmdparse.h>

#define MORE_OBJ "/std/board/board_more"

string scroll_file;
int auto_load;

/*
 * Function name: init
 * Description:   Initialize commandverbs
 */
public void
init()
{
    ::init();

    add_action("read_scroll", "mread");
    add_action("read_scroll", "read");
}

/*
 * Function name: create_scroll
 * description:	  Create the scroll
 */
void
create_scroll()
{
    set_name("scroll");
    set_long("An empty scroll, no use to either read or mread it.\n");
}

/*
 * Function name: create_object
 * Description:   Create the object
 */
nomask void
create_object()
{
    add_prop(OBJ_I_WEIGHT, 10); /* 10 g */
    add_prop(OBJ_I_VOLUME, 10); /* 10 ml */
    add_prop(OBJ_I_VALUE, 0); /* 0 copper */

    create_scroll();
}

/*
 * Function name: reset_object
 * Description:   Reset
 */
nomask void
reset_object() { this_object()->reset_scroll(); }

/*
 * Function name: set_file
 * Description:   Set the file to list when player reads this
 */
public void
set_file(string file)
{
    scroll_file = file;
}

/*
 * Function name: query_file
 * Description:   Query what file is connected to read command
 */
string query_file() { return scroll_file; }

/*
 * Function name: set_autoload
 * Description:   Get the file to autoload
 */
void set_autoload() { auto_load = 1; }

/*
 * Function name: init_arg
 * Description:   Set the parameters of the scroll when auto-loading
 */
public void
init_arg(string arg)
{
    string tmp;

    if (!arg || !strlen(arg))
	return;

    sscanf(arg, "%s##%s", scroll_file, tmp);

    if (tmp == "0")
	add_prop(OBJ_M_NO_DROP, 0);
    else if (tmp == "1")
	add_prop(OBJ_M_NO_DROP, 1);
    else
	add_prop(OBJ_M_NO_DROP, tmp);
   
    auto_load = 1;
}

/*
 * Function name: query_auto_load
 * Description:   Autoload this object.
 */
public void
query_auto_load()
{
    if (auto_load)
        return MASTER_OB(this_object()) + ":" + scroll_file + "##" +
		query_prop(OBJ_M_NO_DROP);
}

/*
 * Function name: read_scroll
 * Description:   Read the text
 */
static int
read_scroll(string str)
{
    string vb;
    object *a;

    if (this_player()->query_prop(TEMP_STDREAD_CHECKED))
        return 0;

    vb = query_verb();
    notify_fail(capitalize(vb) + " what?\n");
    if (!str)
	return 0;

    a = CMDPARSE_ONE_ITEM(str, "read_access", "read_access");
    if (sizeof(a) > 0)
    {
        a[0]->read_it(vb);
        return 1;
    }

    call_out("remove_tmp", 1, this_player());
    this_player()->add_prop(TEMP_STDREAD_CHECKED, 1);
    return 0;
}

int
read_access(object ob)
{
    if ((environment(ob) == this_player() ||
		environment(ob) == environment(this_player())) &&
		(function_exists("create_object", ob) == "/std/scroll"))
	return 1;
    else
    	return 0;
}

/*
 * Function name: read_it
 * Description:   Perform the actual read
 */
void
read_it(string str)
{
    seteuid(getuid(this_object()));
    say(QCTNAME(this_player()) + " reads the " + QSHORT(this_object()) + ".\n");
    if (!scroll_file)
        write("There was nothing to read in that book.\n");
    else if (str == "read")
        cat(scroll_file);
    else
        clone_object(MORE_OBJ)->more(scroll_file, 2);
}

void
remove_tmp(object ob)
{
    ob->remove_prop(TEMP_STDREAD_CHECKED);
}

