/*
   /std/shadow.c

   This is a generic shadow object.

*/
#pragma save_binary

#include "/sys/macros.h"

static object	shadow_who;	/* Which object are we shadowing */


/*
 * Prototypes
 */
public void remove_shadow();

/*
 * Function name: autoload_shadow
 * Description:   Called by the autoloading routine in /std/player_sec
 *                to ensure autoloading of the shadow
 */
public void
autoload_shadow(mixed arg)
{
    if (!shadow_who)
    {
	shadow_who = previous_object();
	shadow(shadow_who, 1);
    }
}

static int
shadow_double(string fname, object who)
{
    object ob;

    ob = who;

    while (ob = shadow(ob, 0))
	if (fname == MASTER_OB(ob))
	    return 1;
}    

/*
 * Fucntion: remove_object
 * Description: Ensure that we remove the shadow object too
 */
public void 
remove_object() 
{ 
    if (shadow_who) 
        shadow_who->remove_object(); 
    remove_shadow();
}

/*
 * Function: remove_shadow
 * Description: removes the shadow without removing the shadowed object.
 */
public void 
remove_shadow()
{
    destruct();
}

/*
 * Function name: shadow_me
 * Description:   Called from object to make us shadow it.
 * Returns:       True if shadowed
 */
public varargs int
shadow_me(mixed to_shadow)
{
    if (stringp(to_shadow))
	to_shadow = find_player(to_shadow);
    
    if (!to_shadow)
	to_shadow = previous_object();

    if (!shadow_who && !shadow_double(MASTER,to_shadow))
    {
	shadow_who = to_shadow;
	if (shadow(shadow_who, 1))
	    return 1;
    }
    return 0;
}

/*
 * Function name: query_shadow_who
 * Description:   Returns the object that we are shadowing
 *                
 */
public object
query_shadow_who() { return shadow_who; }
