/*
 * A torch, some code stolen from Nosferatu, the rest made by Nick
 * May 30, 1992
 */

inherit "/std/object";
 
#include <stdproperties.h> 	/* All prop definitions */
#include <cmdparse.h>		/* CMDPARSE_ONE_ITEM macro */
#include <macros.h>		/* QCTNAME and QTNAME macro */
#include <composite.h>		/* COMPOSITE_DEAD and so on */
 
/*
 * Prototype
 */
void set_time(int time);
void set_strength(int strength);
void set_value(int value);

int torch_value;	  	/* The max value of the torch. */
int light_strength;		/* How strongly the 'torch' will shine */
int time_left;			/* How much time is left? */
int max_time;			/* How much is the max time (start time) */
object *gFail;

/*
 * Function name: create_torch
 * Description:   The standard create. This has some standard settings on
 *		  long and short descriptions, but don't be afraid to change
 *		  them.
 */
void
create_torch()
{
    set_name("torch");
    set_pname("torches");
}

/*
 * Function name: create_object
 * Description:   The standard create routine.
 */
nomask void
create_object()
{
    add_prop(OBJ_I_LIGHT, 0);
    add_prop(OBJ_I_WEIGHT, 700);
    add_prop(OBJ_I_VOLUME, 1000);
    add_prop(OBJ_I_VALUE, "@@torch_value"); /* Value depends on usefulness */

    set_time(300);
    set_strength(1);
    set_value(40);

    create_torch();
}
 
/*
 * Function name: reset_object
 * Description:   Reset the object
 */
nomask void reset_object() { this_object()->reset_torch(); }

/*
 * Function name: torch_value
 * Description:   A VBFC gets here when someone wants to know the value of
 *		  the value of this object, default setting
 * Returns:	  The value
 */
int
torch_value()
{
    int v;

    v = find_call_out("burned_out");
    if (v == -1)
	v = time_left;

    return (v * torch_value) / max_time + 5;
}
 
/*
 * Function name: short, pshort, long
 * Description:   We change the short, plural short and long description of
 *		  the torch if it's lit, default settings.
 * Returns:	  The description.
 */
public varargs string
short(object for_obj)
{
    return ::short(for_obj) + ((find_call_out("burned_out") != -1) ?
	" (lit)" : "");
}

public varargs string
plural_short(object for_obj)
{
    if (::plural_short(for_obj))
	return ::plural_short(for_obj) + ((find_call_out("burned_out") != -1) ?
	    " (lit)" : "");

    return 0;
}

public varargs string
long(object for_obj)
{
    return ::long(for_obj) + ((find_call_out("burned_out") != -1) ?
	" It is lit.\n" : "\n");
}

/*
 * Function name: init
 * Description:   Here we add some commands to the player.
 */
void
init()
{
    ::init();

    add_action("do_light", "light");  
    add_action("do_extinguish", "extinguish");
}

/*
 * Function name: set_time
 * Description:   Set how long time the torch can burn.
 */
void set_time(int time) { max_time = time; time_left = time; }

/*
 * Function name: query_time
 * Description:	  Query how long time the torch can burn
 */
int query_time() { return time_left; }

/*
 * Function name: set_strength
 * Description:   Set the light strength of this 'torch'
 */
void set_strength(int strength) { light_strength = strength; }

/*
 * Function name: query_strength
 * Description:   Query how strongly the torch will shine
 */
int query_strength() { return light_strength; }

/*
 * Function name: set_value
 * Description:   Set the max value of the torch
 */
void set_value(int value) { torch_value = value; }

/*
 * Function name: do_light
 * Description:   The player tries to light something.
 * Arguments:	  str - the string describing what he  wants to light.
 * Returns:	  1/0
 */
int
do_light(string str)
{
    object *a;
    string str2, vb;

    if (this_player()->query_prop(TEMP_STDTORCH_CHECKED))
        return 0;

    vb = query_verb();
    notify_fail(capitalize(vb) + " what?\n");
    if (!str)
	return 0;

    seteuid(getuid(this_object()));

    gFail = ({});
    a = CMDPARSE_ONE_ITEM(str, "light_one_thing", "light_access");
    if (sizeof(a) > 0)
    {
	str2 = COMPOSITE_DEAD(a);
        say(QCTNAME(this_player()) + " lights " + QCOMPDEAD + ".\n");
	write("You light " + str2 + ".\n");
        return 1;
    }

    call_out("remove_tmp", 1, this_player());
    this_player()->add_prop(TEMP_STDTORCH_CHECKED, 1);
    if (sizeof(gFail))
	notify_fail("@@light_fail:" + file_name(this_object()));

    return 0;
}

void
remove_tmp(object ob)
{
    ob->remove_prop(TEMP_STDTORCH_CHECKED);
}

string
light_fail()
{
    int i;
    string str;

    str = "";
    for (i = 0; i < sizeof(gFail); i++)
        str += gFail[i]->query_light_fail();

    return str;
}

string
query_light_fail()
{
    if (!time_left)
        return "You try to light " + short() + " but fail... " +
		"It's useless!\n";

    if (find_call_out("burned_out") != -1)
        return capitalize(short()) + " is already lit.\n";

    return "";
}

int
light_access(object ob)
{
    if ((environment(ob) == this_player() ||
	    environment(ob) == environment(this_player())) &&
	    function_exists("create_object", ob) == "/std/torch")
	return 1;

    return 0;
}

int
light_one_thing(object ob)
{
    if (!ob->light_me())
    {
	gFail += ({ ob });
	return 0;
    }
    return 1;
}

int
light_me()
{
    if (!time_left || find_call_out("burned_out") != -1)
	return 0;
    call_out("light_me_after_delay", 1);
    return 1;
}

int
light_me_after_delay()
{
    add_prop(OBJ_I_LIGHT, light_strength);
    add_prop(OBJ_I_HAS_FIRE, 1);
    call_out("burned_out", time_left);
}

/*
 * Function name: do_extinguish
 * Description:   The player tries to extinguish something
 * Arguments:	  str - The string describing what to extinguish
 * Returns:	  1/0
 */
int
do_extinguish(string str)
{
    object *a;
    string str2, vb;

    if (this_player()->query_prop(TEMP_STDTORCH_CHECKED))
        return 0;

    vb = query_verb();
    notify_fail(capitalize(vb) + " what?\n");
    if (!str)
        return 0;

    gFail = ({});
    a = CMDPARSE_ONE_ITEM(str, "extinguish_one_thing", "light_access");
    if (sizeof(a) > 0)
    {
        str2 = COMPOSITE_DEAD(a);
        say(QCTNAME(this_player()) + " extinguishes " + QCOMPDEAD + ".\n");
        write("You extinguish " + str2 + ".\n");
        return 1;
    }

    call_out("remove_tmp", 1, this_player());
    this_player()->add_prop(TEMP_STDTORCH_CHECKED, 1);
    if (sizeof(gFail))
        notify_fail("@@extinguish_fail:" + file_name(this_object()));
    return 0;
}

string
extinguish_fail()
{
    int i;
    string str;

    str = "";
    for (i = 0; i < sizeof(gFail); i++)
	str += gFail[i]->query_extinguish_fail();

    return str;
}

string
query_extinguish_fail()
{
    if (find_call_out("burned_out") == -1)
	return capitalize(short()) + " isn't lit.\n";

    return "";
}

int
extinguish_one_thing(object ob)
{
    if (!ob->extinguish_me())
    {
	gFail += ({ ob });
	return 0;
    }
    return 1;
}

int
extinguish_me()
{
    if (find_call_out("burned_out") == -1)
        return 0;

    time_left = remove_call_out("burned_out");
    add_prop(OBJ_I_LIGHT, 0);
    remove_prop(OBJ_I_HAS_FIRE);
    return 1;
}

/*
 * Function name: burned_out
 * Description:	  If this function is called the torch has burned out.
 */
void
burned_out()
{
    object ob;

    time_left = 0;
    ob = environment();
    if (!ob)
	return;

    if (living(ob))
    {
	tell_object(ob, "The " + short() + " goes out.\n");
	tell_room(environment(ob), "The " + QSHORT(this_object()) + " that " +
		QTNAME(ob) + " is holding goes out.\n", ob);
	add_prop(OBJ_I_LIGHT, 0);
	remove_prop(OBJ_I_HAS_FIRE);
    	return;
    }

    if (ob->query_prop(ROOM_I_IS))
	tell_room(ob, "The " + short() + " goes out.\n");

    add_prop(OBJ_I_LIGHT, 0);
}

/*
 * Function name: query_torch_recover
 * Description:   Return the recover string for changing tourch values
 * Returns:	  part of recover string
 */
string
query_torch_recover()
{
    int tmp;

    tmp = find_call_out("burned_out");
    if (tmp == -1)
	tmp = time_left;

    return "#t_t#" + tmp + "#t_l#" + query_prop(OBJ_I_LIGHT) + "#";
}

/*
 * Function name: init_torch_recover
 * Description:   Initialize the torch variables at recover.
 * Arguments:     arg - The recover string as recieved from
 *			query_torch_recover()
 */
void
init_torch_recover(string arg)
{
    string foobar;
    int tmp;

    sscanf(arg, "%s#t_t#%d#%s", foobar, time_left, foobar);
    sscanf(arg, "%s#t_l#%d#%s", foobar, tmp, foobar);
    if (tmp > 0)
    {
	add_prop(OBJ_I_LIGHT, tmp);
	add_prop(OBJ_I_HAS_FIRE, 1);
	call_out("burned_out", time_left);
    }
}

