/* /sys/formulas.h

  This file holds all system game formulas, examples:

      - Combat
      - Healing

*/
#ifndef WA_TYPES_DEF
#include "/sys/wa_types.h"
#endif

#ifndef SS_TYPES_DEF
#include "/sys/ss_types.h"
#endif

#ifndef F_FORMULAS
#include "/config/sys/formulas2.h"
#endif

#ifndef F_FORMULAS
#define F_FORMULAS

/*
 * Armour
 *
 * Note that these are only valid for humanoid armours.
 */
#define ARMOUR_FILE 				"/std/armour"
#define F_VALUE_ARMOUR(ac) 			(20 + (ac) * ((ac) + 21) / 3)
#define F_ARMOUR_DEFAULT_AT 			A_TORSO
#define F_LEGAL_AM(list) 			(sizeof(list) == 3)
#define F_ARMOUR_CLASS_PROC(proc_of_max) 	(proc_of_max)
#define F_LEGAL_ARMOUR_REPAIR(rep, cond)	((rep) <= (cond)  && \
							(rep) < (cond) / 2 + 3)
#define F_ARMOUR_VALUE_REDUCE(m_cond)		(100 - (m_cond) * 5)
#define F_ARMOUR_BREAK(m_cond, likely)		((m_cond)>(20 - (likely) / 2 ) \
		|| (m_cond) > random(40 - (likely)))

#define F_ARMOUR_CONDITION_WORSE(hits, ac, lik)	((hits) > random(1000) + \
		4 * (40 - (lik)))

#define F_AT_WEIGHT_FACTOR(type) \
   ((type == A_SHIELD) ? 20 : ((type & A_BODY) ? 40 : 0) + \
    ((type & A_LEGS) ? 30 : 0) + \
    ((type & A_HEAD) ? 10 : 0) + \
    ((type & A_R_ARM) ? 10 : 0) + \
    ((type & A_L_ARM) ? 10 : 0) + \
    ((type & A_ROBE) ? 20 : 0))

#define F_WEIGHT_DEFAULT_ARMOUR(ac, at) \
    (F_AT_WEIGHT_FACTOR(at) * (428 * (((ac) > 1) ? (ac) - 1 : 1) + \
     (((ac) > 14) ? 10000 : 0)) / 100)

#define F_WEIGHT_FAULT_ARMOUR(w, ac, at)\
    (F_WEIGHT_DEFAULT_ARMOUR(ac, at) * 800 / 1000 > (w))

/* 
 * Weapon       (Observe that F_VALUE_WEAPON takes two args)
 *
 * Note also that these are only valid for humanoid weapons.
 */
#define WEAPON_FILE 				"/std/weapon"
#define F_VALUE_WEAPON(wch, wcp) 		(20 + (wch * wcp))
#define F_WEAPON_DEFAULT_WT 			W_FIRST
#define F_WEAPON_DEFAULT_DT 			W_IMPALE
#define F_WEAPON_DEFAULT_HANDS			W_ANYH
#define F_WEAPON_CLASS_PROC(proc_of_max)        (proc_of_max)
#define F_WEAPON_VALUE_REDUCE(du, co)		(100 - (du)* 3 - (co)* 6)
#define F_LEGAL_WEAPON_REPAIR_DULL(rep, dull)	((rep) <= (dull) && \
							(rep)< 2*(dull)/ 3 + 3)
#define F_LEGAL_WEAPON_REPAIR_CORR(rep, corr)	((rep) <= (corr) && \
							(rep) < (corr) / 2 + 1)
#define F_WEAPON_BREAK(dull, corr, likely)	((dull) > (20 - (likely)) || \
		(corr) > (5 -(likely)/ 4) || (dull) > random(40 -(likely)) || \
		(corr) > random(10 - (likely) / 4))

#define F_WEAPON_CONDITION_DULL(hits, pen, lik)	((hits) > random(1000) + \
		10 * (30 - (lik)))

#define F_WEIGHT_FAULT_WEAPON(w, wp, wt) \
		((w) < 800 * F_WEIGHT_DEFAULT_WEAPON(wp, wt) / 1000)

#define F_WEIGHT_DEFAULT_WEAPON(wp, wt)		(1000 * ((wp) / 4))

#define F_LEGAL_DT(type) ((type) &        \
			  (W_IMPALE |   \
			   W_SLASH |    \
			   W_BLUDGEON))

#define F_LEGAL_HANDS(which) ((which) == W_ANYH ||  \
			      (which) == W_LEFT ||  \
			      (which) == W_RIGHT || \
			      (which) == W_BOTH)

#define F_LEGAL_WCHIT(wc, type)	      (F_LEGAL_TYPE(type)       && \
				       ((wc) <= W_MAX_HIT[type]))
#define F_LEGAL_WCPEN(wc, type)	      (F_LEGAL_TYPE(type)       && \
				       ((wc) <= W_MAX_PEN[type]))


/* 
 * Living
 */
#define F_KILL_NEUTRAL_ALIGNMENT        10
#define F_KILL_ADJUST_ALIGN(killer_al,victim_al)                             \
                                 ((killer_al) * (victim_al) < 0              \
			             ? -(((victim_al) * 4000) /              \
			                ((killer_al) * (killer_al) + 20000)) \
			             : -((victim_al) / 5))

#define F_DIE_REDUCE_XP(xp) 		(xp / 3)

#define F_KILL_GIVE_EXP(v_av_stat)	((v_av_stat) * (v_av_stat) * 4)
#define F_DIE_START_HP(max_hp) 		((max_hp) / 10)
#define F_MAX_SCAR			10
#define F_SCAR_DESCS ({ 					       \
			  "left leg", "right leg", "nose", "left arm", \
			  "right arm", "left hand", "right hand",      \
			  "forehead", "left cheek", "right cheek"       \
		     })
/*
 * The following constants define how quickly a living heals.
 */
#define F_INTERVAL_BETWEEN_HEALING	10  /*(in heartbeats)*/

/* Amount to heal per interval for various stats */
#define F_HEADACHE_RATE                 1
#define F_SOBER_RATE                    1
#define F_MANA_HEAL_RATE                1
#define F_UNSTUFF_RATE                  5
#define F_UNSOAK_RATE                   16
#define F_HEAL_FORMULA(con, intox) ((con) / 3 + (intox) / 4 + 1)
#define F_FATIGUE_FORMULA(stuffed, max) (10 + (stuffed) * 30 / (max))

/*
 * How long can a temporary stat addition be? (In heartbeats)
 */
#define F_TMP_STAT_MAX_TIME 30

#define F_MAX_REMEMBERED(stat)	(stat / 2 + 10)

/*
 * These macros convert from seconds to heart beats and back.
 */
#define F_SECONDS_PER_BEAT 		2
#define F_NUM_BEATS(seconds) 		((seconds) / F_SECONDS_PER_BEAT)

/* 
 * Death
 */
#define F_DEATH_MESSAGE      "\nYou die.\n" + 				     \
			     "You have a strange feeling.\n" +               \
                             "You can see your own dead body from above.\n\n"
#define F_GHOST_MSGIN 	     "drifts around."
#define F_GHOST_MSGOUT 	     "blows"
#define F_NAME_OF_GHOST	     "some mist"



/*
 * Combat 
 */

#define F_MAX_HP(con)	(((con) + 20) * ((con) + 20) / 5 - 80)

#define F_PENMOD(pen, skill) ((((pen) > (skill) ? (skill) : (pen)) + 50) * \
	(((skill) > (pen) ? (pen) + ((skill) - (pen)) / 2 : (skill)) + 50) / \
	30 - 80)

#define F_AC_MOD(ac) (((ac) + 50) * ((ac) + 50) / 50 - 50)

#define F_DAMAGE(pen, dam) ((pen) - (dam))

#define F_DARE_ATTACK(ob1, ob2)	(ob2->query_average_stat() <= \
				ob1->query_stat(SS_DIS) * 2)

#define F_UNARMED_HIT(skill, dex)    ((skill) / 7 + (dex) / 20)
#define F_UNARMED_PEN(skill, str)    ((skill) / 10 + (str) / 20)

/*
 * Healing alco
 */
#define F_VALUE_ALCO(alco)		(10 + ((alco) * (alco) / 10))

/*
 * Magic 
 */
#define F_VALUE_MAGICOB_HEAL(hp)	(5 * (hp) + (hp) * (hp) / 4)
#define F_VALUE_MAGIC_COMP(hp)		((hp) * 20)

/*
 * Some general values
 *
 */
/*
 * wg in grams (weight it can support), l in centimeters (length of rope)
 */
#define F_VALUE_ROPE(wg, l)		(((wg) / 10000) * ((l) / 100))

/*
 * Some string defines that are only used indirectly.
 *
 * All these kinds of string constants are defined in a /sys/file_desc.h
 * Where 'file' is the original filename. The constants below are
 * referenced by default from those files for backwards compatibilty
 * reasons.
 */
#define F_ALIVE_MSGIN 			"arrives."
#define F_ALIVE_MSGOUT 			"leaves"
#define F_ALIVE_TELEIN 			"arrives in a puff of smoke."
#define F_ALIVE_TELEOUT 		"disappears in a puff of smoke."

#endif
