/*
  /sys/global/language.c

  This file holds some standard natural language functions.

*/
#pragma save_binary

#include <stdproperties.h>

string *nums, *numt, *numnt, *numo;

create()
{
    nums = ({ "one", "two", "three", "four", "five", "six", "seven",
              "eight", "nine", "ten","eleven","twelve","thirteen",
              "fourteen", "fifteen", "sixteen", "seventeen", "eighteen",
              "nineteen"});
    numo = ({ "first", "second", "third", "fourth", "fifth", "sixth",
              "seventh", "eighth", "ninth", "tenth","eleventh","twelfth",
              "thirteenth", "fourteenth", "fifteenth", "sixteenth",
              "seventeenth", "eighteenth", "nineteenth"});
    numt = ({ "twenty", "thirty", "forty", "fifty", "sixty", "seventy",
              "eighty", "ninety"});
    numnt = ({ "twent", "thirt", "fort", "fift", "sixt", "sevent", "eight",
	       "ninet" });
}

#define CFUN
#ifdef CFUN
string
article(string str) = "article";
#else
string
article(string str)
{
    string a, tmp;

    if (!str)
	return 0;

    tmp = str + " ";
    tmp = lower_case(tmp);
    if (sscanf(tmp, "the %s", a) == 1) return "";
    if (sscanf(tmp, "a%s", a) == 1) return "an";
    if (sscanf(tmp, "e%s", a) == 1) return "an";
    if (sscanf(tmp, "i%s", a) == 1) return "an";
    if (sscanf(tmp, "o%s", a) == 1) return "an";
    if (sscanf(tmp, "u%s", a) == 1) return "an";
    return "a";
}
#endif

string
add_article(string str) 
{
    string s;

    s = article(str);
    return strlen(s) ? (s + " " + str) : str;
}

/* This routine returns a number in text form
*/
string
word_number(int num)
{
    int tmp;
    
    if (num < 1) return "no";
    if (num < 20) return nums[num-1];
    if (num > 99)
    {
	if (num > 999)
	{
	    if (num > 999999)
		return "many";
	    tmp = num % 1000;
	    return word_number(num / 1000) + " thousand" +
		(tmp ? " " + word_number(tmp) : "");
	}
	tmp = num % 100;
	return word_number(num / 100) + " hundred" +
	    (tmp ? " and " +  word_number(tmp) : "");
    }
    tmp = num % 10;
    return numt[num / 10 - 2] + (tmp ? "-" + nums[tmp - 1] : "");
}

string
word_ord_number(int num)
{
    int tmp;

    if (num < 1) return "zero";
    if (num < 20) return numo[num-1];
    if (num > 99)
    {
        if (num > 999)
	{
            if (num > 9999)   /* was 999999 */
                return "many";
            if (num == 1000)
                return "thousandth";
            if (!(tmp = num % 1000))
                return word_number(num / 1000) + " thousandth";
            return word_number(num / 1000) + " thousand " +
                word_ord_number(tmp);
        }
        if (num == 100)
            return "hundredth";
   	if (!(tmp = num % 100))
            return word_number(num / 100) + " hundredth";
        return word_number(num / 100) + " hundred " +
	    word_ord_number(tmp);
    }

    if (!(tmp = num % 10))
        return numnt[num / 10 - 2] + "ieth";
    else
        return numt[num / 10 - 2] + " " + numo[tmp - 1];
}

int
number_ord_word(string str)
{
    int i;

    if (!str)
	return 0;

    if ((i = member_array(str, numo)) == -1)
        return 0;
    return i + 1;
}

/* lpc singular to plural converter
*/
#ifdef CFUN
string
plural_word(string str) = "plural_word";
#else
string
plural_word(string str)
{
    string tmp, slask;
    int sl, ch;
    
    if (!str) return 0;

    switch (str)
    {
    case "tooth":
	return "teeth";
    case "foot":
	return "feet";
    case "man":
	return "men";
    case "woman":
	return "women";
    case "child":
	return "children";
    case "sheep":
	return "sheep";
    case "dwarf":
	return "dwarves";
    case "elf":
        return "elves";
    }  
    sl = strlen(str) - 1;
    if (sl < 2)
	return str;
    ch = str[sl];
    tmp = extract(str, 0, sl - 2);
    slask = extract(str, sl - 1, sl - 1);
    switch (ch)
    {
    case 's':
	return tmp + slask + "ses";
    case 'x':
	return tmp + slask + "xes";
    case 'h':
	return tmp + slask + "hes";
    case 'y':
	if (member_array(slask, ({ "a", "e", "o" })) >= 0)
	    return tmp + slask + "ys";
	else
	    return tmp + slask + "ies";
    case 'e':
	if (slask == "f")
	    return tmp + "ves";
    }
    return str + "s";
}
#endif

string
plural_sentence(string str)
{
    int  c;
    string *a;
    
    if (!str)
	return 0;
    
    a = explode(str + " ", " ");
    if ((!a) || (sizeof(a) < 1))
	return 0;
    for (c = 1; c < sizeof(a); c++)
    {
	if (lower_case(a[c]) == "of")
	{
	    a[c - 1] = plural_word(a[c - 1]);
	    return implode(a, " ");
	}
    }
    a[sizeof(a) - 1] = plural_word(a[sizeof(a) - 1]);
    return implode(a, " ");
}

/* Verb in present tence (not ready yet)
*/
string
verb_present(string str)
{
  return str;
}

/* Name in possessive form
*/
string
name_possessive(string str)
{
    if (!strlen(str))
	return 0;

    switch (str)
    {
    case "it": return "its";
    case "It": return "Its";
    case "he": return "his";
    case "He": return "His";
    case "she": return "her";
    case "She": return "Her";
    }
    if (extract(str, strlen(str) - 1) == "s")
	return (str + "'");
    return (str + "'s");
}

/* This routine returns a textform as a number
 */
int
number_word(string str)
{
    string *ex;
    int value, pos;

    if (sscanf(str, "%d", value))
	return value;

    value = 0;

    ex = explode(str, "y");

    if (!sizeof(ex))
        return 0;

    if (sizeof(ex) > 1)
    {
        if ((pos = member_array(ex[0], numnt)) == -1)
	    return 0;
	else
	    value = (pos + 2) * 10;
    }

    if (value)
    {
	if (ex[1][0] == ' ' || ex[1][0] == '-')
	    ex[1] = extract(ex[1], 1, strlen(ex[1]) - 1);
	if ((pos = member_array(ex[1], nums)) == -1)
	    return 0;
	else
	    value += pos + 1;
    }

    if (!value)
	if ((pos = member_array(ex[0], nums)) == -1)
	    {
		if ((pos = member_array(ex[0], numnt)) == -1)
		    return 0;
		else
		    value = (pos + 2) * 10;
	    }
	else
	    value = pos + 1;

    return value;
}

string
singular_form(string str)
{
    string singular, one, two, three;
    int last;

    last = strlen(str);
    one  = extract(str, last - 1, last - 1);
    two  = extract(str, last - 2, last - 2);
    three = extract(str, last - 3, last - 3);

    if (one != "s")
	return str;
    if (two != "e" || three == "c")
        return extract(str, 0, last - 2);
    if (three != "v")
        return extract(str, 0, last - 3);

    return extract(str, 0, last - 4) + "fe";
}

string
lang_a_short(object ob)
{
    if (ob->query_prop(HEAP_I_IS))
	return ob->short();
    return add_article(ob->short());
}

string
lang_the_short(object ob)
{
    if (ob->query_prop(HEAP_I_IS))
	return ob->short();
    return "the " + ob->short();
}

